@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,18 +36,19 @@
 ////////////////////////////////////////////////////////////////////////
 
 #include "ai.h"
+#include "game.h"
 #include "misc.h"
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : AI::AI
 // DESCRIPTION: Initialize a computer controlled player.
-// PARAMETERS : -
+// PARAMETERS : mission - current mission object
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-AI::AI( Player *player, Map *map ) {
-  this->player = player;
-  this->map = map;
+AI::AI( Mission &mission ) : mission(mission) {
+  player = &mission.GetPlayer();
+  map = &mission.GetMap();
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -90,7 +91,7 @@ void AI::Play( void ) {
 void AI::IdentifyObjectives( void ) {
   AIObj *obj;
 
-  for ( Building *b = static_cast<Building *>(Gam->BuildingsList()->Head());
+  for ( Building *b = static_cast<Building *>(mission.GetShops().Head());
         b; b = static_cast<Building *>(b->Next()) ) {
     unsigned char pri = AI_PRI_LOW;
     obj = new AIObj;
@@ -102,7 +103,7 @@ void AI::IdentifyObjectives( void ) {
     else obj->type = AI_OBJ_CONQUER;
 
     // detect enemy unit presence in the vicinity
-    UnitPresence( Gam->NextPlayer(), obj );
+    UnitPresence( &mission.GetOtherPlayer(*player), obj );
 
     if ( b->IsFactory() ) pri += 10;
     else if ( b->IsWorkshop() ) pri += 2;
@@ -111,7 +112,8 @@ void AI::IdentifyObjectives( void ) {
     if ( obj->type == AI_OBJ_DEFEND ) {
       // check enemy presence; this also affects priority
       // locate closest enemy unit able to conquer buildings or transport
-      Unit *u = ClosestUnit( Gam->NextPlayer(), obj->pos, U_CONQUER|U_TRANSPORT, U_DESTROYED );
+      Unit *u = ClosestUnit( &mission.GetOtherPlayer(*player), obj->pos,
+                             U_CONQUER|U_TRANSPORT, U_DESTROYED );
       if ( u && (Distance( u->Position(), obj->pos ) <= AI_ATTENTION_RADIUS) )
         pri = AI_PRI_CRITICAL;
       else pri = MIN( obj->needed_ground + obj->needed_air + obj->needed_ship, AI_PRI_MAX );
@@ -234,7 +236,7 @@ void AI::AssignObjectives( void ) {
   }
 
   // lastly, assign all unassigned units to a task
-  for ( u = static_cast<Unit *>( Gam->UnitsList()->Head() );
+  for ( u = static_cast<Unit *>( mission.GetUnits().Head() );
         u; u = static_cast<Unit *>(u->Next()) ) {
     if ( (u->Owner() == player) && !u->IsBusy() ) {
       AIObj *best = NULL;
@@ -278,7 +280,7 @@ void AI::BuildReinforcements( void ) const {
                  p2_aair = 1, p2_aground = 1, p2_aship = 1,  // prevent div by 0
                  amul, gmul, smul;
 
-  for ( Unit *u = static_cast<Unit *>(Gam->UnitsList()->Head()); u;
+  for ( Unit *u = static_cast<Unit *>(mission.GetUnits().Head()); u;
         u = static_cast<Unit *>(u->Next()) ) {
     if ( u->Owner() == player ) {
       unsigned short defxp = u->DefensiveStrength() + 3 * u->XPLevel();
@@ -302,7 +304,7 @@ void AI::BuildReinforcements( void ) const {
   gmul = p1_ground / p2_aground;
   smul = p1_ship / p2_aship;
 
-  for ( Building *b = static_cast<Building *>(Gam->BuildingsList()->Head());
+  for ( Building *b = static_cast<Building *>(mission.GetShops().Head());
         b; b = static_cast<Building *>(b->Next()) ) {
 
     if ( (b->Owner() == player) && b->IsFactory() ) {
@@ -317,7 +319,7 @@ void AI::BuildReinforcements( void ) const {
 
         for ( int i = 0; i < 32; ++i ) {
           if ( blueprints & (1 << i) ) {
-            type = Gam->GetUnitType( i );
+            type = mission.GetUnitSet().GetUnitInfo( i );
             if ( crystals >= type->Cost() ) {
               unsigned short val = type->Firepower(U_AIR) * amul +
                                    type->Firepower(U_GROUND) * gmul +
@@ -333,7 +335,7 @@ void AI::BuildReinforcements( void ) const {
         }
 
         if ( best ) {
-          Gam->CreateUnit( best, b->Position(), player );
+          mission.CreateUnit( best->ID(), *player, b->Position() );
           b->SetCrystals( crystals - best->Cost() );
 
           // recalculate unit type multipliers
@@ -461,11 +463,12 @@ void AI::CommandUnitDefend( Unit *u, AIObj &obj ) {
     bool attacked = false;
     Unit *last = NULL;
     do {
-      tg = ClosestUnit( Gam->NextPlayer(), u->Position(), U_GROUND|U_AIR|U_SHIP, U_DESTROYED|U_SHELTERED, last );
+      tg = ClosestUnit( &mission.GetOtherPlayer(*player), u->Position(),
+                  U_GROUND|U_AIR|U_SHIP, U_DESTROYED|U_SHELTERED, last );
       if ( tg && u->CanHitType( tg ) && SameDirection( u->Position(), tg->Position(), obj.pos ) &&
            UnitGoTo( u, tg->Position(), u->WeaponRange(tg) ) ) {
         attacked = true;
-        if ( u->CanHit( tg ) ) Gam->RegisterCombat( u, tg );
+        if ( u->CanHit( tg ) ) mission.RegisterBattle( u, tg );
         break;
       }
       last = tg;
@@ -473,7 +476,8 @@ void AI::CommandUnitDefend( Unit *u, AIObj &obj ) {
 
     if ( !attacked ) UnitGoTo( u, obj.pos, AI_ATTENTION_RADIUS );
   } else {
-    tg = ClosestUnit( Gam->NextPlayer(), obj.pos, U_GROUND|U_AIR|U_SHIP, U_DESTROYED|U_SHELTERED );
+    tg = ClosestUnit( &mission.GetOtherPlayer(*player), obj.pos,
+                      U_GROUND|U_AIR|U_SHIP, U_DESTROYED|U_SHELTERED );
 
     // if the enemy comes too close attack
     // if our objective is between us and the enemy try to take a position in front of the objective
@@ -503,7 +507,7 @@ void AI::CommandUnitConquer( Unit *u, AIObj &obj ) {
                     // hostile units along the way
     enemy = FindBestTarget( u );
     if ( enemy && UnitGoTo( u, enemy->Position(), u->WeaponRange(enemy) ) ) {
-      if ( u->CanHit( enemy ) ) Gam->RegisterCombat( u, enemy );
+      if ( u->CanHit( enemy ) ) mission.RegisterBattle( u, enemy );
     } else if ( !enemy ) CommandUnitReturnToBase( u );
   }
 }
@@ -524,7 +528,7 @@ void AI::CommandUnitAttack( Unit *u, AIObj &obj ) {
       UnitGoTo( u, tg->Position(), 0 ); // FIXME: is 0 ok, or should it be WeaponRange? How to get units out then?
     else {
       UnitGoTo( u, dest, 0 );
-      if ( u->CanHit( tg ) ) Gam->RegisterCombat( u, tg );
+      if ( u->CanHit( tg ) ) mission.RegisterBattle( u, tg );
     }
   } else CommandUnitReturnToBase( u );
 }
@@ -651,7 +655,7 @@ unsigned short AI::UnitStrength( Unit *u ) const {
 ////////////////////////////////////////////////////////////////////////
 
 void AI::UnitPresence( Player *owner, AIObj *obj, short radius /* = -1 */ ) const {
-  for ( Unit *u = static_cast<Unit *>(Gam->UnitsList()->Head());
+  for ( Unit *u = static_cast<Unit *>(mission.GetUnits().Head());
         u; u = static_cast<Unit *>(u->Next()) ) {
     if ( (u->Owner() == owner) && ((radius == -1) ||
        (Distance( obj->pos, u->Position() ) <= radius)) ) {
@@ -691,11 +695,11 @@ Unit *AI::ClosestUnit( Player *owner, const Point &p, unsigned long uflags,
     last_dist = Distance( p, last->Position() );
     u = static_cast<Unit *>( last->Next() );
     if ( !u ) {
-      u = static_cast<Unit *>( Gam->UnitsList()->Head() );
+      u = static_cast<Unit *>( mission.GetUnits().Head() );
       ++last_dist;
     }
   } else {
-    u = static_cast<Unit *>( Gam->UnitsList()->Head() );
+    u = static_cast<Unit *>( mission.GetUnits().Head() );
     last_dist = -1;
   }
 
@@ -710,7 +714,7 @@ Unit *AI::ClosestUnit( Player *owner, const Point &p, unsigned long uflags,
     }
     u = static_cast<Unit *>( u->Next() );
     if ( !u && last ) {
-      u = static_cast<Unit *>( Gam->UnitsList()->Head() );
+      u = static_cast<Unit *>( mission.GetUnits().Head() );
       ++last_dist;      // increase distance for the next loop
     }
   }
@@ -740,11 +744,11 @@ Building *AI::ClosestBuilding( Player *owner, const Point &p,
     last_dist = Distance( p, last->Position() );
     b = static_cast<Building *>( last->Next() );
     if ( !b ) {
-      b = static_cast<Building *>( Gam->BuildingsList()->Head() );
+      b = static_cast<Building *>( mission.GetShops().Head() );
       ++last_dist;
     }
   } else {
-    b = static_cast<Building *>( Gam->BuildingsList()->Head() );
+    b = static_cast<Building *>( mission.GetShops().Head() );
     last_dist = -1;
   }
 
@@ -757,7 +761,7 @@ Building *AI::ClosestBuilding( Player *owner, const Point &p,
     }
     b = static_cast<Building *>( b->Next() );
     if ( !b && last ) {
-      b = static_cast<Building *>( Gam->BuildingsList()->Head() );
+      b = static_cast<Building *>( mission.GetShops().Head() );
       ++last_dist;      // increase distance for the next loop
     }
   }
@@ -802,7 +806,7 @@ Point AI::FollowPath( const Unit *u, const Path &path, unsigned short dist ) con
   bool leave;
   short dir;
 
-  MoveShader shader( map, *Gam->UnitsList() );
+  MoveShader shader( map, mission.GetUnits() );
   shader.ShadeMap( u );
 
   do {
@@ -840,9 +844,9 @@ Unit *AI::FindBestTarget( const Unit *u ) {
 
   if ( u->IsDefensive() ) return NULL;
 
-  for ( Unit *tg = static_cast<Unit *>( Gam->UnitsList()->Head() );
+  for ( Unit *tg = static_cast<Unit *>( mission.GetUnits().Head() );
         tg; tg = static_cast<Unit *>( tg->Next() ) ) {
-    if ( (tg->Owner() == Gam->NextPlayer()) && u->CanHitType( tg )
+    if ( (tg->Owner() == &mission.GetOtherPlayer(*player)) && u->CanHitType( tg )
          && !tg->IsSheltered() ) {
 
       if ( UnitCanReach( u, tg->Position(), u->WeaponRange(tg) ) ) {
