@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,8 +28,10 @@
 
 class Building : public UnitContainer, public Node, public MapObject {
 public:
-  Building( SDL_RWops *file, Player **players );
-  int Save( SDL_RWops *file ) const;
+  Building( void ) :
+    MapObject(MO_BUILDING), b_name(0), b_player(0) {}
+  short Load( File &file );
+  int Save( File &file ) const;
 
   bool Allow( const Unit *unit ) const;
 
@@ -42,16 +44,18 @@ public:
   unsigned short CrystalStore( void ) const { return b_crystalstore; }
   unsigned short ID( void ) const { return b_id; }
   const char *Name( void ) const { return b_name; }
+  signed char NameID( void ) const { return b_name_id; }
   Player *Owner( void ) const { return b_player; }
   const Point &Position( void ) const { return b_pos; }
   unsigned long UnitProduction( void ) const { return b_uprod; }
 
-  void SetOwner( Player *player );
+  void SetOwner( Player *player, bool recurse = true );
   void SetCrystalProduction( unsigned char crystals ) { b_cprod = crystals; }
   void ModifyCrystalProduction( short crystals );
   void SetCrystals( unsigned short crystals );
   void ModifyCrystals( short crystals );
   void SetUnitProduction( unsigned long unitmask ) { b_uprod |= unitmask; }
+  void SetName( const char *name ) { b_name = name; }
 
 private:
   Point b_pos;
@@ -65,7 +69,8 @@ private:
   unsigned char b_cprod;          // crystal production per turn for mines
   unsigned long b_uprod;          // unit types that can be built for factories
 
-  char b_name[20];
+  signed char b_name_id;
+  const char *b_name;
 
   Player *b_player;
 };
