@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,10 +39,9 @@ public:
 
 class UnitContainer {
 public:
-  UnitContainer( unsigned char type );
+  UnitContainer( void );
   virtual ~UnitContainer( void ) {}
 
-  virtual Player *Owner( void ) const = 0;
   virtual unsigned short Crystals( void ) const = 0;
   virtual void SetCrystals( unsigned short crystals ) = 0;
   void SetWeightLimits( unsigned char min, unsigned char max )
@@ -53,20 +52,19 @@ public:
 
   Unit *GetUnit( short slot ) const;
   virtual bool Allow( const Unit *unit ) const;
+  bool Allow( const UnitType *type ) const;
 
   unsigned char UnitCount( void ) const { return uc_units.CountNodes(); }
   unsigned short Slots( void ) const { return uc_slots; }
   unsigned short FullSlots( void ) const { return uc_slots_full; }
   unsigned char MinWeight( void ) const { return uc_min_weight; }
   unsigned char MaxWeight( void ) const { return uc_max_weight; }
-  unsigned char CType( void ) const { return uc_type; }
 
 protected:
   unsigned short uc_slots;
   unsigned short uc_slots_full;
   unsigned char uc_min_weight;
   unsigned char uc_max_weight;
-  unsigned char uc_type;
 
   List uc_units;
 };
@@ -74,14 +72,15 @@ protected:
 
 class Transport : public Unit, public UnitContainer {
 public:
+  Transport( void ) : t_crystals(0) {}
   Transport( const UnitType *type, Player *player, unsigned short id,
-             short x, short y );
-  Transport( SDL_RWops *file, const UnitType *type, Player *player );
+             const Point &pos );
+
+  int Load( File &file, const UnitType *type, Player *player );
+  int Save( File &file ) const;
 
-  int Save( SDL_RWops *file ) const;
   void SetCrystals( unsigned short crystals );
   unsigned short Crystals( void ) const { return t_crystals; }
-  Player *Owner( void ) const { return Unit::Owner(); }
   void SetPosition( short x, short y );
   bool Hit( unsigned short damage );
   bool Allow( const Unit *unit ) const;
