@@ -23,11 +23,7 @@
 #ifndef _INCLUDE_GAME_H
 #define _INCLUDE_GAME_H
 
-#include <string>
-using namespace std;
-
-#include "lset.h"
-#include "player.h"
+#include "mission.h"
 #include "mapwindow.h"
 #include "extwindow.h"
 #include "list.h"
@@ -50,9 +46,6 @@ extern class Image *Images[];
 
 #define DEFAULT_DELAY	(5 * ANIM_SPEED_UNIT)
 
-class History;
-class Event;
-
 class Game : public WidgetHook {
 public:
   Game( View *view );
@@ -60,97 +53,67 @@ public:
 
   int Load( const char *file );
   int Save( const char *file );
-  int SwitchMap( const char *mission );
 
+  void InitWindows( void );
+  void InitKeys( void );
   void Quit( void ) const;
-  Unit *LoadUnit( SDL_RWops *file, bool force = false );
+  void Shutdown( void ) const;
+  GUI_Status HandleEvent( const SDL_Event &event );
 
-  void StartTurn( void );
+  GUI_Status StartTurn( void );
   GUI_Status EndTurn( void );
 
-  Building *GetBuildingByID( unsigned short id ) const;
-  Unit *GetUnitByID( unsigned short id ) const;
-  Event *GetEventByID( unsigned char id ) const;
-
   void SetCursor( const Point &cursor ) const;
   Unit *MoveUnit( Unit *u, const Point &dest );
   int MoveUnit( Unit *u, Direction dir );
   void SelectUnit( Unit *u );
-  void SelectNextUnit( void );
   void DeselectUnit( bool update = true );
-  void EnterSpecialMode( unsigned char mode );
-  bool UnitTargets( Unit *u ) const;
-  Unit *CreateUnit( const UnitType *type, const Point &hex, Player *player );
-  void RegisterCombat( Unit *att, Unit *def );
 
-  Player *CurrentPlayer( void ) const { return player; }
-  Player *NextPlayer( void ) const { return players[player->ID()^1]; }
-  unsigned short Turn( void ) const { return turn; }
-  List *UnitsList( void ) { return &units; }
-  List *BuildingsList( void ) { return &buildings; }
-  Player *GetPlayer( unsigned char id ) const { return players[id]; }
-  void SetNextMap( signed char mapid ) { next_map = mapid; }
-
-  bool HaveWinner( void );
-  GUI_Status CheckEvents( void );
-  History *GetHistory( void ) const { return history; }
   MapWindow *GetMapWindow( void ) const { return mwin; }
-  const char *GetMessage( unsigned short id ) const { return messages[id]; }
-
-  const UnitType *GetUnitType( unsigned short type ) const { return unit_set->GetUnitInfo(type); }
+  Mission *GetMission( void ) const { return mission; }
   void UnitInfo( Unit *unit );
-  void ShowBriefing( void ) const;
-  void ShowDebriefing( Player *player, bool restart );
-  void ShowLevelInfo( void ) const;
-  void ContainerContent( UnitContainer *c );
-  void GameMenu( void );
-  void UnitMenu( Unit *unit );
-  void SetPlayerPassword( Player *player ) const;
-  bool CheckPassword( const char *title, const char *msg, const char *pw, bool retry ) const;
-  const char *GetLevelPassword( void ) const { return (password[0] != '\0') ? password : NULL; }
-
-  GUI_Status HandleEvent( const SDL_Event &event );
-  GUI_Status WidgetActivated( Widget *button, Window *win );
+  void ResolveBattle( Combat *com );
 
 private:
   string CreateSaveFileName( const char *filename ) const;
   void ClearMine( Transport *sweeper, Unit *mine );
+  bool HaveWinner( void );
+  GUI_Status CheckEvents( void );
 
   void MoveCommand( int key );
   void SelectCommand( const Point &hex );
+  void ScrollCommand( int key );
   void HandleLMB( const Point &hex );
+  void EnterSpecialMode( unsigned char mode );
+  void SelectNextUnit( void );
 
+  bool UnitTargets( Unit *u ) const;
+  bool MinesweeperTargets( Unit *u ) const;
+
+  void ShowBriefing( void ) const;
+  GUI_Status ShowDebriefing( Player &player, bool restart );
+  void ShowLevelInfo( void ) const;
+  void ContainerContent( UnitContainer *c );
+  void GameMenu( void );
+  void UnitMenu( Unit *unit );
+  int SwitchMap( const char *mission );
+
+  bool SetPlayerPassword( Player *player ) const;
+  bool CheckPassword( const char *title, const char *msg, const char *pw, short retry ) const;
+
+  GUI_Status WidgetActivated( Widget *button, Window *win );
+
+  Mission *mission;
   MapWindow *mwin;
 
-  unsigned short flags;
-  unsigned short turn;
-  unsigned short msg_num;
-  unsigned long msg_len;
-
-  List units;           // list of units
-  List buildings;       // list of structures
-  List events;          // list of events
-  List combat;          // list of combat records
-
-  Player *player;       // current player
   Unit *unit;		// selected unit
 
-  Player *players[2];   // array of players
-  char **messages;
-  string last_file_name; // remember save file names
+  short keys[6];	// keyboard shortcuts
 
-  unsigned char turn_phase;
-  signed char level_info;
-  signed char next_map;
-  char password[8];
-
-  Map *map;
-  History *history;
-  UnitSet *unit_set;
-  TerrainSet *terrain_set;
   MoveShader *shader;
   View *view;
 
+  string last_file_name; // remember save file names
   Unit *g_tmp_prv_unit;
 };
 
