@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,122 +20,115 @@
 // history.cpp
 ////////////////////////////////////////////////////////////////////////
 
-#include "SDL_endian.h"
-
 #include "history.h"
+#include "game.h"
 #include "options.h"
+#include "msgs.h"
 
 extern Options CFOptions;
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : HistEvent::HistEvent
-// DESCRIPTION: Create a new event for the turn history.
-// PARAMETERS : -
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-HistEvent::HistEvent( void ) {
-}
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : HistEvent::HistEvent
+// NAME       : HistEvent::Load
 // DESCRIPTION: Load an event from a file.
-// PARAMETERS : file - SDL_RWops file descriptor
-// RETURNS    : -
+// PARAMETERS : file - file descriptor
+// RETURNS    : -1 on error, 0 otherwise
 ////////////////////////////////////////////////////////////////////////
 
-HistEvent::HistEvent( SDL_RWops *file ) {
-  SDL_RWread( file, &type, 1, 1 );
-  for ( int i = 0; i < 4; ++i ) data[i] = SDL_ReadLE16( file );
+int HistEvent::Load( File &file ) {
+  type = file.Read8();
+  for ( int i = 0; i < 4; ++i ) data[i] = file.Read16();
+  return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : HistEvent::Save
 // DESCRIPTION: Save an event to a file.
-// PARAMETERS : file - SDL_RWops file descriptor
+// PARAMETERS : file - file descriptor
 // RETURNS    : 0 on success, non-0 otherwise
 ////////////////////////////////////////////////////////////////////////
 
-int HistEvent::Save( SDL_RWops *file ) const {
-  SDL_RWwrite( file, &type, 1, 1 );
-  for ( int i = 0; i < 4; ++i ) SDL_WriteLE16( file, data[i] );
+int HistEvent::Save( File &file ) const {
+  file.Write8( type );
+  for ( int i = 0; i < 4; ++i ) file.Write16( data[i] );
   return 0;
 }
 
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : History::History
+// NAME       : History::Load
 // DESCRIPTION: Load a History from a file.
-// PARAMETERS : file - open SDL_RWops descriptor
-//              num  - number of recorded events in file
-// RETURNS    : -
+// PARAMETERS : file    - open file descriptor
+//              mission - current mission
+// RETURNS    : -1 on error, number of events loaded otherwise
 ////////////////////////////////////////////////////////////////////////
 
-History::History( SDL_RWops *file, unsigned short num ) {
-  unsigned short num_units, i;
+short History::Load( File &file, Mission &mission ) {
+  unsigned short num_events;
 
-  num_units = SDL_ReadLE16( file );
+  num_events = file.Read16();
 
-  for ( i = 0; i < num; ++i ) {
-    events.AddTail( new HistEvent( file ) );
-  }
+  if ( num_events > 0 ) {
+    unsigned short num_units, i;
+    num_units = file.Read16();
 
-  for ( i = 0; i < num_units; ++i ) {
-    Unit *u = Gam->LoadUnit( file, true );
-    if ( u ) units.AddTail( u );
+    for ( i = 0; i < num_events; ++i ) {
+      HistEvent *he = new HistEvent();
+      he->Load( file );
+      events.AddTail( he );
+    }
+
+    for ( i = 0; i < num_units; ++i ) {
+      Unit *u = mission.LoadUnit( file, true );
+      if ( u ) units.AddTail( u );
+    }
+    return num_events;
   }
+  return 0;
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : History::History
-// DESCRIPTION: Create a new (empty) History from the current game.
-// PARAMETERS : -
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-History::History( void ) {
-  // create the unit copies
-  Unit *dummy, *u = static_cast<Unit *>( Gam->UnitsList()->Head() );
-  while ( u ) {
-    dummy = new Unit( *u );
-    dummy->SetFlags( U_DUMMY );
-    units.AddTail( dummy );
-    u = static_cast<Unit *>( u->Next() );
-  }
-}
-
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : History::Save
 // DESCRIPTION: Save a history to file.
-// PARAMETERS : file - open SDL_RWops file descriptor
+// PARAMETERS : file - open file descriptor
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int History::Save( SDL_RWops *file ) const {
+int History::Save( File &file ) const {
   unsigned short num = events.CountNodes();
-  SDL_WriteLE16( file, num );
+  file.Write16( num );
   if ( num == 0 ) return 0;
 
-  num = units.CountNodes();
-  SDL_WriteLE16( file, num );
+  file.Write16( units.CountNodes() );
 
   // save events
-  HistEvent *he = static_cast<HistEvent *>( events.Head() );
-  while ( he ) {
+  for ( HistEvent *he = static_cast<HistEvent *>( events.Head() );
+        he; he = static_cast<HistEvent *>( he->Next() ) )
     he->Save( file );
-    he = static_cast<HistEvent *>( he->Next() );
-  }
 
   // save units
-  Unit *u = static_cast<Unit *>( units.Head() );
-  while ( u ) {
+  for ( Unit *u = static_cast<Unit *>( units.Head() );
+        u; u = static_cast<Unit *>( u->Next() ) )
     u->Save( file );
-    u = static_cast<Unit *>( u->Next() );
-  }
   return 0;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : History::StartRecording
+// DESCRIPTION: Create a new (empty) History from the current game.
+// PARAMETERS : list - list of units
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void History::StartRecording( List &list ) {
+  // create the unit copies
+  for ( Unit *u = static_cast<Unit *>( list.Head() );
+        u; u = static_cast<Unit *>( u->Next() ) ) {
+    Unit *dummy = new Unit( *u );
+    dummy->SetFlags( U_DUMMY );
+    units.AddTail( dummy );
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : History::Replay
 // DESCRIPTION: Play all events that have been stored during the last
@@ -161,53 +154,60 @@ void History::Replay( MapWindow *mapwin ) {
     List units_bak;
     BeginReplay( units_bak, map );
 
-    unsigned short count = events.CountNodes();
-
-    // undo the map tile changes which have been recorded this turn
-    while ( he && (he->type == HIST_TILE_INTERNAL) ) {
-      map->SetHexType( he->data[1], he->data[2], he->data[0] );
-      he = static_cast<HistEvent *>( he->Next() );
-      --count;
-    }
-
     if ( quick ) delay = 0;
     else {
+      unsigned short count = events.CountNodes();
+
+      // undo the map tile changes which have been recorded this turn
+     while ( he && (he->type == HIST_TILE_INTERNAL) ) {
+        map->SetHexType( he->data[1], he->data[2], he->data[0] );
+        he = static_cast<HistEvent *>( he->Next() );
+        --count;
+      }
+
       delay = DEFAULT_DELAY;
       mv->Enable();
       mapwin->Draw();
       mapwin->Show();
-    }
 
-    progwin = new ProgressWindow( 5,
+      progwin = new ProgressWindow( mapwin->Width() / 4,
               mapwin->Height() - view->SmallFont()->Height() - 30,
               mapwin->Width() / 2, view->SmallFont()->Height() + 20,
               0, count, WIN_PROG_ABORT, view );
+    }
 
     while ( he && !abort ) {
 
-      switch ( he->type ) {
-        case HIST_MOVE:
-          ReplayMoveEvent( *he, mapwin ); break;
-        case HIST_ATTACK:
-          ReplayAttackEvent( *he, mapwin ); break;
-        case HIST_COMBAT:
-          ReplayCombatEvent( *he, mapwin ); break;
-        case HIST_TILE:
-          ReplayTileEvent( *he, mapwin ); break;
-        case HIST_MSG:
-          ReplayMessageEvent( *he, view ); break;
-        default: break;
+      if ( he->type == HIST_MOVE )
+        ReplayMoveEvent( *he, mapwin );
+      else if ( he->type == HIST_COMBAT )
+        ReplayCombatEvent( *he, mapwin );
+      else if ( he->type == HIST_MSG )
+        ReplayMessageEvent( *he, view );
+
+      else if ( !quick ) {
+        // these events are skipped for quick replays
+        if ( he->type == HIST_ATTACK )
+          ReplayAttackEvent( *he, mapwin );
+        else if ( he->type == HIST_TILE )
+          ReplayTileEvent( *he, mapwin );
       }
+
+      if ( !quick ) {
+        progwin->Advance( 1 );
+        abort = progwin->Aborted();
+      }
+
       he = static_cast<HistEvent *>( he->Next() );
-      progwin->Advance( 1 );
-      abort = progwin->Aborted();
     }
 
     EndReplay( units_bak, map );
+    if ( progwin ) view->CloseWindow( progwin );
   }
 
+
   if ( replay && !abort ) SDL_Delay( delay * 2 );
-  else {
+  else if ( !quick ) {
     // if replay has been stopped or disabled we still need to revert
     // all changes to the map and display the cached messages
     while ( he ) {
@@ -216,8 +216,6 @@ void History::Replay( MapWindow *mapwin ) {
       he = static_cast<HistEvent *>( he->Next() );
     }
   }
-
-  if ( progwin ) view->CloseWindow( progwin );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -231,18 +229,18 @@ void History::Replay( MapWindow *mapwin ) {
 ////////////////////////////////////////////////////////////////////////
 
 void History::BeginReplay( List &backup, Map *map ) {
-  List *gam_units = Gam->UnitsList();
+  List &gam_units = Gam->GetMission()->GetUnits();
   Unit *u;
 
-  while ( !gam_units->IsEmpty() ) {
-    u = static_cast<Unit *>( gam_units->RemHead() );
+  while ( !gam_units.IsEmpty() ) {
+    u = static_cast<Unit *>( gam_units.RemHead() );
     backup.AddTail( u );
     if ( !u->IsSheltered() ) map->SetUnit( NULL, u->Position() );
   }
 
   while ( !units.IsEmpty() ) {
     u = static_cast<Unit *>( units.RemHead() );
-    gam_units->AddTail( u );
+    gam_units.AddTail( u );
     if ( !u->IsSheltered() ) {
       map->SetUnit( u, u->Position() );
       if ( u->IsMine() ) u->SetMoves( MCOST_MIN );
@@ -263,18 +261,18 @@ void History::BeginReplay( List &backup, Map *map ) {
 ////////////////////////////////////////////////////////////////////////
 
 void History::EndReplay( List &backup, Map *map ) {
-  List *gam_units = Gam->UnitsList();
+  List &gam_units = Gam->GetMission()->GetUnits();
   Unit *u;
 
-  while ( !gam_units->IsEmpty() ) {
-    u = static_cast<Unit *>( gam_units->RemHead() );
+  while ( !gam_units.IsEmpty() ) {
+    u = static_cast<Unit *>( gam_units.RemHead() );
     if ( !u->IsSheltered() ) map->SetUnit( NULL, u->Position() );
     delete u;
   }
 
   while ( !backup.IsEmpty() ) {
     u = static_cast<Unit *>( backup.RemHead() );
-    gam_units->AddTail( u );
+    gam_units.AddTail( u );
     if ( !u->IsSheltered() ) map->SetUnit( u, u->Position() );
   }
 }
@@ -404,19 +402,23 @@ int History::RecordMsgEvent( short title, unsigned short msg ) {
 //              neighboring hex.
 // PARAMETERS : event  - move event
 //              mapwin - map window
+//              quick  - for quick replays we only need to update the
+//                       unit positions
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 void History::ReplayMoveEvent( const HistEvent &event, MapWindow *mapwin ) {
   MapView *mv = mapwin->GetMapView();
   Map *map = mv->GetMap();
-  Unit *u = Gam->GetUnitByID( event.data[0] );
+  Mission *mission = Gam->GetMission();
+
+  Unit *u = mission->GetUnit( event.data[0] );
   if ( u ) {
     if ( u != lastunit ) {
       if ( lastunit && lastunit->IsTransport() && !map->GetMapObject( lastpos ) ) {
         // we must adjust the position of carried units manually because
         // those pseudo-transports can't handle this themselves
-        for ( Unit *carry = static_cast<Unit *>( Gam->UnitsList()->Head() );
+        for ( Unit *carry = static_cast<Unit *>( mission->GetUnits().Head() );
               carry; carry = static_cast<Unit *>( carry->Next() ) ) {
           if ( carry->Position() == lastpos )
              carry->SetPosition( lastunit->Position().x, lastunit->Position().y );
@@ -428,7 +430,7 @@ void History::ReplayMoveEvent( const HistEvent &event, MapWindow *mapwin ) {
       SDL_Delay( delay );
     }
 
-    if ( !mv->HexVisible( u->Position() ) ) {
+    if ( mv->Enabled() && !mv->HexVisible( u->Position() ) ) {
       mv->CenterOnHex( u->Position() );
       mapwin->Show( *mv );
     }
@@ -443,7 +445,7 @@ void History::ReplayMoveEvent( const HistEvent &event, MapWindow *mapwin ) {
 
     Gam->MoveUnit( u, (Direction)event.data[1] );
     map->SetUnit( u, u->Position() );
-    mapwin->Show( mv->UpdateHex( u->Position() ) );
+    if ( mv->Enabled() ) mapwin->Show( mv->UpdateHex( u->Position() ) );
   }
 }
 
@@ -457,7 +459,7 @@ void History::ReplayMoveEvent( const HistEvent &event, MapWindow *mapwin ) {
 
 void History::ReplayAttackEvent( const HistEvent &event, MapWindow *mapwin ) {
   MapView *mv = mapwin->GetMapView();
-  Unit *u = Gam->GetUnitByID( event.data[0] );
+  Unit *u = Gam->GetMission()->GetUnit( event.data[0] );
   if ( u ) {
 
     if ( u != lastunit ) SDL_Delay( delay );
@@ -493,8 +495,8 @@ void History::ReplayAttackEvent( const HistEvent &event, MapWindow *mapwin ) {
 
 void History::ReplayMessageEvent( const HistEvent &event, View *view ) {
   MessageWindow *msgw = new MessageWindow(
-    (event.data[0] != -1) ? Gam->GetMessage( event.data[0] ) : "Message",
-    Gam->GetMessage( event.data[1] ), view );
+    (event.data[0] != -1) ? Gam->GetMission()->GetMessage( event.data[0] ) : MSG(MSG_MESSAGE),
+    Gam->GetMission()->GetMessage( event.data[1] ), view );
   msgw->EventLoop();
   view->CloseWindow( msgw );
 }
@@ -535,14 +537,11 @@ void History::ReplayCombatEvent( const HistEvent &event, MapWindow *mapwin ) {
     mapwin->Show();
   }
 
-  Combat cmb( Gam->GetUnitByID( event.data[0] ),
-              Gam->GetUnitByID( event.data[1] ) );
+  Combat cmb( Gam->GetMission()->GetUnit( event.data[0] ),
+              Gam->GetMission()->GetUnit( event.data[1] ) );
   cmb.aamod = (signed char)event.data[3];
   cmb.ddmod = (signed char)event.data[2];
-  CombatWindow *cwin = cmb.Resolve( mapwin, mapwin->GetView() );
-  if ( cwin ) {
-    cwin->EventLoop();
-    mapwin->GetView()->CloseWindow( cwin );
-  }
+
+  Gam->ResolveBattle( &cmb );
 }
 
