@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,20 +36,24 @@ enum {  HIST_MOVE = 0,
 
 class HistEvent : public Node {
 public:
-  HistEvent( void );
-  HistEvent( SDL_RWops *file );
-  int Save( SDL_RWops *file ) const;
+  HistEvent( void ) {}
+  int Load ( File &file );
+  int Save( File &file ) const;
 
   unsigned char type;
   short data[4];
 };
 
 
+class MapWindow;
+
 class History {
 public:
-  History( void );
-  History( SDL_RWops *file, unsigned short num );
-  int Save( SDL_RWops *file ) const;
+  History( void ) {}
+  short Load( File &file, Mission &mission );
+  int Save( File &file ) const;
+
+  void StartRecording( List &list );
 
   int RecordMoveEvent( const Unit &u, Direction dir );
   int RecordAttackEvent( const Unit &u, const Unit &target );
