@@ -21,74 +21,33 @@
 ////////////////////////////////////////////////////////////////////////
 
 #include <string.h>
-#include "SDL_endian.h"
 
 #include "initwindow.h"
-#include "fileio.h"
+#include "filewindow.h"
 #include "misc.h"
 #include "game.h"
+#include "strutil.h"
 #include "options.h"
+#include "msgs.h"
 
-#define ICON_QUIT_X	185
-#define ICON_QUIT_Y	57
-#define ICON_QUIT_W	7
-#define ICON_QUIT_H	7
+#define IW_WIDGET_TYPE		0
+#define IW_WIDGET_MAPTYPE	1
+#define IW_WIDGET_LEVELS	2
+#define IW_WIDGET_START		3
+#define IW_WIDGET_OPTIONS	4
+#define IW_WIDGET_GENERAL_OPTIONS  5
+#define IW_WIDGET_LANGUAGE_OPTIONS 6
+#define IW_WIDGET_VIDEO_OPTIONS    7
+#define IW_WIDGET_SOUND_OPTIONS    8
+#define IW_WIDGET_DIFFICULTY	9
 
-#define IW_WIDGET_LIST  	0
-#define IW_WIDGET_START_NEW	1
-#define IW_WIDGET_START_SAVED	2
-#define IW_WIDGET_LOAD_SAVED	3
-#define IW_WIDGET_LOAD_NEW	4
-#define IW_WIDGET_PBEM		5
-#define IW_WIDGET_AI		6
-#define IW_WIDGET_VIDEO_OPTIONS 7
+#define MAPTYPE_SINGLE   0
+#define MAPTYPE_CAMPAIGN 1
+#define MAPTYPE_SAVE     2
 
 extern Options CFOptions;
 
-struct initLevInfo {    // used as user_data for the LevListWidget
-  bool password;
-  bool singleplayer;
-  string *full_path;
-};
-
-const short GenericOptionsWindow::B_ID_OK      = 1;
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : LevListWidget::PrintItem
-// DESCRIPTION: Print a level name. If the map is protected by a
-//              password print it in red colour, otherwise in green.
-//              Put the default number of players behind the title in
-//              brackets.
-// PARAMETERS : item - node to print
-//              dest - destination surface
-//              x    - left edge of printing area
-//              y    - top edge of printing area
-//              clip - clipping rectangle
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void LevListWidget::PrintItem( const TLWNode *item, Surface *dest,
-                               short x, short y, const Rect &clip ) const {
-  Color col( CF_COLOR_WHITE );
-  string name( item->Name() );
-  name.erase( name.length() - 4 );   // remove .lev or .sav suffix
-
-  if ( item->UserData() ) {
-    initLevInfo *info = (initLevInfo *)item->UserData();
-    if ( info->password ) col = surface->GetFGPen();
-    name += ' ';
-    name += '(';
-    name += (info->singleplayer ? '1' : '2');
-    name += ')';
-  }
-
-  Rect oldclip;
-  dest->GetClipRect( oldclip );
-  dest->SetClipRect( clip );
-  font->Write( name.c_str(), dest, x, y, col );
-  dest->SetClipRect( oldclip );
-}
-
+const short GenericOptionsWindow::B_ID_OK = 1;
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : InitWindow::InitWindow
@@ -100,72 +59,103 @@ void LevListWidget::PrintItem( const TLWNode *item, Surface *dest,
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-InitWindow::InitWindow( View *view, Window *title ) : Window( WIN_CENTER, view ) {
-  this->title = title;
+InitWindow::InitWindow( View *view, Window *title ) :
+    Window( WIN_CENTER, view ), title(title) {
+  // read list of maps/saves
+  const string home_lev( get_home_levels_dir() );
+  if ( !home_lev.empty() )
+    FileWindow::CreateFilesList( home_lev.c_str(), ".lev", levels );
+  FileWindow::CreateFilesList( get_levels_dir().c_str(), ".lev", levels );
+  CompleteFilesList( levels );
+  FileWindow::CreateFilesList( get_save_dir().c_str(), ".sav", saves );
+  CompleteFilesList( saves );
+
+  Audio::PlayMusic( CF_MUSIC_THEME );
 
-  // calculate window dimensions
-  SetSize( MIN(380, view->Width()), sfont->Height() * 15 + 10 );
   view->SetFGPen( Color( 0x00d87c00 ) );
   view->SetBGPen( Color( CF_COLOR_BLACK ) );
 
-  const string home_lev( get_home_levels_dir() );
-  if ( !home_lev.empty() ) create_files_list( home_lev.c_str(), ".lev", levels );
-  create_files_list( get_levels_dir().c_str(), ".lev", levels );
-  CompleteFilesList( levels );
-  create_files_list( get_save_dir().c_str(), ".sav", saves );
-  CompleteFilesList( saves );
+  // calculate window dimensions
+  const char *typelbl = MSG(MSG_B_GAME_TYPE);
+  const char *mapslbl = MSG(MSG_B_MAP_TYPE);
+  const char *hcaplbl = MSG(MSG_B_HANDICAP);
+  unsigned short typewidth = sfont->TextWidth(typelbl);
+  unsigned short mapswidth = sfont->TextWidth(mapslbl);
+  unsigned short wdh = sfont->Height() + 8;
+
+  unsigned short xoff = MAX( typewidth, mapswidth ) - sfont->CharWidth('_') + 10;
+
+  SetSize( MIN(view->Width(), xoff + sfont->Width() * 40 + 20),
+           MIN(view->Height(), wdh * 12 + 10 ) );
 
   // create widgets
-  levwidget = new LevListWidget( IW_WIDGET_LIST, 5, 5,
-                    w/2 - 10, sfont->Height() * 15, &levels,
+  Widget *wd;
+
+  const char *typelabels[4];
+  typelabels[0] = MSG(MSG_GAME_HOT_SEAT);
+  typelabels[1] = MSG(MSG_GAME_AI);
+  typelabels[2] = MSG(MSG_GAME_PBEM);
+  typelabels[3] = 0;
+
+  const char *mapslabels[4];
+  mapslabels[0] = MSG(MSG_MAPS_SINGLES);
+  mapslabels[1] = MSG(MSG_MAPS_CAMPAIGNS);
+  mapslabels[2] = MSG(MSG_MAPS_SAVES);
+  mapslabels[3] = 0;
+
+  const char *difflabels[4];
+  difflabels[0] = MSG(MSG_HANDICAP_NONE);
+  difflabels[1] = MSG(MSG_HANDICAP_P1);
+  difflabels[2] = MSG(MSG_HANDICAP_P2);
+  difflabels[3] = 0;
+
+  gtypewidget = new CycleWidget( IW_WIDGET_TYPE, xoff, 5,
+                (w - xoff - 10) / 2, wdh, WIDGET_ALIGN_LEFT,
+                typelbl, CFOptions.GetGameType(), typelabels, this );
+  gtypewidget->SetHook( this );
+
+  xoff = gtypewidget->LeftEdge() + gtypewidget->Width() + 5;
+  wd = new DropWidget( IW_WIDGET_OPTIONS, xoff, gtypewidget->TopEdge(),
+           w - xoff - 5, wdh, 0, MSG(MSG_B_OPTIONS), this );
+  wd->SetHook( this );
+
+  mtypewidget = new CycleWidget( IW_WIDGET_MAPTYPE,
+           gtypewidget->LeftEdge(), gtypewidget->TopEdge() + wdh + 5,
+           gtypewidget->Width(), wdh, 0, mapslbl, 0, mapslabels, this );
+  mtypewidget->SetHook( this );
+  CFOptions.SetCampaign( false );
+
+  xoff += sfont->TextWidth( hcaplbl ) - sfont->CharWidth( '_' ) + 5;
+  diffwidget = new CycleWidget( IW_WIDGET_DIFFICULTY,
+           xoff, mtypewidget->TopEdge(), w - xoff - 5, wdh,
+           0, hcaplbl, 0, difflabels, this );
+
+  levwidget = new TextListWidget( IW_WIDGET_LEVELS,
+                    5, mtypewidget->TopEdge() + wdh + 5,
+                    w/2 - 7, h - (wdh + 5) * 3 - 5, &levels,
                     -1, WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY,
                     NULL, this );
+  levwidget->SetHook( this );
 
-  startwidget = new ButtonWidget( IW_WIDGET_START_NEW, w/2 + 5,
-                levwidget->TopEdge() + levwidget->Height() - sfont->Height() - 8,
-                w/2 - 10, sfont->Height() + 8, WIDGET_DEFAULT,
-                "_Start", this );
-  startwidget->SetHook( this );
+  maxmap = Rect( w/2 + 2, levwidget->TopEdge(), w/2 - 7, levwidget->Height() );
 
-  videowidget = new ButtonWidget( IW_WIDGET_VIDEO_OPTIONS,
-                startwidget->LeftEdge(), levwidget->TopEdge(),
-                startwidget->Width() - ICON_QUIT_W - 13, startwidget->Height(),
-                0, "_Video Options", this );
-  videowidget->SetHook( this );
+  // of the these two widgets, only one is actually present at a time
+  mapwidget = new MapWidget( 0, maxmap.x, maxmap.y, maxmap.w, maxmap.h,
+              0, this );
+  campinfowidget = new TextScrollWidget( 0, maxmap.x, maxmap.y,
+              maxmap.w, maxmap.h, 0, 0, 0, this );
+  RemoveWidget( campinfowidget );
 
-  listctrlwidget = new ButtonWidget( IW_WIDGET_LOAD_SAVED, startwidget->LeftEdge(),
-                   videowidget->TopEdge() + videowidget->Height() + 5,
-                   startwidget->Width(), videowidget->Height(), 0,
-                   "_Load saved game", this );
-  listctrlwidget->SetHook( this );
 
-  pbemwidget = new CheckboxWidget( IW_WIDGET_PBEM, listctrlwidget->LeftEdge() + 5,
-                   listctrlwidget->TopEdge() + listctrlwidget->Height() + 10,
-                   DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, CFOptions.GetPBEM(),
-                   WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT|
-                   (CFOptions.GetComputerPlayer() ? WIDGET_DISABLED : 0),
-                   "Play By _e-Mail", this );
-  pbemwidget->SetHook( this );
+  wd = new ButtonWidget( IW_WIDGET_START, 5, h - wdh - 2, (w - 10) / 2 - 2, wdh,
+                WIDGET_DEFAULT, MSG(MSG_B_START), this );
+  wd->SetHook( this );
 
-  aiwidget = new CheckboxWidget( IW_WIDGET_AI, pbemwidget->LeftEdge(),
-                   pbemwidget->TopEdge() + pbemwidget->Height() + 3,
-                   DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, CFOptions.GetComputerPlayer(),
-                   WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT|
-                   (CFOptions.GetPBEM() ? WIDGET_DISABLED : 0),
-                   "Play vs. _Computer", this );
-  aiwidget->SetHook( this );
-
-  // the quit button
-  ButtonWidget *btn = new ButtonWidget( GUI_QUIT, w - ICON_QUIT_W - 9,
-                   videowidget->TopEdge(), ICON_QUIT_W + 4, ICON_QUIT_H + 4,
-                   WIDGET_STYLE_GFX, NULL, this );
-  btn->SetKey( SDLK_ESCAPE );
-  btn->SetImage( view->GetSystemIcons(),
-                   Rect( ICON_QUIT_X, ICON_QUIT_Y, ICON_QUIT_W, ICON_QUIT_H ),
-                   Rect( ICON_QUIT_X, ICON_QUIT_Y, ICON_QUIT_W, ICON_QUIT_H ) );
+  wd = new ButtonWidget( GUI_QUIT, wd->LeftEdge() + wd->Width() + 4,
+                wd->TopEdge(), wd->Width(), wdh,
+                0, MSG(MSG_B_EXIT), this );
 
   Draw();
-  Show();
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -177,43 +167,32 @@ InitWindow::InitWindow( View *view, Window *title ) : Window( WIN_CENTER, view )
 
 InitWindow::~InitWindow( void ) {
   TLWNode *n;
-  initLevInfo *li;
+  string *str;
 
   while ( !levels.IsEmpty() ) {
     n = static_cast<TLWNode *>( levels.RemHead() );
-    li = (initLevInfo *)n->UserData();
-    if ( li ) {
-      delete li->full_path;
-      delete li;
-    }
+    str = (string *)n->UserData();
+    delete str;
+    delete n;
+  }
+
+  while ( !campaigns.IsEmpty() ) {
+    n = static_cast<TLWNode *>( campaigns.RemHead() );
+    str = (string *)n->UserData();
+    delete str;
     delete n;
   }
 
   while ( !saves.IsEmpty() ) {
     n = static_cast<TLWNode *>( saves.RemHead() );
-    li = (initLevInfo *)n->UserData();
-    if ( li ) {
-      delete li->full_path;
-      delete li;
-    }
+    str = (string *)n->UserData();
+    delete str;
     delete n;
   }
-}
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : InitWindow::Draw
-// DESCRIPTION: Draw the main window.
-// PARAMETERS : -
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void InitWindow::Draw( void ) {
-  Window::Draw();
-  DrawBox( Rect( listctrlwidget->LeftEdge(),
-                 pbemwidget->TopEdge() - 5,
-                 listctrlwidget->Width(),
-                 startwidget->TopEdge() - pbemwidget->TopEdge() - 5),
-           BOX_CARVED );
+  // one of these is not attached to the window
+  if ( mtypewidget->GetValue() == 1 ) delete mapwidget;
+  else delete campinfowidget;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -225,82 +204,162 @@ void InitWindow::Draw( void ) {
 ////////////////////////////////////////////////////////////////////////
 
 GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
+  TLWNode *n;
+  GUI_Status rc = GUI_OK;
 
   switch ( button->ID() ) {
-  case IW_WIDGET_START_NEW:
-  case IW_WIDGET_START_SAVED: {
-    TLWNode *n = static_cast<TLWNode *>( levwidget->Selected() );
-    if ( !n ) new NoteWindow( "Error!", "No level selected!", WIN_CLOSE_ESC, view );
+  case IW_WIDGET_LEVELS:  // user selected mission, show map
+    n = static_cast<TLWNode *>( levwidget->Selected() );
+    if ( n ) {
+      string *lname = (string *)n->UserData();
+      Mission *m = LoadMission( lname->c_str() );
+      if ( m ) {
+        if ( mtypewidget->GetValue() == MAPTYPE_CAMPAIGN ) {
+          campinfowidget->SetText( m->GetCampaignInfo() );
+        } else {
+          Map &map = m->GetMap();
+          unsigned char magnify = MIN( maxmap.Width() / map.Width(),
+                                       maxmap.Height() / map.Height() );
+          if ( magnify == 0 ) magnify = 1;
+          else if ( magnify > 6 ) magnify = 6;
+
+          mapwidget->w = magnify * map.Width() + 2;
+          mapwidget->h = magnify * map.Height() + 2 + magnify/2;
+          mapwidget->Center( maxmap );
+          mapwidget->Clip( maxmap );
+
+          mapwidget->SetPlayerColors( m->GetPlayer(PLAYER_ONE).LightColor(),
+                                      m->GetPlayer(PLAYER_TWO).LightColor() );
+          mapwidget->SetMap( &map, Rect(0,0,250,250), magnify );
+        }
+        delete m;
+        Draw();
+        Show();
+      }
+    }
+    break;
+  case IW_WIDGET_START:
+    n = static_cast<TLWNode *>( levwidget->Selected() );
+    if ( !n ) new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_NO_MAP), WIN_CLOSE_ESC, view );
     else {
-      string *lname = ((initLevInfo *)n->UserData())->full_path;
+      string *lname = (string *)n->UserData();
 
       Gam = new Game( view );
       int val = Gam->Load( lname->c_str() );
       if ( val  ) {
-        NoteWindow *nw = new NoteWindow( "Error!", "Failed to load mission!", 0, view );
+        NoteWindow *nw = new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_LOAD_MAP), 0, view );
         nw->SetButtonID( 0, GUI_RESTART );
-      } else if ( !Gam->GetLevelPassword() ||
-                  Gam->CheckPassword( "Access restricted", "Enter access code",
-                                      Gam->GetLevelPassword(), false ) ) {
-        view->CloseWindow( this );
-        view->Refresh();
-        Gam->StartTurn();         // let the games begin...
       } else {
-        delete Gam;
-        Gam = NULL;
-        new NoteWindow( "Access denied!", "Invalid access code for this level.",
-                        WIN_CLOSE_ESC, view );
+        Mission *m = Gam->GetMission();
+        Audio::StopMusic( CF_MUSIC_FADE_TIME );
+        view->CloseWindow( this );
+
+        switch (mtypewidget->GetValue() ) {
+        case MAPTYPE_SINGLE:
+          if ( CFOptions.IsAI() ) {
+            string pnames;
+            pnames.assign( m->GetPlayer(PLAYER_ONE).Name() );
+            pnames += '|';
+            pnames += m->GetPlayer(PLAYER_TWO).Name();
+
+            DialogWindow *pwin = new DialogWindow( MSG(MSG_PLAYER_SELECTION),
+                  MSG(MSG_ASK_SIDE), pnames, 0, 0, view );
+            pwin->SetButtonID( 0, PLAYER_ONE );
+            pwin->SetButtonID( 1, PLAYER_TWO );
+            short rc = pwin->EventLoop();
+            view->CloseWindow( pwin );
+            m->GetPlayer( rc ).SetType( HUMAN );
+          } else {
+            m->GetPlayer(PLAYER_ONE).SetType( HUMAN );
+            m->GetPlayer(PLAYER_TWO).SetType( HUMAN );
+          }
+          break;
+        case MAPTYPE_CAMPAIGN:
+          m->GetPlayer(PLAYER_ONE).SetType( HUMAN );
+          break;
+        }
+
+        if ( mtypewidget->GetValue() != MAPTYPE_SAVE )
+          m->SetHandicap( 1 << diffwidget->GetValue() );
+
+        view->Flood( Color(CF_COLOR_SHADOW) );
+        view->Update();
+        Gam->InitWindows();
+        rc = Gam->StartTurn();         // let the games begin...
       }
     }
+    break;
+  case IW_WIDGET_MAPTYPE:
+    switch ( mtypewidget->GetValue() ) {
+    case MAPTYPE_SINGLE:
+      mapwidget->SetMap( 0, Rect(0,0,0,0), 0 );
+      mapwidget->SetSize( maxmap.x, maxmap.y, maxmap.w, maxmap.h );
+      levwidget->SwitchList( &levels, -1 );
+      gtypewidget->Enable();
+      diffwidget->Enable();
+      break;
+    case MAPTYPE_CAMPAIGN:
+      levwidget->SwitchList( &campaigns, -1 );
+      gtypewidget->Disable();
+      RemoveWidget( mapwidget );
+      AddWidget( campinfowidget );
+      CFOptions.SetCampaign( true );
+      break;
+    case MAPTYPE_SAVE:
+      mapwidget->SetMap( 0, Rect(0,0,0,0), 0 );
+      mapwidget->SetSize( maxmap.x, maxmap.y, maxmap.w, maxmap.h );
+      levwidget->SwitchList( &saves, -1 );
+      diffwidget->Disable();
+      RemoveWidget( campinfowidget );
+      AddWidget( mapwidget );
+      CFOptions.SetCampaign( false );
+      break;
+    }
+
+    Draw();
+    Show();
+    break;
+
+  case IW_WIDGET_OPTIONS:{
+    MenuWindow *menu = new MenuWindow( 0, this, view );
+    menu->AddItem( 0, IW_WIDGET_GENERAL_OPTIONS, 0, MSG(MSG_B_OPT_GENERAL) );
+    menu->AddItem( 0, IW_WIDGET_LANGUAGE_OPTIONS, 0, MSG(MSG_B_OPT_LANGUAGE) );
+    menu->AddItem( 0, IW_WIDGET_VIDEO_OPTIONS, 0, MSG(MSG_B_OPT_VIDEO) );
+#ifndef DISABLE_SOUND
+    menu->AddItem( 0, IW_WIDGET_SOUND_OPTIONS, 0, MSG(MSG_B_OPT_AUDIO) );
+#endif
+    menu->SetPosition(
+      Point( x + button->LeftEdge() + 1,
+             y + button->TopEdge() + button->Height() - 1 )
+    );
+    menu->SetMinWidth( button->Width() - 2 );
+    menu->Layout();
     break; }
-  case IW_WIDGET_LOAD_NEW:
-    startwidget->SetID( IW_WIDGET_START_NEW );
-    listctrlwidget->SetID( IW_WIDGET_LOAD_SAVED );
-    listctrlwidget->SetTitle( "Load saved game" );
-    listctrlwidget->Draw();
-    levwidget->SwitchList( &levels, -1 );
-    pbemwidget->Enable();
-    pbemwidget->Draw();
-    aiwidget->Enable();
-    aiwidget->Draw();
-    Show();
+
+  case IW_WIDGET_TYPE:
+    CFOptions.SetGameType( gtypewidget->GetValue() );
     break;
-  case IW_WIDGET_LOAD_SAVED:
-    startwidget->SetID( IW_WIDGET_START_SAVED );
-    listctrlwidget->SetID( IW_WIDGET_LOAD_NEW );
-    listctrlwidget->SetTitle( "Launch new game" );
-    listctrlwidget->Draw();
-    levwidget->SwitchList( &saves, -1 );
-    pbemwidget->Disable();
-    pbemwidget->Draw();
-    aiwidget->Disable();
-    aiwidget->Draw();
-    Show();
-    break;
-  case IW_WIDGET_PBEM:
-    CFOptions.SetPBEM( pbemwidget->Clicked() );
-    if ( pbemwidget->Clicked() ) {
-      CFOptions.SetComputerPlayer( false );
-      aiwidget->Disable();
-    } else aiwidget->Enable();
-    aiwidget->Draw();
-    Show();
-    break;
-  case IW_WIDGET_AI:
-    CFOptions.SetComputerPlayer( aiwidget->Clicked() );
-    if ( aiwidget->Clicked() ) {
-      CFOptions.SetPBEM( false );
-      pbemwidget->Disable();
-    } else pbemwidget->Enable();
-    pbemwidget->Draw();
-    Show();
+  case IW_WIDGET_GENERAL_OPTIONS:
+    view->CloseWindow( win );
+    new GeneralOptionsWindow( 0, view );
     break;
   case IW_WIDGET_VIDEO_OPTIONS:
+    view->CloseWindow( win );
     new VideoOptionsWindow( view );
     break;
+  case IW_WIDGET_LANGUAGE_OPTIONS:
+    view->CloseWindow( win );
+    new LocaleOptionsWindow( 0, view );
+    break;
+#ifndef DISABLE_SOUND
+  case IW_WIDGET_SOUND_OPTIONS:
+    view->CloseWindow( win );
+    new SoundOptionsWindow( view );
+    break;
+#endif
   }
 
-  return GUI_OK;
+  return rc;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -309,45 +368,105 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
 //              about the maps (password protection, 1 or 2 players).
 //              This information is attached to the list items.
 // PARAMETERS : list - list of files created using create_files_list()
-// RETURNS    : GUI status
+// RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void InitWindow::CompleteFilesList( TLWList &list ) const {
-  initLevInfo *info;
+void InitWindow::CompleteFilesList( TLWList &list ) {
+  bool reorder = false;
 
   for ( TLWNode *n = static_cast<TLWNode *>(list.Head()), *next; n; n = next ) {
     next = static_cast<TLWNode *>(n->Next());
-    string *fname = (string *)n->UserData();
-    bool ok = false;
+    string *fullname = (string *)n->UserData();
+    bool remove = false;
 
-    SDL_RWops *file = SDL_RWFromFile( fname->c_str(), "rb" );
-    if ( file ) {
-      unsigned long id;
-      unsigned short flags;
-      unsigned char ver;
+    Mission *m = LoadMission( fullname->c_str() );
+    if ( m ) {
+      string filename;
+      string title;
 
-      id = SDL_ReadLE32( file );
-      if ( id == FID_MISSION ) {
-        SDL_RWread( file, &ver, 1, 1 );
-        if ( ver == FILE_VERSION ) {
-          flags = SDL_ReadLE16( file );
-          info = new initLevInfo;
-          info->password = ((flags & GI_PASSWORD) != 0);
-          info->singleplayer = ((flags & GI_AI) != 0);
-          info->full_path = fname;
-          n->SetUserData( info );
-          ok = true;
+      filename.assign( n->Name() );
+      filename.erase( filename.size() - 4 );    // remove .sav/.lev
+
+      if ( (m->GetFlags() & GI_SAVEFILE) || !m->GetName() ) {
+        // saved game -> use file name
+        title = filename;
+      } else {
+        // new mission
+        // campaign maps are only available if they have been unlocked
+        // by playing the campaign
+        remove = (m->GetFlags() & GI_CAMPAIGN) &&  // campaign map
+                 (m->GetCampaignName() == 0)   &&  // not first map of a campaign
+                 CFOptions.IsLocked( filename );   // not played yet
+
+        if ( !remove ) {
+          if ( m->GetCampaignName() ) {
+            TLWNode *cnode = new TLWNode( m->GetCampaignName() );
+            string *full_path = new string(*fullname);
+            cnode->SetUserData( full_path );
+            campaigns.InsertNodeSorted( cnode );
+          }
+
+          title.assign( m->GetName() );
+          reorder = true;
         }
       }
-      SDL_RWclose( file );
-    }
 
-    if ( !ok ) {
+      if ( !remove ) {
+        title += ' ';
+        title += '(';
+        if ( m->GetFlags() & GI_PBEM ) title += MSG(MSG_TAG_PBEM);
+        else if ( m->GetFlags() & GI_SAVEFILE ) {
+          if ( !m->GetPlayer(PLAYER_ONE).IsHuman() ||
+               !m->GetPlayer(PLAYER_TWO).IsHuman() )
+            title += '1';
+          else title += '2';
+        } else title += ((m->GetFlags() & GI_AI) != 0 ? '1' : '2');
+
+        if ( m->GetFlags() & GI_SAVEFILE ) {
+          title += ", ";
+          title += MSG(MSG_TURN);
+          title += ' ';
+          title += StringUtil::tostring(m->GetTurn());
+        }
+        title += ')';
+        n->SetName( title );
+      }
+      delete m;
+    } else remove = true;
+
+    if ( remove ) {
       n->Remove();
       delete (string *)n->UserData();
       delete n;
     }
   }
+
+  if ( reorder ) list.Sort();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : InitWindow::LoadMission
+// DESCRIPTION: Load a mission into memory for inspection.
+// PARAMETERS : filename - mission filename
+// RETURNS    : mission if successful, NULL on error
+////////////////////////////////////////////////////////////////////////
+
+Mission *InitWindow::LoadMission( const char *filename ) const {
+  Mission *m = 0;
+
+  File file( filename );
+  if ( file.Open( "rb" ) ) {
+
+    m = new Mission();
+    if ( m->Load( file ) != -1 ) {
+      m->SetLocale( CFOptions.GetLanguage() );
+    } else {
+      delete m;
+      m = 0;
+    }
+  }
+
+  return m;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -369,7 +488,6 @@ TitleWindow::TitleWindow( View *view ) : Window( WIN_CENTER, view ) {
     Center( *view ); 
 
     Show();
-    EventLoop();
   }
 }
 
@@ -397,9 +515,7 @@ GUI_Status TitleWindow::HandleEvent( const SDL_Event &event ) {
 ////////////////////////////////////////////////////////////////////////
 
 VideoOptionsWindow::VideoOptionsWindow( View *view ) :
-     GenericOptionsWindow( view->LargeFont()->Width() * 13 + 30,
-                           view->SmallFont()->Height() * 11 + 30,
-                           "Video Options", view ) {
+     GenericOptionsWindow( MSG(MSG_OPTIONS_VIDEO), view ) {
   SDL_Rect std_sizes[] = { { 0, 0, 1280, 1024 }, { 0, 0, 1024, 768 },
                            { 0, 0, 800, 600}, { 0, 0, 640, 480 } };
   SDL_Rect *std_modes[5];
@@ -426,6 +542,12 @@ VideoOptionsWindow::VideoOptionsWindow( View *view ) :
     current = AddMode( &screen );
   }
 
+  // set window size
+  const char *fslabel = MSG(MSG_B_OPT_FULLSCREEN);
+  unsigned short width = sfont->TextWidth(fslabel) + DEFAULT_CBW_SIZE + 20;
+  if ( width < sfont->Width() * 15 ) width = sfont->Width() * 15;
+  SetLayout( width, sfont->Height() * 11 + 30 );
+
   // create widgets
   const Rect &b = GetBounds();
 
@@ -438,7 +560,7 @@ VideoOptionsWindow::VideoOptionsWindow( View *view ) :
                    modewidget->TopEdge() + modewidget->Height() + 10,
                    DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, view->IsFullScreen(),
                    WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
-                   "_Fullscreen", this );
+                   fslabel, this );
 
   Draw();
   Show();
@@ -537,26 +659,25 @@ GUI_Status VideoOptionsWindow::WidgetActivated( Widget *button, Window *win ) {
 
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : GenericOptionsWindow::GenericOptionsWindow
-// DESCRIPTION: Show an options menu.
+// NAME       : GenericOptionsWindow::SetLayout
+// DESCRIPTION: Set the size of the window. This must be called before
+//              trying to display it or using GetBounds().
 // PARAMETERS : w     - width (only the part needed by the subclass)
 //              h     - height (only the part needed by the subclass)
-//              title - window title
-//              view  - view to attach the window to
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-GenericOptionsWindow::GenericOptionsWindow( unsigned short w,
-       unsigned short h, const char *title, View *view ) :
-       Window( 0, 0, w + 10,
-               h + view->LargeFont()->Height() + view->SmallFont()->Height() +
-               25, WIN_CENTER, view ),
-       clientarea(5, 13 + lfont->Height(), Width() - 10,
-                  Height() - lfont->Height() - sfont->Height() - 25),
-       title(title) {
+void GenericOptionsWindow::SetLayout( unsigned short w, unsigned short h ) {
+  unsigned short tw = lfont->TextWidth(title);
+
+  if ( w < tw + 20 ) w = tw + 20;
+
+  clientarea = Rect( 5, 13 + lfont->Height(), w, h );
+
+  Window::SetSize( w + 10, h + lfont->Height() + sfont->Height() + 25 );
 
   Widget *wd = new ButtonWidget( B_ID_OK, 1, Height() - sfont->Height() - 9,
-                   Width() - 2, sfont->Height() + 8, WIDGET_DEFAULT, "_OK", this );
+                   Width() - 2, sfont->Height() + 8, WIDGET_DEFAULT, MSG(MSG_B_OK), this );
   wd->SetHook( this );
 }
 
@@ -572,39 +693,52 @@ void GenericOptionsWindow::Draw( void ) {
 
   DrawBox( clientarea, BOX_RECESSED );
 
-  lfont->Write( title, this, (w - lfont->TextWidth(title)) / 2 + 3,
-                             8, view->GetBGPen() );
-  lfont->Write( title, this, (w - lfont->TextWidth(title)) / 2,
-                             5, view->GetFGPen() );
+  int xpos = (w - lfont->TextWidth(title)) / 2;
+  lfont->Write( title, this, xpos + 3, 8, view->GetBGPen() );
+  lfont->Write( title, this, xpos, 5, view->GetFGPen() );
 }
 
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : GeneralOptionsWindow::GeneralOptionsWindow
 // DESCRIPTION: Show the general options menu.
-// PARAMETERS : mv   - current map view
+// PARAMETERS : mv   - current map view (may be NULL)
 //              view - view to attach the window to
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-GeneralOptionsWindow::GeneralOptionsWindow( MapView &mv, View *view ) :
-     GenericOptionsWindow( view->LargeFont()->Width() * 15 + 30,
-                           view->SmallFont()->Height() * 3 + 35,
-                           "General Options", view ), mv(mv) {
+GeneralOptionsWindow::GeneralOptionsWindow( MapView *mv, View *view ) :
+     GenericOptionsWindow( MSG(MSG_OPTIONS_GENERAL), view ), mv(mv) {
+  unsigned short widths[3] = { 25 + DEFAULT_CBW_SIZE,
+                               25 + DEFAULT_CBW_SIZE,
+                               35 + DEFAULT_CBW_SIZE }, maxw = 0;
+  const char *labels[3];
+
+  labels[0] = MSG(MSG_B_OPT_DAMAGE);
+  labels[1] = MSG(MSG_B_OPT_REPLAYS);
+  labels[2] = MSG(MSG_B_OPT_REPLAYS_QUICK);
+
+  for ( int i = 0; i < 3; ++i ) {
+    widths[i] += sfont->TextWidth(labels[i]);
+    if ( widths[i] > maxw ) maxw = widths[i];
+  }
+
+  SetLayout( maxw, sfont->Height() * 3 + 35 );
+
   // create widgets
   const Rect &b = GetBounds();
 
   // damage indicator
   diwidget = new CheckboxWidget( 0, b.x + 5, b.y + 5,
-               DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, mv.UnitStatsEnabled(),
+               DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, CFOptions.GetDamageIndicator(),
                WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
-               "Show _damage indicators", this );
+               labels[0], this );
 
   // replay
   repwidget = new CheckboxWidget( 0, b.x + 5, b.y + diwidget->Height() + 10,
               DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, CFOptions.GetTurnReplay(),
               WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
-              "Show _turn replays", this );
+              labels[1], this );
   repwidget->SetHook(this);
 
   // quick replays
@@ -612,7 +746,7 @@ GeneralOptionsWindow::GeneralOptionsWindow( MapView &mv, View *view ) :
               DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, CFOptions.GetQuickReplay(),
               WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT|
               (repwidget->Clicked() ? 0 : WIDGET_DISABLED),
-              "Show _battle results only", this );
+              labels[2], this );
   Draw();
   Show();
 }
@@ -643,15 +777,17 @@ GUI_Status GeneralOptionsWindow::WidgetActivated( Widget *widget, Window *win )
 
     view->CloseWindow( win );
 
-    if ( mv.UnitStatsEnabled() != diwidget->Clicked() ) {
+    if ( CFOptions.GetDamageIndicator() != diwidget->Clicked() ) {
       CFOptions.SetDamageIndicator( diwidget->Clicked() );
 
-      if ( diwidget->Clicked() ) mv.EnableUnitStats();
-      else mv.DisableUnitStats();
+      if ( mv ) {
+        if ( diwidget->Clicked() ) mv->EnableUnitStats();
+        else mv->DisableUnitStats();
 
-      // now redraw the map display
-      mv.Draw();
-      view->Refresh( mv );
+        // now redraw the map display
+        mv->Draw();
+        view->Refresh( *mv );
+      }
     }
 
     CFOptions.SetTurnReplay( repwidget->Clicked() );
@@ -661,6 +797,138 @@ GUI_Status GeneralOptionsWindow::WidgetActivated( Widget *widget, Window *win )
   return GUI_OK;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : LocaleOptionsWindow::LocaleOptionsWindow
+// DESCRIPTION: Show the language options menu.
+// PARAMETERS : game - current game. If it is NULL we assume the window
+//                     has been opened from the initial start window.
+//                     Otherwise we do an in-game language switch.
+//              view - view to attach the window to
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+LocaleOptionsWindow::LocaleOptionsWindow( Game *game, View *view ) :
+     GenericOptionsWindow( MSG(MSG_OPTIONS_LANGUAGE), view ), game(game) {
+
+  // collect language data files
+  short current = ReadLocales();
+  short found = locales.CountNodes();
+  if ( found > 5 ) found = 5;
+
+  SetLayout( sfont->Width() * 20 + 20, found * (sfont->Height() + 2) + 25 );
+
+  // create widgets
+  const Rect &b = GetBounds();
+
+  locwidget = new TextListWidget( 0, b.x + 5, b.y + 5, b.w - 10, b.h - 10,
+                  &locales, current,
+                  WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY|WIDGET_ALIGN_CENTER,
+                  NULL, this );
+
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : LocaleOptionsWindow::~LocaleOptionsWindow
+// DESCRIPTION: Free resources.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+LocaleOptionsWindow::~LocaleOptionsWindow( void ) {
+  while ( !locales.IsEmpty() ) {
+    TLWNode *n = static_cast<TLWNode *>(locales.RemHead());
+    delete (Language *)n->UserData();
+    delete n;
+  }
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : LocaleOptionsWindow::ReadLocales
+// DESCRIPTION: Get the list of locales.
+// PARAMETERS : -
+// RETURNS    : index of currently selected language
+////////////////////////////////////////////////////////////////////////
+
+short LocaleOptionsWindow::ReadLocales( void ) {
+
+  TLWList files;
+  FileWindow::CreateFilesList( get_locale_dir().c_str(),
+                               ".dat", files );
+  short i = 0, current = -1;
+
+  while ( !files.IsEmpty() ) {
+    TLWNode *n = static_cast<TLWNode *>(files.RemHead());
+
+    string *fname = (string *)n->UserData();
+
+    Language *l = new Language();
+    if ( l->ReadCatalog( fname->c_str() ) != -1 ) {
+      n->SetName( l->Name() );
+      n->SetUserData( l );
+      n->SetID( i );
+
+      locales.AddTail( n );
+
+      if ( string(l->ID()) == CFOptions.GetLanguage() )
+        current = i;
+
+      ++i;
+    } else {
+      delete l;
+      delete n;
+    }
+
+    delete fname;
+  }
+
+  return current;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : LocaleOptionsWindow::WidgetActivated
+// DESCRIPTION: When the user pushes the 'OK' button, propagate the
+//              selected options.
+// PARAMETERS : widget - calling widget (only OK)
+//              win    - pointer to active window
+// RETURNS    : GUI_OK
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status LocaleOptionsWindow::WidgetActivated( Widget *widget, Window *win ) {
+  GUI_Status rc = GUI_OK;
+
+  view->CloseWindow( win );
+
+  TLWNode *n = static_cast<TLWNode *>(locwidget->Selected());
+  if ( n ) {
+    Language *l = (Language *)n->UserData();
+    if ( string(l->ID()) != CFOptions.GetLanguage() ) {
+
+      Lang = *l;
+      CFOptions.SetLanguage( l->ID() );
+
+      if ( game ) {
+        // we've changed the language in-game:
+        // simply set the language and update the panel
+        game->InitKeys();
+        game->GetMission()->SetLocale( l->ID() );
+
+        MapWindow *mwin = game->GetMapWindow();
+        MapView *mv = mwin->GetMapView();
+        mwin->GetPanel()->Update( mv->GetMap()->GetMapObject(mv->Cursor()) );
+      } else {
+        // we've been called from the start menu: set the new language
+        // and start over. This way we don't need to update the individual
+        // widgets
+        rc = GUI_RESTART;
+      }
+    }
+  }
+
+  return rc;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : SoundOptionsWindow::SoundOptionsWindow
 // DESCRIPTION: Show the sound options menu.
@@ -676,10 +944,17 @@ const short SoundOptionsWindow::S_ID_VOL_SFX   = 12;
 const short SoundOptionsWindow::S_ID_VOL_MUSIC = 13;
 
 SoundOptionsWindow::SoundOptionsWindow( View *view ) :
-     GenericOptionsWindow( view->LargeFont()->Width() * 13 + 30,
-                           view->SmallFont()->Height() * 2 + 20,
-                           "Sound Options", view ),
+     GenericOptionsWindow( MSG(MSG_OPTIONS_AUDIO), view ),
      volgfx( view->GetSystemIcons(), 145, 46, 12 ,12 ) {
+  const char *sfxlabel = MSG(MSG_B_OPT_SFX);
+  const char *musiclabel = MSG(MSG_B_OPT_MUSIC);
+  unsigned short sfxw = sfont->TextWidth(sfxlabel);
+  unsigned short musw = sfont->TextWidth(musiclabel);
+
+  SetLayout( MAX(sfxw, musw) + DEFAULT_CBW_SIZE +
+             volgfx.Width() + 100,
+             view->SmallFont()->Height() * 2 + 20 );
+
   // create widgets
   const Rect &b = GetBounds();
   Widget *wd;
@@ -688,11 +963,11 @@ SoundOptionsWindow::SoundOptionsWindow( View *view ) :
   wd = new CheckboxWidget( B_ID_SFX, b.x + 5, b.y + 5,
            DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, Audio::GetSfxState(),
            WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
-           "_SFX", this );
+           sfxlabel, this );
   wd->SetHook( this );
 
   // sfx volume
-  short xoff = wd->LeftEdge() + wd->Width() + sfont->TextWidth("Music") +
+  short xoff = wd->LeftEdge() + wd->Width() + MAX(sfxw, musw) +
                volgfx.Width() + 15;
   sfxvol = new SliderWidget( S_ID_VOL_SFX,
            xoff, wd->TopEdge() + (wd->Height() - DEFAULT_SLIDER_SIZE)/2,
@@ -707,15 +982,16 @@ SoundOptionsWindow::SoundOptionsWindow( View *view ) :
   wd = new CheckboxWidget( B_ID_MUSIC,
            wd->LeftEdge(), wd->TopEdge() + wd->Height() + 5,
            DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, Audio::GetMusicState(),
-           WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT|WIDGET_DISABLED,
-           "_Music", this );
+           WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
+           musiclabel, this );
   wd->SetHook( this );
 
   // music volume
   musicvol = new SliderWidget( S_ID_VOL_MUSIC,
            xoff, wd->TopEdge() + (wd->Height() - DEFAULT_SLIDER_SIZE)/2,
            b.w - xoff - 5, DEFAULT_SLIDER_SIZE, 0, MIX_MAX_VOLUME,
-           0, 20, WIDGET_HSCROLL|WIDGET_VSCROLLKEY|WIDGET_DISABLED,
+           Audio::GetMusicVolume(), 20, WIDGET_HSCROLL|WIDGET_VSCROLLKEY|
+             (Audio::GetMusicState() ? 0 : WIDGET_DISABLED),
            NULL, this );
   musicvol->SetHook( this );
 
