@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,25 +23,10 @@
 #ifndef _INCLUDE_INITWINDOW_H
 #define _INCLUDE_INITWINDOW_H
 
-#include <string>
-using namespace std;
-
 #include "window.h"
 #include "textbox.h"
 #include "button.h"
-#include "mapview.h"
-
-class LevListWidget : public TextListWidget {
-public:
-  LevListWidget( short id, short x, short y, unsigned short w,
-    unsigned short h, List *list, short selected,
-    unsigned short flags, const char *title, Window *window ) :
-    TextListWidget( id, x, y, w, h, list, selected, flags, title, window ) {};
-
-  void PrintItem( const TLWNode *item, Surface *dest,
-                  short x, short y, const Rect &clip ) const;
-};
-
+#include "mapwindow.h"
 
 class TitleWindow : public Window {
 public:
@@ -56,21 +41,24 @@ public:
   InitWindow( View *view, Window *title );
   ~InitWindow( void );
 
-  void Draw( void );
   void Close( void ) { Window::Close(); view->CloseWindow(title); }
   GUI_Status WidgetActivated( Widget *button, Window *win );
 
 private:
-  void CompleteFilesList( TLWList &list ) const;
+  void CompleteFilesList( TLWList &list );
+  Mission *LoadMission( const char *filename ) const;
 
+  CycleWidget *gtypewidget;
+  CycleWidget *mtypewidget;
+  CycleWidget *diffwidget;
   TextListWidget *levwidget;
-  ButtonWidget *startwidget;
-  ButtonWidget *listctrlwidget;
-  ButtonWidget *videowidget;
-  CheckboxWidget *pbemwidget;
-  CheckboxWidget *aiwidget;
+  MapWidget *mapwidget;
+  TextScrollWidget *campinfowidget;
+
+  Rect maxmap;
 
   TLWList levels;
+  TLWList campaigns;
   TLWList saves;
 
   Window *title;
@@ -78,26 +66,27 @@ private:
 
 class GenericOptionsWindow : public Window, public WidgetHook {
 public:
-  GenericOptionsWindow( unsigned short w, unsigned short h,
-                        const char *title, View *view );
+  GenericOptionsWindow( const char *title, View *view ) :
+                Window( WIN_CENTER, view ), title(title) {}
 
   virtual void Draw( void );
 
 protected:
   virtual GUI_Status WidgetActivated( Widget *widget, Window *win ) = 0;
+  void SetLayout( unsigned short w, unsigned short h );
 
   const Rect &GetBounds( void ) const { return clientarea; }
 
   static const short B_ID_OK;
 
 private:
-  const Rect clientarea;  // the part subclasses may draw into
+  Rect clientarea;  // the part subclasses may draw into
   const char *title;
 };
 
 class GeneralOptionsWindow : public GenericOptionsWindow {
 public:
-  GeneralOptionsWindow( MapView &mv, View *view );
+  GeneralOptionsWindow( MapView *mv, View *view );
 
 private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
@@ -105,7 +94,7 @@ private:
   CheckboxWidget *diwidget;  // damage indicators
   CheckboxWidget *repwidget; // turn replays
   CheckboxWidget *qrepwidget; // quick turn replays
-  MapView &mv;
+  MapView *mv;
 };
 
 class VideoOptionsWindow : public GenericOptionsWindow {
@@ -123,6 +112,22 @@ private:
   List modes;
 };
 
+class Game;
+class LocaleOptionsWindow : public GenericOptionsWindow {
+public:
+  LocaleOptionsWindow( Game *game, View *view );
+  ~LocaleOptionsWindow( void );
+
+private:
+  GUI_Status WidgetActivated( Widget *widget, Window *win );
+
+  short ReadLocales( void );
+
+  TLWList locales;
+  TextListWidget *locwidget;
+  Game *game;
+};
+
 # ifndef DISABLE_SOUND
 class SoundOptionsWindow : public GenericOptionsWindow {
 public:
