@@ -1,5 +1,5 @@
 // Crimson Fields -- a hex-based game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,14 +22,14 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <iostream>
+#include <fstream>
 
 #ifndef _WIN32_WCE
 # include <time.h>
-# include <fstream.h>
 #endif
 
 #include "SDL.h"
-#include "SDL_endian.h"
 
 #include "view.h"
 #include "game.h"
@@ -39,10 +39,13 @@
 #include "globals.h"
 #include "sound.h"
 #include "options.h"
+#include "strutil.h"
+#include "msgs.h"
 
 // global vars
 Game *Gam;
 Image *Images[NUM_IMAGES] = { NULL };
+Language Lang;
 
 // global options, can be accessed from everywhere
 Options CFOptions;
@@ -66,6 +69,7 @@ static View *display;
 static void parse_options( int argc, char **argv, GUIOptions &opts );
 static void print_usage( char *prog );
 static View *init( GUIOptions &opts );
+static bool init_locale( void );
 static void load_settings( GUIOptions &opts );
 static void save_settings( View *display );
 static void set_icon( const Surface &s, const Rect &icon );
@@ -101,11 +105,14 @@ int main( int argc, char **argv ) {
     if ( guiopts.level ) {
       Gam = new Game( display );
       intro = Gam->Load( guiopts.level );
-      if ( !intro &&
-           (!Gam->GetLevelPassword() ||
-             Gam->CheckPassword( "Access restricted", "Enter access code",
-                                 Gam->GetLevelPassword(), false )) ) Gam->StartTurn();
-      else {
+      Mission *m = Gam->GetMission();
+      if ( !intro ) {
+        // default is to play single-player single-map
+        if ( !m->GetFlags() & GI_SAVEFILE )
+          m->GetPlayer(PLAYER_ONE).SetType( HUMAN );
+        Gam->InitWindows();
+        Gam->StartTurn();
+      } else {
         delete Gam;
         Gam = NULL;
         intro = 1;
@@ -117,7 +124,18 @@ int main( int argc, char **argv ) {
       if ( intro ) {
         display->Refresh();
         TitleWindow *twin = new TitleWindow( display );
-        new InitWindow( display, twin );
+
+        // preload main window, levels, etc.
+        InitWindow *iwin = new InitWindow( display, twin );
+
+        // if title image was successfully loaded, wait for click
+        if ( twin->Width() > 0 ) {
+          display->SelectWindow( twin );
+          twin->EventLoop();
+          display->SelectWindow( iwin );
+        }
+
+        iwin->Show();
       } else intro = 1;
 
       do {
@@ -167,7 +185,7 @@ void parse_options( int argc, char **argv, GUIOptions &opts ) {
       else opts.sfx = opts.music = false;
     } else {
       if (strcmp(argv[argc], "--version") == 0)
-        fprintf( stdout, PROGRAMNAME" "VERSION"\n" );
+        cout << PROGRAMNAME" "VERSION << endl;
       else print_usage( argv[0] );
       exit ( 0 );
     }
@@ -185,26 +203,25 @@ void parse_options( int argc, char **argv, GUIOptions &opts ) {
 ////////////////////////////////////////////////////////////////////////
 
 void print_usage( char *prog ) {
-  fprintf( stdout, "Usage: %s [options]\n\n"
-                   "Available options:\n"
-                   "  --level <level>      load level or save file\n"
-                   "  --width <width>      set screen width\n"
-                   "  --height <height>    set screen height\n"
-                   "  --fullscreen <1|0>   enable/disable fullscreen mode\n"
+  cout << "Usage: " << prog << " [options]" << endl << endl
+            << "Available options:" << endl
+            << "  --level <level>      load level or save file" << endl
+            << "  --width <width>      set screen width" << endl
+            << "  --height <height>    set screen height" << endl
+            << "  --fullscreen <1|0>   enable/disable fullscreen mode" << endl
 #ifndef DISABLE_SOUND
-                   "  --sound <1|0>        enable/disable sound\n"
+            << "  --sound <1|0>        enable/disable sound" << endl
 #endif
-                   "  --help               display this help and exit\n"
-                   "  --version            output version information and exit\n",
-        prog );
+            << "  --help               display this help and exit" << endl
+            << "  --version            output version information and exit" << endl;
 }
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : init
 // DESCRIPTION: Initialize the display, the system icons surface and the
 //              game fonts.
-// PARAMETERS : opts  - contains the user settings for the sound and
-//                      graphics subsystems
+// PARAMETERS : opts - contains the user settings for the sound and
+//                     graphics subsystems
 // RETURNS    : a pointer to the display surface on success, NULL
 //              otherwise
 ////////////////////////////////////////////////////////////////////////
@@ -216,10 +233,14 @@ View *init( GUIOptions &opts ) {
   bool ok = false;
 
   if ( SDL_Init( SDL_INIT_VIDEO ) >= 0 ) {
+
+    // load locale
+    if ( !init_locale() ) return NULL;
+
     string datpath( get_data_dir() );
     datpath.append( CF_DATFILE );
-    SDL_RWops *datfile = SDL_RWFromFile( datpath.c_str(), "rb" );
-    if ( datfile ) {
+    File datfile( datpath );
+    if ( datfile.Open( "rb" ) ) {
 
       icons = new Surface;		// load icons surface
       if ( !icons->LoadImageData( datfile ) ) {
@@ -228,18 +249,20 @@ View *init( GUIOptions &opts ) {
 
         view = new View( opts.px_width, opts.px_height, opts.bpp,
                          opts.sdl_flags );
-        if ( view->s_surface ) {
-          f1 = new Font( datfile );	// load fonts
-          f2 = new Font( datfile );
-          ok = true;
-        } else fprintf( stderr, "Couldn't set video mode: %s\n", SDL_GetError() );
+        if ( view->s_surface && !TTF_Init()) {	// load fonts
 
-      } else fprintf( stderr, "Error reading data file\n" );
+          datpath = get_data_dir() + CF_FONT;
+          f1 = new Font();
+          f2 = new Font();
+          if ( !f1->Load( datpath.c_str(), CF_FONT_SMALL ) &&
+               !f2->Load( datpath.c_str(), CF_FONT_LARGE ) )
+            ok = true;
+          else cerr << "Error: Couldn't load font " << datpath << endl;
+        } else cerr << "Error: Couldn't set video mode (" << SDL_GetError() << ')' << endl;
 
-      SDL_RWclose( datfile );
-
-    } else fprintf( stderr, "Error: Couldn't open '%s'\n", datpath.c_str() );
-  } else fprintf( stderr, "Couldn't initialize: %s\n", SDL_GetError() );
+      } else cerr << "Error: Couldn't read data file" << endl;
+    } else cerr << "Error: Couldn't open " << datpath << endl;
+  } else cerr << "Error: Couldn't initialize ( " << SDL_GetError() << ')' << endl;
 
   if ( ok ) {
     // set main window title
@@ -270,10 +293,57 @@ View *init( GUIOptions &opts ) {
   } else {
     delete icons;
     delete view;
+    delete f1;
+    delete f2;
     return NULL;
   }
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : init_locale
+// DESCRIPTION: Load the strings from a file.
+// PARAMETERS : -
+// RETURNS    : TRUE on success, FALSE otherwise
+////////////////////////////////////////////////////////////////////////
+
+bool init_locale( void ) {
+  bool rc = false, retry = false;
+
+  const string ldir( get_locale_dir() );
+  string lid( CFOptions.GetLanguage() );
+
+  // try the user locale first, and use the default if that fails
+  do {
+    string lfile( ldir + lid + ".dat" );
+    short num = Lang.ReadCatalog( lfile.c_str() );
+    if ( num == -1 ) {
+      cerr << "Error: Couldn't load language resources for language '"
+                << lid << "'" << endl;
+    } else if ( num != CF_MSGS ) {
+      cerr << "Error: Language catalog for '" << lid << "' contains "
+                << num << " strings, expected " << CF_MSGS << endl;
+    } else rc = true;
+
+    // only retry once
+    if ( (rc == false) && (retry == false) &&
+         (lid != CF_LANG_DEFAULT) ) {
+      lid.assign( CF_LANG_DEFAULT );
+      cerr << "Using default locale (" << lid << ") instead" << endl;
+      retry = true;
+    } else {
+      retry = false;
+
+      if ( rc == true ) {
+        // change the language setting if necessary
+        if ( lid != CFOptions.GetLanguage() )
+          CFOptions.SetLanguage( lid.c_str() );
+      }
+    }
+  } while ( retry );
+
+  return rc;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : load_settings
 // DESCRIPTION: Read default display settings from the crimsonrc file.
@@ -288,16 +358,21 @@ void load_settings( GUIOptions &opts ) {
   string crimsonrc( get_config_dir() );
   crimsonrc.append( CRIMSONRC );
 
-  FILE *fp = fopen( crimsonrc.c_str(), "r" );
-  if ( fp ) {
-    char linebuf[256], *val;
-    unsigned short linecnt = 0;
+  ifstream file( crimsonrc.c_str() );
+  if ( file.is_open() ) {
+    string buf;
+    const char *val, *linebuf;
+    unsigned int pos, line = 0;
 
-    while ( fgets( linebuf, 255, fp ) ) {
-      ++linecnt;
-      if ( (linebuf[0] != '#') && (linebuf[0] != '\n') ) {
-        val = strchr( linebuf, ' ' );
-        if ( val ) {
+    while (!file.eof()) {
+      getline(file, buf);
+      ++line;
+
+      if ( buf.size() > 0 ) { // ignore empty lines
+        pos = buf.find( ' ' );
+        if ( pos != string::npos ) {
+          linebuf = buf.c_str();
+          val = &linebuf[pos+1];
           while ( *val == ' ' ) ++val;
 
           if ( !strncmp( linebuf, "width", 5 ) ) opts.px_width = atoi(val);
@@ -308,16 +383,18 @@ void load_settings( GUIOptions &opts ) {
           else if ( !strncmp( linebuf, "musicvol", 8 ) ) opts.music_vol = atoi(val);
           else if ( !strncmp( linebuf, "sfx", 3 ) ) opts.sfx = (atoi(val) != 0);
           else if ( !strncmp( linebuf, "music", 5 ) ) opts.music = (atoi(val) != 0);
+          else if ( !strncmp( linebuf, "locale", 6 ) ) CFOptions.SetLanguage(val);
           else if ( !strncmp( linebuf, "showdamage", 10 ) ) CFOptions.SetDamageIndicator( atoi(val) != 0 );
+          else if ( !strncmp( linebuf, "unlock", 6 ) ) CFOptions.Unlock( val );
           else if ( !strncmp( linebuf, "showreplay", 10 ) ) {
             int rep = atoi(val);
             CFOptions.SetTurnReplay( rep != 0 );
             CFOptions.SetQuickReplay( rep == 1 );
-          } else fprintf( stderr, "warning: unrecognized config option in line %d\n", linecnt );
+          } else cerr << "Warning: unrecognized config option in line " << line << endl;
         }
       }
     }
-    fclose( fp );
+    file.close();
   }
 }
 
@@ -332,19 +409,25 @@ void save_settings( View *display ) {
   string crimsonrc( get_config_dir() );
   crimsonrc.append( CRIMSONRC );
 
-  FILE *fp = fopen( crimsonrc.c_str(), "w" );
-  if ( fp ) {
-    fprintf( fp, "width %d\n", display->Width() );
-    fprintf( fp, "height %d\n", display->Height() );
-    fprintf( fp, "fullscreen %d\n", display->IsFullScreen() );
-    fprintf( fp, "sfx %d\n", Audio::GetSfxState() );
-    fprintf( fp, "music %d\n", Audio::GetMusicState() );
-    fprintf( fp, "sfxvol %d\n", Audio::GetSfxVolume() );
-    fprintf( fp, "musicvol %d\n", Audio::GetMusicVolume() );
-    fprintf( fp, "showdamage %d\n", CFOptions.GetDamageIndicator() );
-    fprintf( fp, "showreplay %d\n",
-      CFOptions.GetTurnReplay() ? (CFOptions.GetQuickReplay() ? 1 : 2) : 0 );
-    fclose( fp );
+  ofstream file( crimsonrc.c_str() );
+  if ( file.is_open() ) {
+    file << "width " << display->Width() << '\n';
+    file << "height " << display->Height() << '\n';
+    file << StringUtil::strprintf("fullscreen %d", display->IsFullScreen()) << '\n';
+    file << StringUtil::strprintf("sfx %d", Audio::GetSfxState()) << '\n';
+    file << StringUtil::strprintf("music %d", Audio::GetMusicState()) << '\n';
+    file << StringUtil::strprintf("sfxvol %d", Audio::GetSfxVolume()) << '\n';
+    file << StringUtil::strprintf("musicvol %d", Audio::GetMusicVolume()) << '\n';
+    file << "locale " << CFOptions.GetLanguage() << '\n';
+    file << StringUtil::strprintf("showdamage %d", CFOptions.GetDamageIndicator()) << '\n';
+    file << StringUtil::strprintf("showreplay %d",
+            CFOptions.GetTurnReplay() ? (CFOptions.GetQuickReplay() ? 1 : 2) : 0 ) << '\n';
+
+    const vector<string> &maps = CFOptions.GetUnlockedMaps();
+    for ( vector<string>::const_iterator i = maps.begin(); i != maps.end(); ++i )
+      file << "unlock " << *i << '\n';
+
+    file.close();
   }
 }
 
@@ -433,16 +516,19 @@ bool init_wince( struct GUIOptions &opts ) {
 ////////////////////////////////////////////////////////////////////////
 
 void do_exit( void ) {
-  Audio::ShutdownSfx();
-  Audio::ShutdownMusic();
+  delete Gam;
 
   if ( display ) {
     save_settings( display );
     delete display;
   }
 
+  Audio::ShutdownSfx();
+  Audio::ShutdownMusic();
+
   for ( int i = 0; i < NUM_IMAGES; ++i ) delete Images[i];
 
+  TTF_Quit();
   SDL_Quit();
   exit( 0 );
 }
