@@ -22,6 +22,7 @@
 
 #include "mapwindow.h"
 #include "game.h"
+#include "msgs.h"
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapWindow::MapWindow
@@ -396,7 +397,7 @@ void Panel::Update( MapObject *obj ) {
 
     // resize and reposition if necessary
     w = sfont->TextWidth( obj->Name() ) + 10;
-    if ( obj->IsUnit() ) w += 15 + XP_ICON_WIDTH + sfont->Width() * 3;
+    if ( obj->IsUnit() ) w += 5 + XP_ICON_WIDTH + sfont->Width() * 3;
     else {
       Building *b = static_cast<Building *>(obj);
       if ( b->IsWorkshop() ) w += 5 + XP_ICON_WIDTH;
@@ -460,12 +461,14 @@ void Panel::VideoModeChange( void ) {
 // NAME       : TacticalWindow::TacticalWindow
 // DESCRIPTION: Create a window and show an overview map of the level.
 // PARAMETERS : mapview - pointer to map viewport
+//              m       - mission object
 //              view    - pointer to the window's view
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-TacticalWindow::TacticalWindow( MapView *mapview, View *view ) :
-        Window( WIN_CLOSE_ESC|WIN_CENTER, view ), mv(mapview)  {
+TacticalWindow::TacticalWindow( MapView *mapview, Mission &m, View *view ) :
+        Window( WIN_CLOSE_ESC|WIN_CENTER, view ), mv(mapview), mission(m),
+        p1(m.GetPlayer(PLAYER_ONE)), p2(m.GetPlayer(PLAYER_TWO)) {
   Map *pmap = mv->GetMap();
 
   Rect win, map, viewport;
@@ -480,7 +483,13 @@ TacticalWindow::TacticalWindow( MapView *mapview, View *view ) :
                view->Height() - sfont->Height() * 3 - 40 );
 
   // calculate window dimensions
-  win.w = MAX( map.w + 20, sfont->Width() * 35 + 10 );
+  const char *ulabel = MSG(MSG_UNITS);
+  const char *slabel = MSG(MSG_SHOPS);
+  unsigned short ulen = sfont->TextWidth( ulabel );
+  unsigned short slen = sfont->TextWidth( slabel );
+  unsigned short labellen = MAX( ulen, slen );
+
+  win.w = MAX( map.w + 20, labellen + sfont->Width() * 12 + 30 );
   win.h = MIN( map.h + sfont->Height() * 3 + 40, view->h );
   win.Center( *view );
   win.Align( *view );
@@ -493,12 +502,11 @@ TacticalWindow::TacticalWindow( MapView *mapview, View *view ) :
 
   new ButtonWidget( GUI_CLOSE, 1, h - sfont->Height() - 9,
                     w - 2, sfont->Height() + 8,
-                    WIDGET_DEFAULT, "_OK", this );
+                    WIDGET_DEFAULT, MSG(MSG_B_OK), this );
 
   mw = new MapWidget( 0, map.x, map.y, map.w, map.h,
                       WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY, this );
-  mw->SetPlayerColors( Gam->GetPlayer(PLAYER_ONE)->LightColor(),
-                       Gam->GetPlayer(PLAYER_TWO)->LightColor() );
+  mw->SetPlayerColors( p1.LightColor(), p2.LightColor() );
   mw->SetMap( pmap, viewport, magnify );
   mw->SetHook( this );
 
@@ -542,12 +550,12 @@ void TacticalWindow::Draw( void ) {
   Window::Draw();
 
   // show unit and building count
-  Color col, colbg;
-  unsigned short linewidth = sfont->Width() * 20, lx, l1y, l2y, lh;
+  Color col;
+  unsigned short linewidth = w - 40, lx, l1y, l2y, lh, xoff1, xoff2;
   char numbuf[4];
 
   unsigned short bs1 = 0, bs2 = 0, bstotal = 0;
-  Building *b = static_cast<Building *>( Gam->BuildingsList()->Head() );
+  Building *b = static_cast<Building *>( mission.GetShops().Head() );
   while ( b ) {
     if ( b->Owner() ) {
       if ( b->Owner()->ID() == PLAYER_ONE ) ++bs1;
@@ -566,25 +574,21 @@ void TacticalWindow::Draw( void ) {
            BOX_RECESSED );
 
   lx = (w - linewidth - 10) / 2;
-  sfont->Write( "Units", this, lx, l1y );
-  sfont->Write( "Buildings", this, lx, l2y );
+  xoff1 = sfont->Write( MSG(MSG_UNITS), this, lx, l1y );
+  xoff2 = sfont->Write( MSG(MSG_SHOPS), this, lx, l2y );
 
-  lx += sfont->Width() * 10;
-  col = Gam->GetPlayer(PLAYER_ONE)->LightColor();
-  colbg = Gam->GetPlayer(PLAYER_ONE)->DarkColor();
-  sfont->Write( itoa( Gam->GetPlayer(PLAYER_ONE)->Units(0), numbuf ), this,
-                lx, l1y, col );
-  sfont->Write( itoa( bs1, numbuf ), this, lx, l2y, col );
+  lx += MAX( xoff1, xoff2 ) + 20;
+  col = p1.LightColor();
+  xoff1 = sfont->Write( itoa( p1.Units(0), numbuf ), this, lx, l1y, col );
+  xoff2 = sfont->Write( itoa( bs1, numbuf ), this, lx, l2y, col );
 
-  lx += sfont->Width() * 4;
-  col = Gam->GetPlayer(PLAYER_TWO)->LightColor();
-  colbg = Gam->GetPlayer(PLAYER_TWO)->DarkColor();
-  sfont->Write( itoa( Gam->GetPlayer(PLAYER_TWO)->Units(0), numbuf ), this,
-                lx, l1y, col );
-  sfont->Write( itoa( bs2, numbuf ), this, lx, l2y, col );
+  lx += MAX( xoff1, xoff2 ) + 20;
+  col = p2.LightColor();
+  xoff1 = sfont->Write( itoa( p2.Units(0), numbuf ), this, lx, l1y, col );
+  xoff2 = sfont->Write( itoa( bs2, numbuf ), this, lx, l2y, col );
 
-  lx += sfont->Width() * 4;
-  sfont->Write( itoa( Gam->UnitsList()->CountNodes(), numbuf ), this, lx, l1y );
+  lx += MAX( xoff1, xoff2 ) + 20;
+  sfont->Write( itoa( mission.GetUnits().CountNodes(), numbuf ), this, lx, l1y );
   sfont->Write( itoa( bstotal, numbuf ), this, lx, l2y );
 }
 
@@ -603,3 +607,135 @@ void TacticalWindow::CalcViewPort( Rect &vp ) const {
   vp.h = (mv->MaxYHex(-1,0) + 1) - vp.y;
 }
 
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CombatWindow::CombatWindow
+// DESCRIPTION: The CombatWindow visualizes the fight between two units.
+// PARAMETERS : combat - pointer to a combat object
+//              mapwin - pointer to the map window; required to update
+//                       the display when a unit is destroyed
+//              view   - view the window will belong to
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+CombatWindow::CombatWindow( Combat *combat, MapWindow *mapwin, View *view ) :
+  Window( 0, view ), mapwin( mapwin ), att( combat->GetAttacker() ),
+    def( combat->GetDefender() ), apos( att->Position() ), dpos( def->Position() ),
+    startgroup1( att->GroupSize() ), startgroup2( def->GroupSize() ) {
+
+  // calculate window dimensions
+  short wwidth = (MAX( sfont->TextWidth( att->Name() ),
+                       sfont->TextWidth( def->Name() ) ) + XP_ICON_WIDTH + 25) * 2;
+  wwidth = MAX( wwidth, GFX_WIDTH * 6 + 50 );
+  Rect win( 0, 0, wwidth, GFX_HEIGHT * 3 + sfont->Height() * 2 + 50 );
+  MapView *mv = mapwin->GetMapView();
+
+  // place window on map window
+  mv->CenterOnHex( Point( (apos.x + dpos.x) / 2, (apos.y + dpos.y) / 2 ) );
+  mapwin->BoxAvoidHexes( win, apos, dpos );
+  SetSize( win.x, win.y, win.w, win.h );
+
+  // highlight the fighting units
+  Rect clip;
+  Point punit = mv->Hex2Pixel( apos );
+  clip = Rect( punit.x, punit.y, GFX_WIDTH, GFX_HEIGHT );
+  clip.Clip( *mv );
+  mv->DrawTerrain( IMG_CURSOR_HIGHLIGHT, mapwin, punit.x, punit.y, clip );
+  punit = mv->Hex2Pixel( dpos );
+  clip = Rect( punit.x, punit.y, GFX_WIDTH, GFX_HEIGHT );
+  clip.Clip( *mv );
+  mv->DrawTerrain( IMG_CURSOR_HIGHLIGHT, mapwin, punit.x, punit.y, clip );
+
+  // create button widget, message areas, and unit display areas
+  Rect brect( 1, h - sfont->Height() - 10, w - 2, sfont->Height() + 10 );
+
+  msgbar1 = Rect( 4, h - sfont->Height() - 14 - brect.h, (w - 16) / 2, sfont->Height() + 6 );
+  msgbar2 = Rect( msgbar1.x + msgbar1.w + 8, msgbar1.y, msgbar1.w, msgbar1.h );
+
+  att_anchor = Rect( w/4 - GFX_WIDTH/2, 10 + GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
+  def_anchor = Rect( (w * 3)/4 - GFX_WIDTH/2, att_anchor.y, GFX_WIDTH, GFX_HEIGHT );
+
+  clock[0][0] = Rect( att_anchor.x, att_anchor.y - GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
+  clock[0][1] = Rect( att_anchor.x + GFX_WIDTH - GFX_OVERLAP_X, att_anchor.y - GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+  clock[0][2] = Rect( att_anchor.x + GFX_WIDTH - GFX_OVERLAP_X, att_anchor.y + GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+  clock[0][3] = Rect( att_anchor.x, att_anchor.y + GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
+  clock[0][4] = Rect( att_anchor.x - GFX_WIDTH + GFX_OVERLAP_X, att_anchor.y + GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+  clock[0][5] = Rect( att_anchor.x - GFX_WIDTH + GFX_OVERLAP_X, att_anchor.y - GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+  clock[1][0] = Rect( def_anchor.x, def_anchor.y - GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
+  clock[1][1] = Rect( def_anchor.x + GFX_WIDTH - GFX_OVERLAP_X, def_anchor.y - GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+  clock[1][2] = Rect( def_anchor.x + GFX_WIDTH - GFX_OVERLAP_X, def_anchor.y + GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+  clock[1][3] = Rect( def_anchor.x, def_anchor.y + GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
+  clock[1][4] = Rect( def_anchor.x - GFX_WIDTH + GFX_OVERLAP_X, def_anchor.y + GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+  clock[1][5] = Rect( def_anchor.x - GFX_WIDTH + GFX_OVERLAP_X, def_anchor.y - GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+
+  button = new ButtonWidget( GUI_CLOSE, brect.x, brect.y, brect.w, brect.h, WIDGET_DEFAULT, MSG(MSG_B_OK), this );
+
+  Draw();
+  view->Refresh();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CombatWindow::Draw
+// DESCRIPTION: Draw the window.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void CombatWindow::Draw( void ) {
+  Window::Draw();
+  DrawBox( msgbar1, BOX_RECESSED );
+  DrawBox( msgbar2, BOX_RECESSED );
+
+  DrawBox( Rect( msgbar1.x, 4, msgbar1.w, msgbar1.y - 8 ), BOX_CARVED );
+  DrawBox( Rect( msgbar2.x, 4, msgbar2.w, msgbar2.y - 8 ), BOX_CARVED );
+
+  Images[ICON_XP_BASE + att->XPLevel()]->Draw( this, msgbar1.x + 4, msgbar1.y + (msgbar1.h - XP_ICON_HEIGHT)/2 );
+  Images[ICON_XP_BASE + def->XPLevel()]->Draw( this, msgbar2.x + 4, msgbar2.y + (msgbar2.h - XP_ICON_HEIGHT)/2 );
+
+  sfont->Write( att->Name(), this, msgbar1.x + XP_ICON_WIDTH + 8,
+                msgbar1.y + (msgbar1.h - sfont->Height())/2 );
+  sfont->Write( def->Name(), this, msgbar2.x + XP_ICON_WIDTH + 8,
+                msgbar2.y + (msgbar2.h - sfont->Height())/2 );
+
+  // draw terrain and unit image to the center of the 'clock'
+  MapView *mv = mapwin->GetMapView();
+  Map *map = mv->GetMap();
+  mv->DrawTerrain( map->HexImage( apos ), this, att_anchor.x, att_anchor.y, att_anchor );
+  mv->DrawTerrain( map->HexImage( dpos ), this, def_anchor.x, def_anchor.y, def_anchor );
+  mv->DrawUnit( att->Image(), this, att_anchor.x, att_anchor.y, att_anchor );
+  mv->DrawUnit( def->Image(), this, def_anchor.x, def_anchor.y, def_anchor );
+
+  DrawState();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CombatWindow::DrawState
+// DESCRIPTION: Update the two 'clocks' to display the current standing.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void CombatWindow::DrawState( void ) {
+  short group1 = att->GroupSize(), group2 = def->GroupSize();
+  MapView *mv = mapwin->GetMapView();
+
+  for ( int i = NORTH; i <= NORTHWEST; ++i ) {
+
+    if ( i < startgroup1 ) {
+      mv->DrawTerrain( IMG_RECESSED_HEX, this, clock[0][i].x, clock[0][i].y, clock[0][i] );
+      mv->DrawUnit( att->BaseImage() + i, this, clock[0][i].x, clock[0][i].y, clock[0][i] );
+
+      if ( i >= group1 )
+        mv->DrawFog( this, clock[0][i].x, clock[0][i].y, clock[0][i] );
+    }
+
+    if ( i < startgroup2 ) {
+      mv->DrawTerrain( IMG_RECESSED_HEX, this, clock[1][i].x, clock[1][i].y, clock[1][i] );
+      mv->DrawUnit( def->BaseImage() + i, this, clock[1][i].x, clock[1][i].y, clock[1][i] );
+
+      if ( i >= group2 )
+        mv->DrawFog( this, clock[1][i].x, clock[1][i].y, clock[1][i] );
+    }
+  }
+}
+
