@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,30 +27,48 @@
 #ifndef _INCLUDE_OPTIONS_H
 #define _INCLUDE_OPTIONS_H
 
+#include <algorithm>
+#include "lang.h"
 
 class Options {
 public:
-  Options( void ) : ai(true), pbem(false), show_damage(true),
-                    replay(true), quick_replay(false) {}
+  Options( void ) : gametype(0), show_damage(true),
+                    replay(true), quick_replay(false),
+                    campaign(false), language(CF_LANG_DEFAULT) {}
 
-  void SetComputerPlayer( bool cp ) { ai = cp; }
-  void SetPBEM( bool mail ) { pbem = mail; }
   void SetDamageIndicator( bool di ) { show_damage = di; }
   void SetTurnReplay( bool rep ) { replay = rep; }
   void SetQuickReplay( bool rep ) { quick_replay = rep; }
+  void SetCampaign( bool flag ) { campaign = flag; }
+  void SetLanguage( const char *lang ) { language.assign(lang); }
+  void SetGameType( unsigned short type ) { gametype = type; }
 
-  bool GetComputerPlayer( void ) { return ai; }
-  bool GetPBEM( void ) { return pbem; }
-  bool GetDamageIndicator( void ) { return show_damage; }
-  bool GetTurnReplay( void ) { return replay; }
-  bool GetQuickReplay( void ) { return quick_replay; }
+  bool GetDamageIndicator( void ) const { return show_damage; }
+  bool GetTurnReplay( void ) const { return replay; }
+  bool GetQuickReplay( void ) const { return quick_replay; }
+  bool GetCampaign( void ) const { return campaign; }
+  const char *GetLanguage( void ) const { return language.c_str(); }
+  unsigned short GetGameType( void ) const { return gametype; }
+
+  bool IsAI( void ) const { return gametype == 1; }
+  bool IsPBEM( void ) const { return gametype == 2; }
+
+  bool IsLocked( const string &map ) const
+    { return find(unlocked_maps.begin(), unlocked_maps.end(), map)
+             == unlocked_maps.end(); }
+  void Unlock( const string &map )
+    { if (IsLocked(map)) unlocked_maps.push_back( map ); }
+  const vector<string> &GetUnlockedMaps( void ) const
+    { return unlocked_maps; }
 
 private:
-  bool ai;            // play against computer
-  bool pbem;          // play by email
+  unsigned short gametype; // hot seat, ai, or pbem
   bool show_damage;   // show damage indicator
   bool replay;        // show turn replays
   bool quick_replay;  // show only combat results
+  bool campaign;      // playing a campaign
+  string language;
+  vector<string> unlocked_maps;
 };
 
 #endif	/* _INCLUDE_OPTIONS_H */
