@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -348,7 +348,15 @@ short Path::RecordPath( const Unit *u, const PathNode *last,
 ////////////////////////////////////////////////////////////////////////
 
 PathNode *MoveShader::GetBestNode( const List &open ) const {
-  return static_cast<PathNode *>( open.Head() );
+  PathNode *p1, *p2;
+
+  p1 = static_cast<PathNode *>( open.Head() );
+  p2 = static_cast<PathNode *>( p1->Next() );
+  while ( p2 ) {
+    if ( p2->cost < p1->cost ) p1 = p2;
+    p2 = static_cast<PathNode *>( p2->Next() );
+  }
+  return p1;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -443,11 +451,10 @@ void MoveShader::MarkBuffer( PathNode *next, Direction dir, List &open ) const {
 
 short MoveShader::RecordPath( const Unit *u, const PathNode *last,
                               unsigned char off ) const {
-  Unit *target = static_cast<Unit *>( units->Head() );
-  while ( target ) {
+  for ( Unit *target = static_cast<Unit *>(units.Head());
+        target; target = static_cast<Unit *>(target->Next()) ) {
     if ( u->CanHit( target ) )
       path[map->Hex2Index( target->Position() )] = 1;
-    target = static_cast<Unit *>( target->Next() );
   }
   path[map->Hex2Index( u->Position() )] = 1;
 
