@@ -21,39 +21,38 @@
 ////////////////////////////////////////////////////////////////////////
 
 #include <string.h>
-#include "SDL_endian.h"
 
 #include "player.h"
-#include "misc.h"
+#include "strutil.h"
 #include "gamedefs.h"
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : Player::Player
+// NAME       : Player::Load
 // DESCRIPTION: Load player information from a file.
 // PARAMETERS : file - data file descriptor
-//              type - HUMAN or COMPUTER
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-Player::Player( SDL_RWops *file, unsigned char type ) {
-  SDL_RWread( file, &p_id, 1, 1 );
-  SDL_RWread( file, &p_level, 1, 1 );
-  SDL_RWread( file, &p_success, 1, 1 );
-  SDL_RWread( file, &p_briefing, 1, 1 );
-  SDL_RWread( file, p_name, 1, 16 );
-  SDL_RWread( file, p_password, 1, 8 );
-  crypt( p_password );
+int Player::Load( File &file ) {
+  p_id = file.Read8();
+  p_type = file.Read8();
+  p_success = file.Read8();
+  p_briefing = file.Read8();
+  p_name_id = file.Read8();
+
+  unsigned char len = file.Read8();
+  p_password = StringUtil::crypt( file.ReadS(len) );
+
+  p_col_light.r = file.Read8();
+  p_col_light.g = file.Read8();
+  p_col_light.b = file.Read8();
+  p_col_dark.r = file.Read8();
+  p_col_dark.g = file.Read8();
+  p_col_dark.b = file.Read8();
 
-  p_type = type;
   p_mode = MODE_IDLE;
   p_units = 0;
-  if ( p_id == PLAYER_ONE ) {
-    p_col_light = Color( P1_COLOR_LIGHT );
-    p_col_dark = Color( P1_COLOR_SHADOW );
-  } else {
-    p_col_light = Color( P2_COLOR_LIGHT );
-    p_col_dark = Color( P2_COLOR_SHADOW );
-  }
+  return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -63,17 +62,21 @@ Player::Player( SDL_RWops *file, unsigned char type ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int Player::Save( SDL_RWops *file ) const {
-  char crypted[8];
-  strcpy( crypted, p_password );
-  crypt( crypted );
+int Player::Save( File &file ) const {
+  file.Write8( p_id );
+  file.Write8( p_type );
+  file.Write8( p_success );
+  file.Write8( p_briefing );
+  file.Write8( p_name_id );
+  file.Write8( p_password.size() );
+  file.WriteS( StringUtil::crypt( p_password ) );
 
-  SDL_RWwrite( file, &p_id, 1, 1 );
-  SDL_RWwrite( file, &p_level, 1, 1 );
-  SDL_RWwrite( file, &p_success, 1, 1 );
-  SDL_RWwrite( file, &p_briefing, 1, 1 );
-  SDL_RWwrite( file, p_name, 1, 16 );
-  SDL_RWwrite( file, crypted, 1, 8 );
+  file.Write8( p_col_light.r );
+  file.Write8( p_col_light.g );
+  file.Write8( p_col_light.b );
+  file.Write8( p_col_dark.r );
+  file.Write8( p_col_dark.g );
+  file.Write8( p_col_dark.b );
   return 0;
 }
 
@@ -85,7 +88,7 @@ int Player::Save( SDL_RWops *file ) const {
 ////////////////////////////////////////////////////////////////////////
 
 const char *Player::Password( void ) const {
-  if ( p_password[0] ) return p_password;
+  if ( !p_password.empty() ) return p_password.c_str();
   return NULL;
 }
 
@@ -100,8 +103,8 @@ const char *Player::Password( void ) const {
 ////////////////////////////////////////////////////////////////////////
 
 void Player::SetPassword( const char *password ) {
-  if ( password == NULL ) p_password[0] = '\0';
-  else strcpy( p_password, password );
+  if ( password == NULL ) p_password.erase();
+  else p_password.assign( password );
 }
 
 ////////////////////////////////////////////////////////////////////////
