@@ -23,6 +23,8 @@
 #include "unitwindow.h"
 #include "extwindow.h"
 #include "game.h"
+#include "strutil.h"
+#include "msgs.h"
 
 #define ICON_MINI_WIDTH		15
 #define ICON_MINI_HEIGHT	15
@@ -73,8 +75,8 @@ public:
     unsigned short h, List *list, short selected,
     unsigned short flags, MapView *mview, Window *window );
 
-  virtual void DrawNodes( void );
-  virtual unsigned short ItemHeight( void ) const { return itemh; }
+  void DrawNodes( void );
+  unsigned short ItemHeight( void ) const { return itemh; }
 
 private:
   unsigned short itemh;
@@ -154,8 +156,8 @@ UnitListWidget::UnitListWidget( short id, short x, short y, unsigned short w,
 ////////////////////////////////////////////////////////////////////////
 
 void UnitListWidget::DrawNodes( void ) {
-  Rect box( x + 4, y + 1 + spacing, w - 8, h - 2 - 2 * spacing );
-  Rect area( x + 1, y + 1, w - 2, h - 2 );
+  Rect box( x + 4, y + 1 + spacing, listboxw - 8, h - 2 - 2 * spacing );
+  Rect area( x + 1, y + 1, listboxw - 2, h - 2 );
   short num = toprow / ItemHeight();                          // number of top node
   ULWNode *n = static_cast<ULWNode *>(list->GetNode( num ));  // top node
   short xoff = box.x, yoff = box.y + (num * ItemHeight()) - toprow;
@@ -165,7 +167,7 @@ void UnitListWidget::DrawNodes( void ) {
 
   while ( n ) {
     if ( num == current ) {
-      Rect hilite( x + 2, yoff, w - 4, ItemHeight() );
+      Rect hilite( x + 2, yoff, listboxw - 4, ItemHeight() );
       hilite.Clip( area );
       surface->FillRectAlpha( hilite, surface->GetFGPen() );
     }
@@ -219,8 +221,9 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
   int i;
   Building *b;
   Transport *t;
+  Player *player = dynamic_cast<MapObject *>(c)->Owner();;
 
-  if ( c->CType() == MO_UNIT ) {
+  if ( dynamic_cast<MapObject *>(c)->IsUnit() ) {
     unit = true;
     t = static_cast<Transport *>(c);
     b = NULL;
@@ -244,8 +247,8 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
     n->unit = c->GetUnit( i );
     n->type = n->unit->Type();
 
-    if ( !c->Owner() ) {
-      n->image = n->unit->Type()->Image() + Gam->CurrentPlayer()->ID() * 6;
+    if ( !player ) {
+      n->image = n->unit->Type()->Image() + Gam->GetMission()->GetPlayer().ID() * 6;
       n->ok = false;
     } else {
       n->image = n->unit->BaseImage();
@@ -255,7 +258,7 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
   }
 
   // create list widget
-  width = GFX_WIDTH + XP_ICON_WIDTH + 25;   // listwidget width
+  width = GFX_WIDTH + XP_ICON_WIDTH + DEFAULT_SLIDER_SIZE + 20;   // listwidget width
   listwidget = new UnitListWidget( CH_LIST_UNITS, 10, 10,
                width, h - sfont->Height() - 28, &normal, -1,
                WIDGET_VSCROLL|WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY,
@@ -265,9 +268,9 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
   // create buttons
   Surface *icons = view->GetSystemIcons();
   new ButtonWidget( GUI_CLOSE, 10, h - sfont->Height() - 18,
-      width, sfont->Height() + 8, WIDGET_DEFAULT, "E_xit", this );
+      width, sfont->Height() + 8, WIDGET_DEFAULT, MSG(MSG_B_EXIT), this );
 
-  if ( c->Owner() == Gam->CurrentPlayer() ) {
+  if ( player == &Gam->GetMission()->GetPlayer() ) {
     ButtonWidget *btn;
     unsigned short xoff = width + 15, yoff = lfont->Height() + 35;
 
@@ -280,7 +283,6 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
                                   ICON_MINI_WIDTH, ICON_MINI_HEIGHT ),
                             Rect( ICON_MINI_REPAIR2_X, ICON_MINI_REPAIR2_Y,
                                   ICON_MINI_WIDTH, ICON_MINI_HEIGHT ) );
-      btn->SetKey( 'r' );
       btn->SetHook( this );
       xoff += btn->Width();
       buttons[CH_BUTTON_REPAIR] = btn;
@@ -295,7 +297,6 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
                                     ICON_MINI_WIDTH, ICON_MINI_HEIGHT ),
                               Rect( ICON_MINI_BUILD2_X, ICON_MINI_BUILD2_Y,
                                     ICON_MINI_WIDTH, ICON_MINI_HEIGHT ) );
-        btn->SetKey( 'b' );
         btn->SetHook( this );
         xoff += btn->Width();
         buttons[CH_BUTTON_BUILD] = btn;
@@ -309,7 +310,6 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
                                 ICON_MINI_WIDTH, ICON_MINI_HEIGHT ),
                           Rect( ICON_MINI_TRANSFER2_X, ICON_MINI_TRANSFER2_Y,
                                 ICON_MINI_WIDTH, ICON_MINI_HEIGHT ) );
-    btn->SetKey( 't' );
     btn->SetHook( this );
     buttons[CH_BUTTON_TRANSFER] = btn;
   }
@@ -345,14 +345,19 @@ void ContainerWindow::Draw( void ) {
   Gam->GetMapWindow()->DrawUnitInfo( NULL, this, unitinfo );
 
   // draw a separator bar
-  unsigned short sep = 45 + GFX_WIDTH + XP_ICON_WIDTH;
+  unsigned short sep = unitinfo.x;
 
   // show the name of the unit/building
   DrawBox( Rect( sep, 10, w - sep - 10, lfont->Height() + 20 ), BOX_RAISED );
-  const char *name;
-  if ( unit ) name = static_cast<Transport *>(c)->Name();
-  else name = static_cast<Building *>(c)->Name();
 
+  string namestr( dynamic_cast<MapObject *>(c)->Name() );
+  unsigned short nlen = lfont->FitText( namestr.c_str(), w - sep - 14, false );
+  if ( nlen < namestr.size() ) {
+    namestr.erase( nlen - 1 );
+    namestr += "...";
+  }
+
+  const char *name = namestr.c_str();
   lfont->Write( name, this,
                 sep + 3 + (w - sep - 10 - lfont->TextWidth(name)) / 2, 23,
                 view->GetBGPen() );
@@ -361,7 +366,7 @@ void ContainerWindow::Draw( void ) {
                 view->GetFGPen() );
 
   // show crystal storage
-  crystals_icon.Draw( this, w - sfont->Width() * 10 - ICON_WIDTH - 10, lfont->Height() + 30 );
+  crystals_icon.Draw( this, w - 5 - ICON_WIDTH, lfont->Height() + 30 );
   DrawCrystals();
 }
 
@@ -376,9 +381,6 @@ void ContainerWindow::Draw( void ) {
 void ContainerWindow::DrawCrystals( void ) {
   char buf[16];
   unsigned short stock, change;
-  Rect rect( w - sfont->Width() * 10 - 10,
-             lfont->Height() + 30 + (ICON_HEIGHT - sfont->Height()) / 2,
-             sfont->Width() * 10, sfont->Height() );
 
   stock = c->Crystals();
   if ( unit ) change = 0;
@@ -387,8 +389,12 @@ void ContainerWindow::DrawCrystals( void ) {
   if ( change > 0 ) sprintf( buf, "%d (+%d)", stock, change );
   else itoa( stock, buf );
 
+  unsigned short tlen = sfont->TextWidth( buf );
+  Rect rect( w - ICON_WIDTH - tlen - 5 - 30,
+             lfont->Height() + 30 + (ICON_HEIGHT - sfont->Height()) / 2,
+             tlen + ICON_WIDTH + 5, sfont->Height() );
   DrawBack( rect );
-  sfont->Write( buf, this, rect.x, rect.y );
+  sfont->Write( buf, this, rect.x + 30, rect.y );
 } 
 
 ////////////////////////////////////////////////////////////////////////
@@ -435,7 +441,7 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
       for ( int i = 0; i < 32; ++i ) {
         if ( prod & (1 << i) ) {        // this unit type can be built
           ULWNode *n = new ULWNode;
-          n->type = Gam->GetUnitType( i );
+          n->type = Gam->GetMission()->GetUnitSet().GetUnitInfo( i );
           n->unit = NULL;
           n->image = n->type->Image() + b->Owner()->ID() * 6;
           n->ok = (n->type->Cost() <= crystals);
@@ -473,22 +479,23 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
     min = MAX( 0, fullhost - (maxclient - fullclient) );
     max = MIN( maxhost, fullhost + fullclient );
 
-    slider = new SliderWidget( 0, unitinfo.x + GFX_WIDTH + 20,
-                               unitinfo.y + (unitinfo.h - DEFAULT_SLIDER_SIZE) / 2,
-                               unitinfo.w - 2 * ICON_WIDTH - 40, DEFAULT_SLIDER_SIZE,
-                               min, max, fullhost, max - min + 1,
-                               WIDGET_HSCROLL|WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY,
-                               NULL, this );
-    stringwidgets[0] = new StringWidget( 0, unitinfo.x + 10,
-                               unitinfo.y + unitinfo.h / 2, sfont->Width() * 4 + 8,
-                               sfont->Height() + 8, itoa( fullclient, buf ), 4,
-                               WIDGET_ALIGN_CENTER|WIDGET_STR_CONST, NULL, this );
-    stringwidgets[1] = new StringWidget( 0, slider->LeftEdge() + slider->Width() + 10,
-                               unitinfo.y + unitinfo.h / 2, sfont->Width() * 4 + 8,
-                               sfont->Height() + 8, itoa( fullhost, buf ), 4,
-                               WIDGET_ALIGN_CENTER|WIDGET_STR_CONST, NULL, this );
+    slider = new SliderWidget( 0, unitinfo.x + 10,
+                       unitinfo.y + (unitinfo.h - DEFAULT_SLIDER_SIZE) / 2,
+                       unitinfo.w - 20, DEFAULT_SLIDER_SIZE,
+                       min, max, fullhost, max - min + 1,
+                       WIDGET_HSCROLL|WIDGET_HSCROLLKEY, NULL, this );
+    stringwidgets[0] = new StringWidget( 0, slider->LeftEdge(),
+                       slider->TopEdge() + slider->Height() + 5,
+                       sfont->Width() * 4 + 8, sfont->Height() + 8,
+                       itoa( fullclient, buf ), 4,
+                       WIDGET_ALIGN_CENTER|WIDGET_STR_CONST, NULL, this );
+    stringwidgets[1] = new StringWidget( 0,
+                       slider->LeftEdge() + slider->Width() - stringwidgets[0]->Width(),
+                       stringwidgets[0]->TopEdge(), stringwidgets[0]->Width(),
+                       stringwidgets[0]->Height(), itoa( fullhost, buf ), 4,
+                       WIDGET_ALIGN_CENTER|WIDGET_STR_CONST, NULL, this );
     transferscroll = new TransferScrollable( fullhost, fullclient, stringwidgets[1],
-                               stringwidgets[0], c, t );
+                       stringwidgets[0], c, t );
     slider->SetHook( transferscroll );
 
     slider->Draw();
@@ -497,10 +504,10 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
 
     // draw images
     Gam->GetMapWindow()->GetMapView()->DrawUnit(
-                   t->BaseImage(), this, stringwidgets[0]->LeftEdge(),
-                   stringwidgets[0]->TopEdge() - ICON_HEIGHT - 2, unitinfo );
-    crystals_icon.Draw( this, stringwidgets[1]->LeftEdge(),
-                   stringwidgets[1]->TopEdge() - ICON_HEIGHT - 2 );
+                   t->BaseImage(), this, slider->LeftEdge(),
+                   slider->TopEdge() - ICON_HEIGHT - 3, unitinfo );
+    crystals_icon.Draw( this, slider->LeftEdge() + slider->Width() - ICON_WIDTH,
+                   slider->TopEdge() - ICON_HEIGHT - 3 );
     Show();
   } else {                                          // switch back to CW_MODE_NORMAL...
     if ( mode == CW_MODE_PRODUCTION ) {             // ...from CW_MODE_PRODUCTION
@@ -565,7 +572,7 @@ GUI_Status ContainerWindow::SelectNode( ULWNode *node ) {
     } else {                        // production mode
       if ( node->ok ) {             // sufficient amount of crystals
         Building *b = static_cast<Building *>(c);
-        Unit *u = Gam->CreateUnit( node->type, b->Position(), b->Owner() );
+        Unit *u = Gam->GetMission()->CreateUnit( node->type->ID(), *b->Owner(), b->Position() );
 
         // add unit to the widget's list
         ULWNode *ulw = new ULWNode;
@@ -579,7 +586,7 @@ GUI_Status ContainerWindow::SelectNode( ULWNode *node ) {
         DrawCrystals();
         Show();
         SwitchMode( CW_MODE_NORMAL );
-      } else new NoteWindow( "Insufficient supplies", "You cannot build this unit.", 0, view );
+      } else new NoteWindow( MSG(MSG_ERR_NO_CRYSTALS), MSG(MSG_ERR_NO_PRODUCTION), 0, view );
     }
   }
   return rc;
@@ -603,15 +610,20 @@ GUI_Status ContainerWindow::WidgetActivated( Widget *widget, Window *win ) {
     if ( n ) {
       Unit *u = n->unit;
       if ( u->GroupSize() < MAX_GROUP_SIZE ) {
-        char repmsg[64];
-        sprintf( repmsg, "Repairing the %s requires %d crystals.", u->Name(), CRYSTALS_REPAIR );
+        string repmsg;
+        repmsg = StringUtil::strprintf( MSG(MSG_ASK_REPAIR), u->Name() );
+        repmsg = StringUtil::strprintf( repmsg, CRYSTALS_REPAIR );
 
         if ( c->Crystals() >= CRYSTALS_REPAIR ) {
-          DialogWindow *reqwin = new DialogWindow( NULL, repmsg, "_Repair|_Cancel",
-                                                   0, 0, view );
+          string buttons;
+          buttons.append( MSG(MSG_B_REPAIR) );
+          buttons += '|';
+          buttons.append( MSG(MSG_B_CANCEL) );
+          DialogWindow *reqwin =
+            new DialogWindow( NULL, repmsg, buttons, 0, 0, view );
           reqwin->SetButtonID( 0, CH_BUTTON_REPAIR_CONFIRM );
           reqwin->SetButtonHook( 0, this );
-        } else new NoteWindow( "Insufficient stocks!", repmsg, WIN_CLOSE_ESC, view );
+        } else new NoteWindow( MSG(MSG_ERR_NO_CRYSTALS), repmsg, WIN_CLOSE_ESC, view );
       }
     }
     break; }
@@ -624,7 +636,7 @@ GUI_Status ContainerWindow::WidgetActivated( Widget *widget, Window *win ) {
       Unit *u = n->unit;
       if ( u->IsTransport() ) {
         SwitchMode( (Mode() == CW_MODE_NORMAL) ? CW_MODE_TRANSFER : CW_MODE_NORMAL );
-      } else new NoteWindow( "Transfer", "Unit cannot transport crystals",
+      } else new NoteWindow( MSG(MSG_TRANSFER), MSG(MSG_ERR_NO_TRANSPORTER),
                              WIN_CLOSE_ESC, view );
     }
     break; }
