@@ -24,8 +24,6 @@
 #define _INCLUDE_EDWIDGET_H
 
 #include "textbox.h" // for NumberWidget
-#include "button.h"  // for CycleWidget
-
 
 class NumberWidget : public StringWidget, public InputValidator {
 public:
@@ -48,25 +46,5 @@ private:
   char numbuf2[8];
 };
 
-
-class CycleWidget : public ButtonWidget {
-public:
-  CycleWidget( short id, short x, short y, unsigned short w, unsigned short h,
-               unsigned short flags, const char *title, unsigned short defval,
-               const char *labels[], Window *window );
-
-  void SetValue( unsigned short value );
-  unsigned short GetValue( void ) const { return val; }
-  const char *GetLabel( void ) const { return choices[val]; }
-
-  void Draw( void );
-  void Release( void );
-
-private:
-  unsigned short val;
-  unsigned short maxval;
-  const char **choices;
-};
-
 #endif	// _INCLUDE_EDWIDGET_H
 
