@@ -284,9 +284,15 @@ void EdWindow::SetCursor( const Point &cursor ) {
 void EdWindow::LeftMouseButton( const Point &hex ) {
   Map *map = mv.GetMap();
 
-  if ( mode == ED_MODE_TERRAIN ) map->SetHexType( hex, selected );
-  else if ( mode == ED_MODE_UNIT ) {
-    if ( map->GetMapObject(hex) ) new NoteWindow( "Error!", "Hex occupied.", 0, view );
+  if ( mode == ED_MODE_TERRAIN ) {
+    if ( map->IsBuilding( hex ) && !(selected & TT_ENTRANCE) && map->GetBuilding( hex ) )
+      new NoteWindow( "Warning", "You have just removed a shop entrance. "
+                      "The corresponding shop has NOT been deleted! "
+                      "(to do so restore the entrance and delete the shop before "
+                      "erasing the entrance)", 0, view );
+    map->SetHexType( hex, selected );
+  } else if ( mode == ED_MODE_UNIT ) {
+    if ( map->GetMapObject(hex) ) new NoteWindow( "Error", "Hex occupied.", 0, view );
     else {
       UnitSet *us = map->GetUnitSet();
       const UnitType *ut = us->GetUnitInfo( selected < us->NumUT() ?
@@ -325,44 +331,23 @@ Mission *EdWindow::LoadMission( const char *file, bool filereq ) {
         mission = NULL;
         new NoteWindow( "Error", "Couldn't load map. This could be a file type mismatch.", 0, view );
       } else {
-        bool allowed = true;
+        Map &map = mission->GetMap();
+        mode = ED_MODE_VIEW;
+        mv.SetMap( &map );
+        mv.SetCursor( Point(0,0) );
 
-        if ( mission->GetPassword() ) {
-          PasswordWindow *pw = new PasswordWindow( "Access restricted", "Please enter access code",
-                                   mission->GetPassword(), view );
-          pw->EventLoop();
-          allowed = pw->PasswordOk();
-          view->CloseWindow( pw );
-        }
+        tile_wd->SetTerrainSet( map.GetTerrainSet(), GFX_WIDTH, GFX_HEIGHT );
+        unit_wd->SetUnitSet( map.GetUnitSet(), GFX_WIDTH, GFX_HEIGHT );
 
-        if ( allowed ) {
-          Map &map = mission->GetMap();
-          mode = ED_MODE_VIEW;
-          mv.SetMap( &map );
-          mv.SetCursor( Point(0,0) );
+        short max = MAX( map.Height() * GFX_HEIGHT + GFX_OVERLAP_Y - mv.Height(), 0 );
+        ud_wd->Adjust( 0, max, mv.Height() );
+        ud_wd->ScrollTo( 0 );
 
-          tile_wd->SetTerrainSet( map.GetTerrainSet(), GFX_WIDTH, GFX_HEIGHT );
-          unit_wd->SetUnitSet( map.GetUnitSet(), GFX_WIDTH, GFX_HEIGHT );
+        max = MAX( map.Width() * (GFX_WIDTH - GFX_OVERLAP_X) + GFX_OVERLAP_X - mv.Width(), 0 );
+        lr_wd->Adjust( 0, max, mv.Width() );
+        lr_wd->ScrollTo( 0 );
 
-          short max = MAX( map.Height() * GFX_HEIGHT + GFX_OVERLAP_Y - mv.Height(), 0 );
-          ud_wd->Adjust( 0, max, mv.Height() );
-          ud_wd->ScrollTo( 0 );
-
-          max = MAX( map.Width() * (GFX_WIDTH - GFX_OVERLAP_X) + GFX_OVERLAP_X - mv.Width(), 0 );
-          lr_wd->Adjust( 0, max, mv.Width() );
-          lr_wd->ScrollTo( 0 );
-
-          mv.Enable();
-        } else {
-          Audio::PlaySfx( Audio::SND_GUI_ERROR, 0 );
-          new NoteWindow( "Error", "Code rejected. Access denied.", WIN_CENTER, view );
-
-          tile_wd->SetTerrainSet( NULL, GFX_WIDTH, GFX_HEIGHT );
-          unit_wd->SetUnitSet( NULL, GFX_WIDTH, GFX_HEIGHT );
-          delete mission;
-          mission = NULL;
-        }
-      
+        mv.Enable();
         Draw();
         Show();
       }
@@ -430,10 +415,16 @@ GUI_Status EdWindow::NewMission( const Point &size, const string &tset,
   TerrainSet *ts = new TerrainSet;
   UnitSet *us = new UnitSet;
 
-  if ( ts->Load( tset.c_str() ) ) {
+  string tname( get_data_dir() + tset + ".tiles" );
+  File tfile( tname );
+
+  string uname( get_data_dir() + uset + ".units" );
+  File ufile( uname );
+
+  if ( !tfile.Open("rb") || ts->Load( tfile, tset.c_str() ) ) {
     new NoteWindow( "Error", "Tile set not available", 0, view );
     delete ts;
-  } else if ( us->Load( uset.c_str() ) ) {
+  } else if ( !ufile.Open("rb") || us->Load( ufile, uset.c_str() ) ) {
     new NoteWindow( "Error", "Unit set not available", 0, view );
     delete ts;
     delete us;
@@ -464,6 +455,8 @@ GUI_Status EdWindow::NewMission( const Point &size, const string &tset,
     Show();
   }
 
+  tfile.Close();
+  ufile.Close();
   return GUI_OK;
 }
 
@@ -511,7 +504,7 @@ string EdWindow::GetFileName( const char *filename, const char *suffix,
 
     } while ( !done );
 
-    if ( file.length() > 0 && (flag == WIN_FILE_SAVE) && exists( file.c_str() ) ) {
+    if ( file.length() > 0 && (flag == WIN_FILE_SAVE) && File::Exists( file ) ) {
       // if file exists let user confirm the write
       char *conmsg = new char[ file.length() + 20 ];
       strcpy( conmsg, file.c_str() );
@@ -770,10 +763,10 @@ void EdWindow::SwapTiles( unsigned short t1, unsigned short t2 ) {
 ////////////////////////////////////////////////////////////////////////
 
 void EdWindow::ValidateMission( void ) const {
-  string msg;
+  stringstream msg;
 
   if ( mission->Validate( msg ) > 0 ) {
-    new MessageWindow( "Validation Results", msg.c_str(), view );
+    new MessageWindow( "Validation Results", msg.str().c_str(), view );
   } else {
      new NoteWindow( "Validation successful",
          "No errors or warnings have been detected.", 0, view );
