@@ -40,8 +40,8 @@ const short EdEventGenericWindow::B_ID_DEPEND_OK = 9;
 
 const short EdEventCreateUnitWindow::B_ID_UNIT = 1;
 const short EdEventCreateUnitWindow::B_ID_UNIT_OK = 2;
-const short EdEventCreateUnitWindow::B_ID_SHOP = 3;
-const short EdEventCreateUnitWindow::B_ID_SHOP_OK = 4;
+const short EdEventCreateUnitWindow::N_ID_XPOS = 3;
+const short EdEventCreateUnitWindow::N_ID_YPOS = 4;
 
 const short EdEventManipEventWindow::B_ID_EVENT = 1;
 const short EdEventManipEventWindow::B_ID_EVENT_OK = 2;
@@ -61,6 +61,12 @@ const short EdEventResearchWindow::B_ID_UNIT_OK = 2;
 const short EdEventResearchWindow::B_ID_SHOP = 3;
 const short EdEventResearchWindow::B_ID_SHOP_OK = 4;
 
+const short EdEventScoreWindow::B_ID_MSG = 1;
+const short EdEventScoreWindow::B_ID_MSG_OK = 2;
+const short EdEventScoreWindow::B_ID_TITLE = 3;
+const short EdEventScoreWindow::B_ID_TITLE_OK = 4;
+const short EdEventScoreWindow::N_ID_SCORE = 5;
+
 const short EdTrigHaveShopWindow::B_ID_SHOP = 1;
 const short EdTrigHaveShopWindow::B_ID_SHOP_OK = 2;
 const short EdTrigHaveShopWindow::B_ID_PLAYER = 3;
@@ -250,6 +256,8 @@ GUI_Status EdEventGenericWindow::WidgetActivated( Widget *widget, Window *win )
       new EdTrigUnitDestroyedWindow( event, mission, view ); break;
     case ETRIGGER_UNIT_POSITION:
       new EdTrigUnitPositionWindow( event, mission, view ); break;
+    case ETRIGGER_HANDICAP:
+      new EdTrigHandicapWindow( event, view ); break;
     default:
       new NoteWindow("Sorry", "Not yet implemented", 0, view);
     }
@@ -268,7 +276,7 @@ GUI_Status EdEventGenericWindow::WidgetActivated( Widget *widget, Window *win )
     case EVENT_RESEARCH:
       new EdEventResearchWindow( event, mission, view ); break;
     case EVENT_SCORE:
-      new EdEventScoreWindow( event, view ); break;
+      new EdEventScoreWindow( event, mission, view ); break;
     default:
       new NoteWindow("Sorry", "Not yet implemented", 0, view);
     }
@@ -308,14 +316,17 @@ EdEventCreateUnitWindow::EdEventCreateUnitWindow( Event &event,
            mission.GetUnitSet().GetUnitInfo(event.GetData(0))->Name(),
            30, WIDGET_STR_CONST, NULL, this );
 
-  wd = new ButtonWidget( B_ID_SHOP, 5, wd->TopEdge() + wdh + 5,
-       sfont->Width() * 4 + 10, wdh, 0, "_Shop", this );
+  wdx = 10 + sfont->TextWidth( "X" );
+  wd = new NumberWidget( N_ID_XPOS, wdx, wd->TopEdge() + wdh + 5,
+       sfont->Width() * 4 + 10, wdh, event.GetData(1), 0,
+       mission.GetMap().Width() - 1, WIDGET_ALIGN_LEFT, "_X", this );
   wd->SetHook( this );
 
-  Building *b = mission.GetBuildingByID( event.GetData(1) );
-  e_shop = new StringWidget( 0, wdx, wd->TopEdge(),
-           e_unit->Width(), wdh, b ? b->Name() : "Error",
-           30, WIDGET_STR_CONST, NULL, this );
+  wdx += wd->Width() + 10 + sfont->TextWidth( "Y" );
+  wd = new NumberWidget( N_ID_YPOS, wdx, wd->TopEdge(),
+       wd->Width(), wdh, event.GetData(2), 0,
+       mission.GetMap().Height() - 1, WIDGET_ALIGN_LEFT, "_Y", this );
+  wd->SetHook( this );
 
   new ButtonWidget( GUI_CLOSE, 1, h - wdh - 1, w - 2, wdh,
                     WIDGET_DEFAULT, "_OK", this );
@@ -347,16 +358,11 @@ GUI_Status EdEventCreateUnitWindow::WidgetActivated( Widget *widget, Window *win
     e_unit->SetString( n ? n->Name() : "Error" );
     tlist.Clear();
     break;
-  case B_ID_SHOP:
-    TLWListBuilder::BuildShopList( mission.GetBuildings(), tlist );
-    new ListSelectWindow( "Select shop", tlist,
-        event.GetData(1), false, this, B_ID_SHOP_OK, view );
+  case N_ID_XPOS:
+  case N_ID_YPOS:
+    event.SetData( widget->ID() - N_ID_XPOS + 1,
+                   static_cast<NumberWidget *>(widget)->Number() );
     break;
-  case B_ID_SHOP_OK:
-    n = static_cast<ListSelectWindow *>(win)->Selected();
-    event.SetData( 1, n ? n->ID() : -1 );
-    e_shop->SetString( n ? n->Name() : "Error" );
-    tlist.Clear();
   }
 
   return GUI_OK;
@@ -704,27 +710,49 @@ GUI_Status EdEventResearchWindow::WidgetActivated( Widget *widget, Window *win )
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdEventScoreWindow::EdEventScoreWindow
 // DESCRIPTION: This window allows configuration of a SCORE event.
-// PARAMETERS : event - event to edit
-//              view  - pointer to the window's view
+// PARAMETERS : event   - event to edit
+//              mission - current mission
+//              view    - pointer to the window's view
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-EdEventScoreWindow::EdEventScoreWindow( Event &event, View *view ) :
-      Window(WIN_CENTER, view), event(event) { 
-  SetSize( MIN(sfont->Width() * 15 + 15, view->Width()),
-           sfont->Height() * 2 + 30 );
+EdEventScoreWindow::EdEventScoreWindow( Event &event, Mission &mission, View *view ) :
+      Window(WIN_CENTER, view), event(event), mission(mission) { 
+  SetSize( MIN(sfont->Width() * 20 + 15, view->Width()),
+           sfont->Height() * 4 + 50 );
 
   unsigned short wdx = sfont->Width() * 5 + 10;
-  Widget *wd = new NumberWidget( 0, wdx, 5,
+  Widget *wd = new NumberWidget( N_ID_SCORE, wdx, 5,
                w - wdx - 5, sfont->Height() + 8,
                event.GetData(0), 1, 100,
                WIDGET_ALIGN_LEFT, "_Score", this );
   wd->SetHook( this );
 
+  wd = new ButtonWidget( B_ID_TITLE, 5, wd->TopEdge() + wd->Height() + 5,
+                    sfont->Width() * 10, wd->Height(), 0, "_Lose Title", this );
+  wd->SetHook( this );
+
+  wdx = wd->LeftEdge() + wd->Width() + 5;
+  const char *str = mission.GetMessage(event.GetData(2));
+  other_title = new StringWidget( 0, wdx, wd->TopEdge(), w - wdx - 5, wd->Height(),
+           (str || event.GetData(2) == -1) ? str : "Error", 30, WIDGET_STR_CONST,
+           NULL, this );
+
+  wd = new ButtonWidget( B_ID_MSG, 5, wd->TopEdge() + wd->Height() + 5,
+                         wd->Width(), wd->Height(), 0, "_Lose Msg", this );
+  wd->SetHook( this );
+
+  str = mission.GetMessage(event.GetData(1));
+  other_msg = new StringWidget( 0, wdx, wd->TopEdge(), other_title->Width(), wd->Height(),
+           (str || event.GetData(1) == -1) ? str : "Error", 30, WIDGET_STR_CONST,
+           NULL, this );
+
   new ButtonWidget( GUI_CLOSE, 1, h - wd->Height() - 1, w - 2, wd->Height(),
                     WIDGET_DEFAULT, "_OK", this );
   Draw();
   Show();
+
+  TLWListBuilder::BuildMsgList( mission.GetMessages(), msglist );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -736,7 +764,31 @@ EdEventScoreWindow::EdEventScoreWindow( Event &event, View *view ) :
 ////////////////////////////////////////////////////////////////////////
 
 GUI_Status EdEventScoreWindow::WidgetActivated( Widget *widget, Window *win ) {
-  event.SetData( 0, static_cast<NumberWidget *>(widget)->Number() );
+  TLWNode *n;
+
+  switch ( widget->ID() ) {
+  case N_ID_SCORE:
+    event.SetData( 0, static_cast<NumberWidget *>(widget)->Number() );
+    break;
+  case B_ID_MSG:
+    new ListSelectWindow( "Select lose message", msglist, event.GetData(1),
+                          true, this, B_ID_MSG_OK, view );
+    break;
+  case B_ID_MSG_OK:
+    n = static_cast<ListSelectWindow *>(win)->Selected();
+    event.SetData( 1, n ? n->ID() : -1 );
+    other_msg->SetString( n ? mission.GetMessage(n->ID()) : NULL );
+    break;
+  case B_ID_TITLE:
+    new ListSelectWindow( "Select lose title", msglist, event.GetData(2),
+                          true, this, B_ID_TITLE_OK, view );
+    break;
+  case B_ID_TITLE_OK:
+    n = static_cast<ListSelectWindow *>(win)->Selected();
+    event.SetData( 2, n ? n->ID() : -1 );
+    other_title->SetString( n ? mission.GetMessage(n->ID()) : NULL );
+  }
+
   return GUI_OK;
 }
 
@@ -903,8 +955,8 @@ GUI_Status EdTrigHaveUnitWindow::WidgetActivated( Widget *widget, Window *win )
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdTrigTurnWindow::EdTrigTurnWindow
 // DESCRIPTION: This window allows configuration of a TURN trigger.
-// PARAMETERS : event   - event to edit
-//              view    - pointer to the window's view
+// PARAMETERS : event - event to edit
+//              view  - pointer to the window's view
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
@@ -1111,3 +1163,58 @@ GUI_Status EdTrigUnitPositionWindow::WidgetActivated( Widget *widget, Window *wi
   return GUI_OK;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdTrigHandicapWindow::EdTrigHandicapWindow
+// DESCRIPTION: This window allows configuration of a HANDICAP trigger.
+// PARAMETERS : event - event to edit
+//              view  - pointer to the window's view
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+EdTrigHandicapWindow::EdTrigHandicapWindow( Event &event, View *view ) :
+                  Window(WIN_CENTER, view), event(event) { 
+  SetSize( MIN(sfont->Width() * 20 + DEFAULT_CBW_SIZE + 15, view->Width()),
+           sfont->Height() * 4 + 40 );
+
+  Widget *wd = new CheckboxWidget( HANDICAP_NONE, 5, 5,
+               DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE,
+               event.GetTData(0) & HANDICAP_NONE,
+               WIDGET_STYLE_GFX|WIDGET_STYLE_NOBORDER|WIDGET_ALIGN_RIGHT,
+               "_No handicap", this );
+  wd->SetHook( this );
+
+  wd = new CheckboxWidget( HANDICAP_P1, 5, wd->TopEdge() + wd->Height() + 5,
+       DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE,
+       event.GetTData(0) & HANDICAP_P1,
+       WIDGET_STYLE_GFX|WIDGET_STYLE_NOBORDER|WIDGET_ALIGN_RIGHT,
+       "Player _1 handicapped", this );
+  wd->SetHook( this );
+
+  wd = new CheckboxWidget( HANDICAP_P2, 5, wd->TopEdge() + wd->Height() + 5,
+       DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE,
+       event.GetTData(0) & HANDICAP_P2,
+       WIDGET_STYLE_GFX|WIDGET_STYLE_NOBORDER|WIDGET_ALIGN_RIGHT,
+       "Player _2 handicapped", this );
+  wd->SetHook( this );
+
+  unsigned short wdh = sfont->Height() + 8;
+  new ButtonWidget( GUI_CLOSE, 1, h - wdh - 1, w - 2, wdh,
+                    WIDGET_DEFAULT, "_OK", this );
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdTrigHandicapWindow::WidgetActivated
+// DESCRIPTION: React to user actions.
+// PARAMETERS : widget - activated widget
+//              win    - window containing the widget
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status EdTrigHandicapWindow::WidgetActivated( Widget *widget, Window *win ) {
+  short handicap = event.GetTData( 0 );
+  event.SetTData( 0, handicap ^ widget->ID() );
+  return GUI_OK;
+}
+
