@@ -39,19 +39,20 @@ const short NewMissionWindow::B_ID_TILESET = 2;
 const short NewMissionWindow::B_ID_UNITSET_OK = 3;
 const short NewMissionWindow::B_ID_TILESET_OK = 4;
 
-const short EdMissionSetupWindow::S_ID_PLAYER1 = 1;
-const short EdMissionSetupWindow::S_ID_PLAYER2 = 2;
-const short EdMissionSetupWindow::B_ID_BRIEFING1 = 3;
-const short EdMissionSetupWindow::B_ID_BRIEFING2 = 4;
-const short EdMissionSetupWindow::B_ID_BRIEFING1_OK = 5;
-const short EdMissionSetupWindow::B_ID_BRIEFING2_OK = 6;
-const short EdMissionSetupWindow::B_ID_INFOMSG = 7;
-const short EdMissionSetupWindow::B_ID_INFOMSG_OK = 8;
-const short EdMissionSetupWindow::B_ID_SHOWMSG = 9;
-const short EdMissionSetupWindow::B_ID_NEXTMAP = 10;
-const short EdMissionSetupWindow::B_ID_NEXTMAP_OK = 11;
-const short EdMissionSetupWindow::S_ID_PASSWORD = 12;
-const short EdMissionSetupWindow::B_ID_PLAYERS = 13;
+const short EdMissionSetupWindow::B_ID_NAME = 0;
+const short EdMissionSetupWindow::B_ID_INFO = 1;
+const short EdMissionSetupWindow::B_ID_PLAYER1 = 2;
+const short EdMissionSetupWindow::B_ID_PLAYER2 = 3;
+const short EdMissionSetupWindow::B_ID_BRIEF1 = 4;
+const short EdMissionSetupWindow::B_ID_BRIEF2 = 5;
+const short EdMissionSetupWindow::B_ID_CAMPAIGN_NAME = 6;
+const short EdMissionSetupWindow::B_ID_CAMPAIGN_INFO = 7;
+const short EdMissionSetupWindow::B_ID_SHOWMSG = 8;
+const short EdMissionSetupWindow::B_ID_SET = 9;
+const short EdMissionSetupWindow::S_ID_SEQUEL = 10;
+const short EdMissionSetupWindow::B_ID_PLAYERS = 11;
+const short EdMissionSetupWindow::S_ID_MUSIC = 12;
+const short EdMissionSetupWindow::B_ID_CAMPAIGN = 13;
 
 const short EdBuildingWindow::B_ID_PLAYER = 1;
 const short EdBuildingWindow::B_ID_WORKSHOP = 2;
@@ -59,15 +60,18 @@ const short EdBuildingWindow::B_ID_FACTORY = 3;
 const short EdBuildingWindow::B_ID_CRYSTALS = 4;
 const short EdBuildingWindow::B_ID_OK = 5;
 const short EdBuildingWindow::B_ID_NAME = 6;
-const short EdBuildingWindow::B_ID_UNITS = 7;
-const short EdBuildingWindow::N_ID_MAXCRYSTALS = 8;
-const short EdBuildingWindow::B_ID_FACTORY_UNITS = 9;
+const short EdBuildingWindow::B_ID_NAME_OK = 7;
+const short EdBuildingWindow::B_ID_UNITS = 8;
+const short EdBuildingWindow::N_ID_MAXCRYSTALS = 9;
+const short EdBuildingWindow::B_ID_FACTORY_UNITS = 10;
 
 const short EdUnitWindow::B_ID_PLAYER = 1;
 const short EdUnitWindow::B_ID_DIRECTION = 2;
-const short EdUnitWindow::N_ID_CRYSTALS = 3;
-const short EdUnitWindow::B_ID_UNITS = 4;
-const short EdUnitWindow::B_ID_OK = 5;
+const short EdUnitWindow::B_ID_SIZE = 3;
+const short EdUnitWindow::B_ID_XP = 4;
+const short EdUnitWindow::N_ID_CRYSTALS = 5;
+const short EdUnitWindow::B_ID_UNITS = 6;
+const short EdUnitWindow::B_ID_OK = 7;
 
 const short EdUnitsWindow::B_ID_NEW = 1;
 const short EdUnitsWindow::B_ID_EDIT = 2;
@@ -392,7 +396,7 @@ GUI_Status NewMissionWindow::WidgetActivated( Widget *button, Window *win ) {
     if ( file.length() > 0 ) {
       file = file_part( file );
       file.erase( file.length() - 6 );   // remove file name suffix
-      tileset->SetString( file.c_str() );
+      unitset->SetString( file.c_str() );
     }
     break; }
   }
@@ -415,72 +419,53 @@ EdMissionSetupWindow::EdMissionSetupWindow( Mission &mission, View *view ) :
   SetSize( MIN(sfont->Width() * 40 + 20, view->Width()),
            MIN(sfont->Height() * 11 + 110, view->Height()) );
 
-  static const char *msg_labels[] = { "Level Info", "Briefing 1",
-                                      "Briefing 2", "Sequel Map", 0 };
+  static const char *msg_labels[] = { "Level Name", "Level Info",
+                                      "Name 1", "Name 2",
+                                      "Briefing 1", "Briefing 2",
+                                      "Campaign Name", "Campaign Info", 0 };
   static const char *type_labels[] = { "Solo", "Duel", 0 };
 
-  Player &p1 = mission.GetPlayer(PLAYER_ONE);
-  Player &p2 = mission.GetPlayer(PLAYER_TWO);
   unsigned short wdh = sfont->Height() + 8, wdx, wdy = 5;
   Widget *wd;
 
-  wdx = sfont->Width() * 8 + 10;
-  wd = new StringWidget( S_ID_PLAYER1,
-       wdx, wdy, w/2 - wdx - 5, wdh,
-       p1.Name(), 15, WIDGET_ALIGN_LEFT, "Player _1", this );
-  wd->SetHook( this );
-
-  wd = new StringWidget( S_ID_PLAYER2,
-       wdx * 2 + wd->Width(), wdy, wd->Width(), wdh,
-       p2.Name(), 15, WIDGET_ALIGN_LEFT, "Player _2", this );
-  wd->SetHook( this );
-           
-  wdy += wdh + 5;
-  wd = new ButtonWidget( B_ID_BRIEFING1, 5, wdy,
-       w/2 - 10, wdh, 0, "_Briefing Player 1", this );
-  wd->SetHook( this );
-
-  wd = new ButtonWidget( B_ID_BRIEFING2, w/2 + 5, wdy,
-       wd->Width(), wdh, 0, "B_riefing Player 2", this );
-  wd->SetHook( this );
-
   wdx = sfont->Width() * 4 + 10;
-  wdy += wd->Height() + 5;
-  showmsg = new CycleWidget( B_ID_PLAYERS, wdx, wdy,
+  wd = new CycleWidget( B_ID_PLAYERS, wdx, wdy,
             w/2 - wdx - 5, wdh, WIDGET_ALIGN_LEFT,
             "_Type", mission.GetNumPlayers()-1, type_labels, this );
-  showmsg->SetHook( this );
+  wd->SetHook( this );
 
   wdx = w/2 + sfont->Width() * 8 + 10;
-  wd = new StringWidget( S_ID_PASSWORD, wdx, wdy, w - wdx - 5, wdh,
-       mission.GetPassword(), 7, WIDGET_ALIGN_LEFT,
-       "_Password", this );
+  wd = new StringWidget( S_ID_SEQUEL, wdx, wdy, w - wdx - 5, wdh,
+       mission.GetSequel(), 20, WIDGET_ALIGN_LEFT, "_Next Map", this );
   wd->SetHook( this );
 
-  wdy += wd->Height() + 5;
-  wd = new ButtonWidget( B_ID_INFOMSG, 5, wdy, w/2 - 10, wdh,
-       0, "Level _Info", this );
+  wdy += wdh + 5;
+  wd = new StringWidget( S_ID_MUSIC, wdx, wdy, w - wdx - 5, wdh,
+       mission.GetMusic(), 20, WIDGET_ALIGN_LEFT, "M_usic", this );
   wd->SetHook( this );
 
-  wd = new ButtonWidget( B_ID_NEXTMAP, w/2 + 5, wdy,
-       wd->Width(), wdh, 0, "_Sequel", this );
+  wd = new CheckboxWidget( B_ID_CAMPAIGN, 5, wdy + (wdh - DEFAULT_CBW_SIZE)/2,
+       DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, mission.IsCampaign(),
+       WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
+       "Part of a _campaign", this );
+  wd->SetHook( this );
+
+  wdy += wdh + 10;
+  wd = new ButtonWidget( B_ID_SET, w - 15 - sfont->Width() * 3, wdy,
+       sfont->Width() * 3 + 10, wdh, 0, "_Set", this );
   wd->SetHook( this );
 
   // the user can use the following widget to choose which message to
-  // display in the textbox below: the level info or one of the
-  // briefings
-  wdx = sfont->Width() * 12 + 10;
-  wdy += wd->Height() + 10;
-  showmsg = new CycleWidget( B_ID_SHOWMSG, wdx, wdy, w - wdx - 5, wdh,
-            WIDGET_ALIGN_LEFT, "Show _Message", 0, msg_labels,
-            this );
+  // display in the textbox below
+  wdx = sfont->Width() * 7 + 10;
+  showmsg = new CycleWidget( B_ID_SHOWMSG, wdx, wdy,
+            wd->LeftEdge() - wdx - 5, wdh, WIDGET_ALIGN_LEFT,
+            "_Message", 0, msg_labels, this );
   showmsg->SetHook( this );
 
   wdy += wdh + 5;
   msg = new TextScrollWidget( 0, 5, wdy,
-        w - 10, h - wdh - 5 - wdy,
-        (mission.GetLevelInfoMsg() == -1) ? NULL :
-                 mission.GetMessage(mission.GetLevelInfoMsg()),
+        w - 10, h - wdh - 5 - wdy, mission.GetMessage(mission.GetName()),
         0, NULL, this );
 
   new ButtonWidget( GUI_CLOSE, 1, h - wdh - 1, w - 2, wdh,
@@ -504,63 +489,109 @@ GUI_Status EdMissionSetupWindow::WidgetActivated( Widget *widget, Window *win )
   unsigned char pid;
 
   switch ( widget->ID() ) {
-  case B_ID_BRIEFING1:
-  case B_ID_BRIEFING2:
-    new ListSelectWindow( "Select briefing", tlist,
-        mission.GetPlayer(widget->ID()-B_ID_BRIEFING1).Briefing(),
-        true, this, widget->ID() + 2, view );
-    break;
-
-  case B_ID_BRIEFING1_OK:
-  case B_ID_BRIEFING2_OK:
-    pid = widget->ID() - B_ID_BRIEFING1_OK;
+  case B_ID_BRIEF1:
+  case B_ID_BRIEF2:
+    pid = widget->ID() - B_ID_BRIEF1;
     n = static_cast<ListSelectWindow *>(win)->Selected();
 
     mission.GetPlayer(pid).SetBriefing( n ? n->ID() : -1 );
-    if ( showmsg->GetValue() == pid + 1 )
-      msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
+    msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
     break;
 
-  case B_ID_INFOMSG:
-    new ListSelectWindow( "Select info message", tlist,
-        mission.GetLevelInfoMsg(), true, this,
-        B_ID_INFOMSG_OK, view );
+  case B_ID_NAME:
+    n = static_cast<ListSelectWindow *>(win)->Selected();
+    mission.SetName( n ? n->ID() : -1 );
+    msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
     break;
 
-  case B_ID_INFOMSG_OK:
+  case B_ID_INFO:
     n = static_cast<ListSelectWindow *>(win)->Selected();
     mission.SetLevelInfoMsg( n ? n->ID() : -1 );
-    if ( showmsg->GetValue() == 0 )
-      msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
+    msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
     break;
 
-  case B_ID_NEXTMAP:
-    new ListSelectWindow( "Select next map name", tlist,
-        mission.GetSequel(), true, this, B_ID_NEXTMAP_OK, view );
+  case B_ID_CAMPAIGN:
+    mission.SetCampaign( widget->Clicked() );
     break;
 
-  case B_ID_NEXTMAP_OK:
+  case B_ID_CAMPAIGN_INFO:
     n = static_cast<ListSelectWindow *>(win)->Selected();
-    mission.SetSequel( n ? n->ID() : -1 );
-    if ( showmsg->GetValue() == 3 )
-      msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
+    mission.SetCampaignInfo( n ? n->ID() : -1 );
+    msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
     break;
 
-  case S_ID_PLAYER1:
-  case S_ID_PLAYER2: {
-    const char *pname = static_cast<StringWidget *>(widget)->String();
-    pid = widget->ID() - S_ID_PLAYER1;
-    mission.GetPlayer(pid).SetName( pname ? pname : "" );
+  case B_ID_CAMPAIGN_NAME:
+    n = static_cast<ListSelectWindow *>(win)->Selected();
+    mission.SetCampaignName( n ? n->ID() : -1 );
+    msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
+    break;
+
+  case S_ID_SEQUEL:
+    mission.SetSequel( static_cast<StringWidget *>(widget)->String() );
+    break;
+
+  case S_ID_MUSIC:
+    mission.SetMusic( static_cast<StringWidget *>(widget)->String() );
+    break;
+
+  case B_ID_PLAYER1:
+  case B_ID_PLAYER2: {
+    pid = widget->ID() - B_ID_PLAYER1;
+    n = static_cast<ListSelectWindow *>(win)->Selected();
+
+    const char *name = mission.GetMessage(n->ID());
+    if ( name && strlen(name) > 30 ) {
+      new NoteWindow( "Error",
+                      "Player names must not be longer than 30 characters.",
+                      0, view );
+    } else {
+      mission.GetPlayer(pid).SetNameID( n ? n->ID() : -1 );
+      if ( showmsg->GetValue() == pid + 1 )
+        msg->SetText( name ? name : NULL );
+    }
+    break; }
+
+  case B_ID_SET: {
+    short msgid;
+    bool optional = true;
+    const char *label;
+
+    switch ( showmsg->GetValue() ) {
+    case B_ID_NAME: msgid = mission.GetName(); optional = false;
+                    label = "Select name"; break;
+    case B_ID_INFO: msgid = mission.GetLevelInfoMsg();
+                    label = "Select info"; break;
+    case B_ID_PLAYER1: msgid = mission.GetPlayer(PLAYER_ONE).NameID(); optional = false;
+                    label = "Select name"; break;
+    case B_ID_PLAYER2: msgid = mission.GetPlayer(PLAYER_TWO).NameID(); optional = false;
+                    label = "Select name"; break;
+    case B_ID_BRIEF1: msgid = mission.GetPlayer(PLAYER_ONE).Briefing();
+                    label = "Select briefing"; break;
+    case B_ID_BRIEF2: msgid = mission.GetPlayer(PLAYER_TWO).Briefing();
+                    label = "Select briefing"; break;
+    case B_ID_CAMPAIGN_NAME: msgid = mission.GetCampaignName();
+                    label = "Select campaign name"; break;
+    case B_ID_CAMPAIGN_INFO: msgid = mission.GetCampaignInfo();
+                    label = "Select campaign info"; break;
+    default: return GUI_OK;
+    }
+
+    new ListSelectWindow( label, tlist, msgid, optional,
+        this, showmsg->GetValue(), view );
     break; }
 
   case B_ID_SHOWMSG: {
     short msgid;
 
     switch ( showmsg->GetValue() ) {
-    case 0: msgid = mission.GetLevelInfoMsg(); break;
-    case 1: msgid = mission.GetPlayer(PLAYER_ONE).Briefing(); break;
-    case 2: msgid = mission.GetPlayer(PLAYER_TWO).Briefing(); break;
-    case 3: msgid = mission.GetSequel(); break;
+    case B_ID_NAME: msgid = mission.GetName(); break;
+    case B_ID_INFO: msgid = mission.GetLevelInfoMsg(); break;
+    case B_ID_PLAYER1: msgid = mission.GetPlayer(PLAYER_ONE).NameID(); break;
+    case B_ID_PLAYER2: msgid = mission.GetPlayer(PLAYER_TWO).NameID(); break;
+    case B_ID_BRIEF1: msgid = mission.GetPlayer(PLAYER_ONE).Briefing(); break;
+    case B_ID_BRIEF2: msgid = mission.GetPlayer(PLAYER_TWO).Briefing(); break;
+    case B_ID_CAMPAIGN_INFO: msgid = mission.GetCampaignInfo(); break;
+    case B_ID_CAMPAIGN_NAME: msgid = mission.GetCampaignName(); break;
     default: msgid = -1;
     }
 
@@ -570,12 +601,10 @@ GUI_Status EdMissionSetupWindow::WidgetActivated( Widget *widget, Window *win )
   case B_ID_PLAYERS:
     mission.SetNumPlayers( static_cast<CycleWidget *>(widget)->GetValue()+1 );
     break;
-
-  case S_ID_PASSWORD:
-    mission.SetPassword( static_cast<StringWidget *>(widget)->String() );
-    break;
   }
 
+  msg->Draw();
+  msg->Show();
   return GUI_OK;
 }
 
@@ -596,15 +625,22 @@ EdBuildingWindow::EdBuildingWindow( Building &bld, Mission &mission, View *view
 
   unsigned short xoff, yoff = 5, wdh = sfont->Height() + 10;
 
-  b_name = new StringWidget( B_ID_NAME, 5, yoff, w/2 - 10, wdh,
-                             b.Name(), 19, 0, NULL, this );
+  ButtonWidget *btn;
+  btn = new ButtonWidget( B_ID_NAME, 5, yoff, sfont->Width() * 4 + 10, wdh,
+                          WIDGET_DEFAULT, "_Name", this ),
+  btn->SetHook( this );
+
+  xoff = 10 + btn->Width();
+  const char *name = b.Name();
+  b_name = new StringWidget( 0, xoff, yoff, w/2 - xoff - 5, wdh,
+                             name ? name : "Error", 30, WIDGET_STR_CONST,
+                             NULL, this );
 
   xoff = w/2 + 10 + sfont->TextWidth("Player");
   b_player = new CycleWidget( B_ID_PLAYER, xoff, yoff, w - 5 - xoff, wdh,
                               0, "_Player", b.Owner(), player3_labels, this );
   yoff += 5 + wdh;
 
-  ButtonWidget *btn;
   btn = new CheckboxWidget( B_ID_WORKSHOP, 5, yoff, DEFAULT_CBW_SIZE,
                             DEFAULT_CBW_SIZE, b.IsWorkshop(),
                             WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
@@ -666,6 +702,7 @@ EdBuildingWindow::EdBuildingWindow( Building &bld, Mission &mission, View *view
       n = n2;
     }
   }
+  TLWListBuilder::BuildMsgList( mission.GetMessages(), msg_list );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -704,7 +741,6 @@ GUI_Status EdBuildingWindow::WidgetActivated( Widget *widget, Window *win ) {
   case B_ID_OK:
     view->CloseWindow( win );
     b.SetOwner( b_player->GetValue() );
-    b.SetName( b_name->String() );
     b.SetCrystals( b_crystals->Number() );
     b.SetMaxCrystals( b_maxcrystals->Number() );
     b.SetCrystalProduction( b_mining->Number() );
@@ -729,6 +765,24 @@ GUI_Status EdBuildingWindow::WidgetActivated( Widget *widget, Window *win ) {
     if ( b_crystals->Number() > b_maxcrystals->Number() )
       b_crystals->SetNumber( b_maxcrystals->Number() );
     break;
+  case B_ID_NAME:
+    new ListSelectWindow( "Select name", msg_list,
+        b.NameID(), false, this, B_ID_NAME_OK, view );
+    break;
+  case B_ID_NAME_OK: {
+    TLWNode *mn = static_cast<ListSelectWindow *>(win)->Selected();
+
+    const char *name = mission.GetMessage(mn->ID());
+    if ( name && strlen(name) > 30 ) {
+      new NoteWindow( "Error",
+                      "Shop names must not be longer than 30 characters.",
+                      0, view );
+    } else {
+      b.SetNameID( mn ? mn->ID() : -1 );
+      b.SetName( name );
+      b_name->SetString( name ? name : "Error" );
+    }
+    break; }
   }
 
   return GUI_OK;
@@ -745,6 +799,9 @@ GUI_Status EdBuildingWindow::WidgetActivated( Widget *widget, Window *win ) {
 
 EdUnitWindow::EdUnitWindow( Unit &unit, Mission &mission, View *view ) :
               Window(WIN_CENTER, view), u(unit), mission(mission), mv(0) { 
+  static const char *xp_labels[] = { "0", "1", "2", "3", "4", "5", "6", 0 };
+  static const char *size_labels[] = { "1", "2", "3", "4", "5", "6", 0 };
+
   // calculate dimensions
   SetSize( MIN(sfont->Width() * 50, view->Width()),
            MIN(sfont->Height() * 12, view->Height()) );
@@ -775,6 +832,17 @@ EdUnitWindow::EdUnitWindow( Unit &unit, Mission &mission, View *view ) :
   wd->SetHook( this );
   yoff += 5 + wdh;
 
+  xoff = 10 + sfont->TextWidth("Squad Size");
+  wd = new CycleWidget( B_ID_SIZE, xoff, yoff, w/2 - 5 - xoff, wdh,
+                        0, "Squad _Size", u.GroupSize() - 1, size_labels, this );
+  wd->SetHook( this );
+
+  xoff = w/2 + 10 + sfont->TextWidth("XP Level");
+  wd = new CycleWidget( B_ID_XP, xoff, yoff, w - 5 - xoff, wdh,
+                        0, "_XP Level", u.XPLevel(), xp_labels, this );
+  wd->SetHook( this );
+
+  yoff += 5 + wdh;
   if ( u.IsTransport() ) {
     xoff = 10 + sfont->TextWidth("Crystals");
     // maximum number of crystals should be
@@ -784,7 +852,7 @@ EdUnitWindow::EdUnitWindow( Unit &unit, Mission &mission, View *view ) :
     // widget is activated if necessary
     wd = new NumberWidget( N_ID_CRYSTALS, xoff, yoff, sfont->Width() * 6, wdh,
                            u.Crystals(), 0, u.Type()->Slots() * 10,
-                           WIDGET_ALIGN_LEFT, "Crystals", this );
+                           WIDGET_ALIGN_LEFT, "_Crystals", this );
     wd->SetHook( this );
 
     if ( !u.IsSheltered() ) {
@@ -817,6 +885,12 @@ GUI_Status EdUnitWindow::WidgetActivated( Widget *widget, Window *win ) {
   case B_ID_DIRECTION:
     u.SetDirection( static_cast<CycleWidget *>(widget)->GetValue() );
     break;
+  case B_ID_SIZE:
+    u.SetGroupSize( static_cast<CycleWidget *>(widget)->GetValue() + 1 );
+    break;
+  case B_ID_XP:
+    u.SetXP( static_cast<CycleWidget *>(widget)->GetValue() * XP_PER_LEVEL );
+    break;
   case B_ID_UNITS:
     new EdUnitsWindow( mission, u, view );
     break;
