@@ -20,10 +20,11 @@
 // mission.cpp
 ////////////////////////////////////////////////////////////////////////
 
-#include "SDL_endian.h"
+#include <iostream>
 
 #include "mission.h"
 #include "fileio.h"
+#include "strutil.h"
 #include "globals.h"
 
 const char *event_labels[] = {
@@ -31,7 +32,12 @@ const char *event_labels[] = {
       "Create Unit", "Manipulate Event", "Research", 0 };
 const char *etrigger_labels[] = {
       "Turn", "Unit Destruction", "Shop Owner", "Unit Owner",
-      "Unit Position", 0 };
+      "Unit Position", "Handicap", 0 };
+
+#define P1_COLOR_LIGHT	0x00FEA604		// yellow
+#define P1_COLOR_SHADOW	0x0056360C		// brown
+#define P2_COLOR_LIGHT	0x0000AAFF		// light blue
+#define P2_COLOR_SHADOW	0x0002265C		// dark blue
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Mission::Mission
@@ -45,11 +51,10 @@ const char *etrigger_labels[] = {
 Mission::Mission( const Point &size, TerrainSet *ts, UnitSet *us ) :
                   p1(PLAYER_ONE), p2(PLAYER_TWO) {
   flags = 0;
+  name = -1;
+  campaign_name = -1;
   level_info = -1;
-  next_map = -1;
-  msg_num = 0;
-  msg_len = 0;
-  password[0] = '\0';
+  campaign_info = -1;
   SetTitle( "Unknown" );
 
   unit_set = us;
@@ -81,65 +86,62 @@ Mission::~Mission( void ) {
 
 int Mission::Load( const char *filename ) {
   int rc = -1;
-  SDL_RWops *file = SDL_RWFromFile( filename, "rb" );
-  if ( !file ) return -1;
+  File file( filename );
+  if ( !file.Open( "rb" ) ) return -1;
 
   // read game info
-  unsigned char version;
-  unsigned long id = SDL_ReadLE32( file );
-  SDL_RWread( file, &version, 1, 1 );
-  if ( (version == FILE_VERSION) && (id == FID_MISSION) ) {
+  if ( (file.Read32() == FID_MISSION) && (file.Read8() == FILE_VERSION) ) {
     unsigned short len, i;
-    unsigned char buf;
-    char *uset, *tset;
+    signed char mapid, musicid;
+    string uset, tset;
+    Building *b;
 
     // set last filename
     last_file_name = file_part( filename );
     i = last_file_name.rfind( '.' );
     last_file_name.erase( i );
 
-    flags = SDL_ReadLE16( file );
-    i = SDL_ReadLE16( file );       // turn number - unused
+    flags = file.Read16();
+    i = file.Read16();              // turn number - unused
 
     if ( flags & GI_SAVEFILE )      // refuse to load saved games
-      goto out;
-    else if ( flags & GI_PASSWORD ) {
-      SDL_RWread( file, password, 1, 8 );
-      crypt( password );
-    } else password[0] = '\0';
+      return -1;
 
-    SDL_RWread( file, &level_info, 1, 1 );
-    SDL_RWread( file, &next_map, 1, 1 );
-    SDL_RWread( file, &buf, 1, 1 );     // current player - unused
-    SDL_RWread( file, &buf, 1, 1 );     // turn phase - unused
+    name = file.Read8();
+    level_info = file.Read8();
+    campaign_name = file.Read8();
+    campaign_info = file.Read8();
+    mapid = file.Read8();
+    musicid = file.Read8();
+    file.Read8();                   // handicap - unused
+    file.Read8();                   // current player - unused
+    file.Read8();                   // turn phase - unused
 
-    len = SDL_ReadLE16( file );        // load name of unit set
-    uset = new char [len + 1];
-    SDL_RWread( file, uset, 1, len );
-    uset[len] = '\0';
+    len = file.Read16();            // load name of unit set
+    uset = file.ReadS( len );
 
-    len = SDL_ReadLE16( file );        // load name of terrain set
-    tset = new char [len + 1];
-    SDL_RWread( file, tset, 1, len );
-    tset[len] = '\0';
+    len = file.Read16();            // load name of terrain set
+    tset = file.ReadS( len );
 
-    map.Load( file );                  // load map
+    map.Load( file );               // load map
 
     unit_set = new UnitSet;
-    rc = unit_set->Load( uset );
-    delete [] uset;
-    if ( rc ) {
-      fprintf( stderr, "Error: Unit set not available.\n" );
-      goto out;
+    string usetname( get_data_dir() + uset + ".units" );
+    File ufile( usetname );
+    if ( !ufile.Open("rb") || unit_set->Load( ufile, uset.c_str() ) ) {
+      cerr << "Error: Unit set '" << uset << "' not available." << endl;
+      return -1;
     }
+    ufile.Close();
 
     terrain_set = new TerrainSet;
-    rc = terrain_set->Load( tset );
-    delete [] tset;
-    if ( rc ) {
-      fprintf( stderr, "Error: Terrain set not available.\n" );
-      goto out;
+    string tsetname( get_data_dir() + tset + ".tiles" );
+    File tfile( tsetname );
+    if ( !tfile.Open("rb") || terrain_set->Load( tfile, tset.c_str() ) ) {
+      cerr << "Error: Terrain set '" << tset << "' not available." << endl;
+      return -1;
     }
+    tfile.Close();
 
     map.SetUnitSet( unit_set );
     map.SetTerrainSet( terrain_set );
@@ -147,18 +149,17 @@ int Mission::Load( const char *filename ) {
     p1.Load( file );
     p2.Load( file );
 
-    len = SDL_ReadLE16( file );         // load buildings
+    len = file.Read16();                // load buildings
     for ( i = 0; i < len; ++i ) {
-      Building *b = new Building();
+      b = new Building();
       b->Load( file );
       buildings.AddTail( b );
       map.SetBuilding( b, b->Position() );
     }
 
-    len = SDL_ReadLE16( file );         // load units
+    len = file.Read16();                // load units
     for ( i = 0; i < len; ++i ) {
-      unsigned char tid;
-      SDL_RWread( file, &tid, 1, 1 );
+      unsigned char tid = file.Read8();
 
       Unit *u = new Unit( file, unit_set->GetUnitInfo(tid) );
       if ( u ) {
@@ -169,11 +170,11 @@ int Mission::Load( const char *filename ) {
       }
     }
 
-    len = SDL_ReadLE16( file );         // load combats
+    len = file.Read16();                // load battles
 
     // combat actions - only present in saved games
 
-    len = SDL_ReadLE16( file );         // load events
+    len = file.Read16();                // load events
     for ( i = 0; i < len; ++i ) {
       Event *e = new Event();
       e->Load( file );
@@ -181,35 +182,25 @@ int Mission::Load( const char *filename ) {
     }
 
     // load text messages
-    msg_num = SDL_ReadLE16( file );             // number of messages
-    msg_len = SDL_ReadLE32( file );
-    if ( msg_num ) {                            // load messages
-      char *ptr, *msgs = new char[msg_len];
+    messages.Load( file );
 
-      SDL_RWread( file, msgs, 1, msg_len );
+    Language internal_messages;
+    internal_messages.ReadCatalog( file );
+    SetSequel( internal_messages.GetMsg( mapid ) );
+    SetMusic( internal_messages.GetMsg( musicid ) );
 
-      i = 0;
-      ptr = msgs;
-      while ( i < msg_num ) {
-        crypt( ptr );
-        messages.AddTail( new Message( ptr ) );
-        ++i;
+    // set shop names
+    for ( b = static_cast<Building *>(buildings.Head());
+          b; b = static_cast<Building *>(b->Next()) )
+      b->SetName( GetMessage( b->NameID() ) );
 
-        while ( *ptr != '\0' ) ++ptr;
-        ++ptr;
-      }
-      delete [] msgs;
-    }
-
-    len = SDL_ReadLE16( file );
+    len = file.Read16();
 
     // history - only present in saved games
 
     rc = 0;
   }
 
-out: 
-  SDL_RWclose( file );
   return rc;
 }
 
@@ -221,53 +212,59 @@ out:
 ////////////////////////////////////////////////////////////////////////
 
 int Mission::Save( const char *filename ) {
-  SDL_RWops *file = SDL_RWFromFile( filename, "wb" );
-  if ( !file ) {
-    fprintf( stderr, "Failed to save mission!" );
+  File file( filename );
+  if ( !file.Open( "wb" ) ) {
+    cerr << "Failed to save mission!" << endl;
     return -1;
   }
 
-  unsigned char version = FILE_VERSION, current_player = PLAYER_ONE,
-                turn_phase = TURN_START;
-  unsigned short len, num;
+  unsigned short num;
+  signed char mapid = -1, musicid = -1;
+  Language internal_messages;
+  internal_messages.SetID( "--" );
+  if ( !next_map.empty() ) {
+    mapid = internal_messages.Size();
+    internal_messages.AddMsg( next_map );
+  }
+  if ( !music.empty() ) {
+    musicid = internal_messages.Size();
+    internal_messages.AddMsg( music );
+  }
 
   // set last filename
   last_file_name = file_part( filename );
   num = last_file_name.rfind( '.' );
   last_file_name.erase( num );
 
-  // save game info
-  SDL_WriteLE32( file, FID_MISSION );
-  SDL_RWwrite( file, &version, 1, 1 );
+  file.Write32( FID_MISSION );
+  file.Write8( FILE_VERSION );
 
-  SDL_WriteLE16( file, flags );
-  SDL_WriteLE16( file, 1 );        // turn
+  file.Write16( flags );
+  file.Write16( 1 );        // turn
 
-  if ( flags & GI_PASSWORD ) {
-    crypt( password );
-    SDL_RWwrite( file, password, 1, 8 );
-    crypt( password );
-  }
+  file.Write8( name );
+  file.Write8( level_info );
+  file.Write8( campaign_name );
+  file.Write8( campaign_info );
+  file.Write8( mapid );
+  file.Write8( musicid );
+  file.Write8( HANDICAP_NONE );
+  file.Write8( PLAYER_ONE );
+  file.Write8( TURN_START );
 
-  SDL_RWwrite( file, &level_info, 1, 1 );
-  SDL_RWwrite( file, &next_map, 1, 1 );
-  SDL_RWwrite( file, &current_player, 1, 1 );
-  SDL_RWwrite( file, &turn_phase, 1, 1 );
-
-  len = unit_set->GetName().length();            // save mission set info
-  SDL_WriteLE16( file, len );
-  SDL_RWwrite( file, unit_set->GetName().c_str(), 1, len );
-  len = terrain_set->GetName().length();
-  SDL_WriteLE16( file, len );
-  SDL_RWwrite( file, terrain_set->GetName().c_str(), 1, len );
+  // save mission set info
+  file.Write16( unit_set->GetName().length() );
+  file.WriteS( unit_set->GetName() );
+  file.Write16( terrain_set->GetName().length() );
+  file.WriteS( terrain_set->GetName() );
 
   map.Save( file );            // save map
 
-  p1.Save( file );              // save player data
+  p1.Save( file );             // save player data
   p2.Save( file );
 
-  num = buildings.CountNodes();                 // save buildings
-  SDL_WriteLE16( file, num );
+  // save buildings
+  file.Write16( buildings.CountNodes() );
   for ( Building *b = static_cast<Building *>( buildings.Head() );
         b; b = static_cast<Building *>( b->Next() ) ) b->Save( file );
 
@@ -277,8 +274,7 @@ int Mission::Save( const char *filename ) {
   // make two passes through the list, first saving all unsheltered
   // units (which are possibly transports carrying other units), and
   // all sheltered units in the second run
-  num = units.CountNodes();
-  SDL_WriteLE16( file, num ); 
+  file.Write16( units.CountNodes() ); 
   Unit *u;
   for ( u = static_cast<Unit *>( units.Head() );
         u; u = static_cast<Unit *>( u->Next() ) ) {    // make sure transports are
@@ -290,23 +286,19 @@ int Mission::Save( const char *filename ) {
     if ( u->IsSheltered() ) u->Save( file );
   }
 
-  num = 0;                                             // save combat data
-  SDL_WriteLE16( file, num );
+  file.Write16( 0 );        // save combat data
 
-  num = events.CountNodes();                           // save events
-  SDL_WriteLE16( file, num );
+  // save events
+  file.Write16( events.CountNodes() );
   for ( Event *e = static_cast<Event *>( events.Head() );
         e; e = static_cast<Event *>( e->Next() ) )
     e->Save( file );
 
-  SDL_WriteLE16( file, msg_num );                     // save text messages
-  SDL_WriteLE32( file, msg_len );
-  for ( Message *m = static_cast<Message *>( messages.Head() );
-        m; m = static_cast<Message *>( m->Next() ) ) m->Save( file );
+  messages.Save( file );
+  internal_messages.WriteCatalog( file );
 
-  SDL_WriteLE16( file, 0 );                           // turn history
+  file.Write16( 0 );        // turn history
 
-  SDL_RWclose( file );
   return 0;
 }
 
@@ -326,14 +318,20 @@ int Mission::Export( const string &filename ) const {
   title.erase( suffix );
 
   file << "[mission]\n";
-  file << "title = " << title << '\n';
+  file << "name = " << (short)name << '\n';
   file << "mapwidth = " << map.Width() << '\n';
   file << "mapheight = " << map.Height() << '\n';
   file << "info = " << (short)level_info << '\n';
-  file << "nextmap = " << (short)next_map << '\n';
+  file << "campaign = " << (((flags & GI_CAMPAIGN) != 0) ? '1' : '0') << '\n';
+  file << "campaignname = " << (short)campaign_name << '\n';
+  file << "campaigninfo = " << (short)campaign_info << '\n';
+
+  if ( !next_map.empty() )
+    file << "nextmap = " << next_map << '\n';
+  if ( !music.empty() )
+    file << "music = " << music << '\n';
+
   file << "players = " << (((flags & GI_AI) != 0) ? '1' : '2') << '\n';
-  if ( password[0] != '\0' )
-    file << "password = " << password << '\n';
   file << "tileset = " << terrain_set->GetName() << '\n';
   file << "unitset = " << unit_set->GetName() << "\n\n";
 
@@ -358,18 +356,21 @@ int Mission::Export( const string &filename ) const {
         e = static_cast<Event *>(e->Next()) )
     e->Export( file, unit_set );
 
-  if ( messages.CountNodes() > 0 ) {
-    file << "[messages]";
+  if ( messages.GetMsg(0) != 0 ) {
+    const std::map<const string, Language> &lib = messages.GetLibrary();
+    for ( std::map<const string, Language>::const_iterator iter = lib.begin();
+          iter != lib.end(); ++iter ) {
 
-    for ( Message *m = static_cast<Message *>(messages.Head()); m;
-          m = static_cast<Message *>(m->Next()) ) {
-      file << '\n';
-      m->Export( file );
-      file << '%';
+      file << "[messages(" << iter->first << ")]\n";
+      file << iter->second.GetMsg(0);
+
+      const char *m;
+      for ( int i = 1; (m = iter->second.GetMsg(i)) != 0; ++i ) {
+        file << "\n%\n" << m;
+      }
+      file << "\n[/messages]\n\n";
     }
-    file << "%\n\n";
   }
-
   file.close();
   return 0;
 }
@@ -390,7 +391,10 @@ Unit *Mission::CreateUnit( const UnitType *type, unsigned char pid, const Point
     units.AddTail( u );
 
     if ( !map.GetMapObject( pos ) ) map.SetUnit( u, pos );
-    else u->SetFlags( U_SHELTERED );
+    else {
+      u->SetFlags( U_SHELTERED );
+      if ( u->IsMine() ) u->SetMoves( MCOST_MIN );
+    }
   }
   return u;
 }
@@ -473,7 +477,7 @@ unsigned short Mission::GetUnitID( void ) const {
 ////////////////////////////////////////////////////////////////////////
 
 Building *Mission::CreateBuilding( unsigned char pid, const Point &pos ) {
-  Building *b = new Building( pos, GetBuildingID(), pid, "Depot" );
+  Building *b = new Building( pos, GetBuildingID(), pid );
   if ( b ) {
     buildings.AddTail( b );
     map.SetBuilding( b, pos );
@@ -610,38 +614,6 @@ unsigned char Mission::GetEventID( void ) const {
   return id;
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : Mission::GetMessage
-// DESCRIPTION: Get a message from the catalog.
-// PARAMETERS : id - message identifier of the requested message
-// RETURNS    : pointer to requested message if successful, 0 otherwise
-////////////////////////////////////////////////////////////////////////
-
-const char *Mission::GetMessage( short id ) const {
-  if ( id >= 0 ) {
-    Message *msg = static_cast<Message *>(messages.GetNode(id));
-    if ( msg ) return msg->GetText().c_str();
-  }
-  return 0;
-}
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : Mission::SetPassword
-// DESCRIPTION: Set the map access code for campaign missions.
-// PARAMETERS : pass - password (may be NULL)
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void Mission::SetPassword( const char *pass ) {
-
-  for ( int i = 0; i < 8; ++i ) password[i] = '\0';
-
-  if ( pass ) {
-    strncpy( password, pass, 7 );
-    flags |= GI_PASSWORD;
-  } else flags &= ~GI_PASSWORD;
-}
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Mission::SetNumPlayers
 // DESCRIPTION: Set the intended number of human players for this map.
@@ -666,8 +638,8 @@ void Mission::SetNumPlayers( unsigned char num ) {
 //              returns 0
 ////////////////////////////////////////////////////////////////////////
 
-unsigned short Mission::StorageLeft( Unit &u ) const {
-  unsigned short space;
+short Mission::StorageLeft( Unit &u ) const {
+  short space;
 
   if ( u.IsTransport() ) {
     space = u.Type()->Slots() - ((u.Crystals() + 9) / 10);
@@ -689,49 +661,85 @@ unsigned short Mission::StorageLeft( Unit &u ) const {
 //              mistakes concerning settings which can not directly be
 //              enforced by CoMET (e.g. you could create an event
 //              targetting a unit and delete that unit afterwards).
-// PARAMETERS : errors - string object; for each error found, a short
+// PARAMETERS : errors - stringstream; for each error found, a short
 //                       message will be appended which can afterwards
 //                       be presented to the user
 // RETURNS    : number of errors and warnings detected
 ////////////////////////////////////////////////////////////////////////
 
-unsigned short Mission::Validate( string &errors ) {
+unsigned short Mission::Validate( stringstream &errors ) {
   unsigned short errs = 0;
-  char buf[256];
+
+  if ( name == -1 ) {
+    ++errs;
+    errors << "Error: No level name configured.\n\n";
+  } else if ( !GetMessage(name) ) {
+    ++errs;
+    errors << "Error: Invalid level name " << (short)name << " configured.\n\n";
+  }
 
   Player *pls[2] = { &p1, &p2 };
   for ( int i = 0; i < 2; ++i ) {
+    if ( pls[i]->NameID() == -1 ) {
+      ++errs;
+      errors << "Error: No name set for player " << i+1 << ".\n\n";
+    } else if ( !GetMessage(pls[i]->NameID()) ) {
+      ++errs;
+      errors << "Error: Invalid name " << (short)pls[i]->NameID()
+             << " set for player " << i+1 << ".\n\n";
+    }
+
     if ( pls[i]->Briefing() == -1 ) {
       ++errs;
-      sprintf( buf, "Warning: No briefing set for %s.\n\n", pls[i]->Name() );
-      errors.append( buf );
+      errors << "Warning: No briefing set for player " << i+1 << ".\n\n";
     } else if ( !GetMessage(pls[i]->Briefing()) ) {
       ++errs;
-      sprintf( buf, "Error: Invalid briefing %d set for %s.\n\n",
-               pls[i]->Briefing(), pls[i]->Name() );
-      errors.append( buf );
+      errors << "Error: Invalid briefing " << (short)pls[i]->Briefing()
+             << " set for player " << i+1 << ".\n\n";
     }
   }
 
   if ( level_info == -1 ) {
     ++errs;
-    errors.append( "Warning: No level info message configured.\n\n" );
+    errors << "Warning: No level info message configured.\n\n";
   } else if ( !GetMessage(level_info) ) {
     ++errs;
-    sprintf( buf, "Error: Invalid level info message %d configured.\n\n",
-             level_info );
-    errors.append( buf );
+    errors << "Error: Invalid level info message " << (short)level_info << " configured.\n\n";
+  }
+
+  if ( (campaign_name != -1) && !GetMessage(campaign_name) ) {
+    ++errs;
+    errors << "Error: Invalid campaign name " << (short)campaign_name << " configured.\n\n";
+  }
+
+  if ( (campaign_info != -1) && !GetMessage(campaign_info) ) {
+    ++errs;
+    errors << "Error: Invalid campaign info message " << (short)campaign_info << " configured.\n\n";
+  } else if ( (campaign_name != -1) && (campaign_info == -1) ) {
+    ++errs;
+    errors << "Warning: Campaign name set, but campaign info unset.\n\n";
+  } else if ( (campaign_name == -1) && (campaign_info != -1) ) {
+    ++errs;
+    errors << "Warning: Campaign info set, but campaign name unset.\n\n";
+  }
+
+  if ( !IsCampaign() && (GetSequel() || (campaign_name != -1) || (campaign_info != -1)) ) {
+    ++errs;
+    errors << "Error: Campaign name, info, or sequel set, but map is not part of a campaign.\n\n";
   }
 
   errs += ValidateMap( errors );
 
-  for ( Node *n = events.Head(); n; n = n->Next() )
+  Node *n;
+  for ( n = events.Head(); n; n = n->Next() )
     errs += ValidateEvent( *static_cast<Event *>(n), errors );
 
-  sprintf( buf, "Validation summary\n"
-                "------------------\n"
-                "Detected %d errors or warnings.", errs );
-  errors.append( buf );
+  for ( n = buildings.Head(); n; n = n->Next() )
+    errs += ValidateShop( *static_cast<Building *>(n), errors );
+
+  errors << "Validation summary\n"
+         << "------------------\n"
+         << "Detected " << errs << " errors or warnings.";
 
   return errs;
 }
@@ -739,34 +747,35 @@ unsigned short Mission::Validate( string &errors ) {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Mission::ValidateMap
 // DESCRIPTION: Check the map for errors.
-// PARAMETERS : errors - error string
+// PARAMETERS : errors - error stream
 // RETURNS    : number of errors and warnings detected
 ////////////////////////////////////////////////////////////////////////
 
-unsigned short Mission::ValidateMap( string &errors ) const {
+unsigned short Mission::ValidateMap( stringstream &errors ) const {
   unsigned short errs = 0;
-  char buf[512];
   Point p;
 
   for ( p.y = 0; p.y < map.Height(); ++p.y ) {
     for ( p.x = 0; p.x < map.Width(); ++p.x ) {
 
-      if ( map.IsBuilding( p ) && !map.GetBuilding( p ) ) {
+      Building *b = map.GetBuilding( p );
+      bool isshop = map.IsBuilding( p );
+      if ( isshop && !b ) {
         ++errs;
-        sprintf( buf, "Error: Tile at %d/%d denotes a shop "
-                      "entrance, but no shop has been created.\n\n",
-                 p.x, p.y );
-        errors.append( buf );
+        errors << "Error: Tile at " << p.x << '/' << p.y
+               << " denotes a shop entrance, but no shop has been created.\n\n";
+      } else if ( !isshop && b ) {
+        ++errs;
+        errors << "Error: A shop exists at " << p.x << '/' << p.y
+               << ", but there is no entrance.\n\n";
       }
 
       Unit *u = map.GetUnit( p );
       if ( u && !u->IsSheltered() &&
            ((map.TerrainTypes(p) & u->Terrain()) == 0) ) {
         ++errs;
-        sprintf( buf, "Warning: '%s' (%d) at %d/%d "
-                      "is walking on invalid terrain.\n\n",
-                 u->Name(), u->ID(), p.x, p.y );
-        errors.append( buf );
+        errors << "Warning: '" << u->Name() << "' (" << u->ID() << ") at "
+               << p.x << '/' << p.y << " walking on invalid terrain.\n\n";
       }
     }
   }
@@ -776,26 +785,24 @@ unsigned short Mission::ValidateMap( string &errors ) const {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Mission::ValidateEvent
 // DESCRIPTION: Check the event for errors.
-// PARAMETERS : errors - error string
+// PARAMETERS : e - event to check
+//              errors - error stream
 // RETURNS    : number of errors and warnings detected
 ////////////////////////////////////////////////////////////////////////
 
-unsigned short Mission::ValidateEvent( Event &e, string &errors ) const {
+unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
   unsigned short errs = 0;
-  char buf[512];
 
   if ( (e.Title() != -1) && !GetMessage(e.Title()) ) {
     ++errs;
-    sprintf( buf, "Error: Invalid title %d for %s (%d) event.\n\n",
-             e.Title(), e.Name(), e.ID() );
-    errors.append( buf );
+    errors << "Error: Invalid title " << (short)e.Title() << " for "
+           << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
   }
 
   if ( (e.Message() != -1) && !GetMessage(e.Message()) ) {
     ++errs;
-    sprintf( buf, "Error: Invalid message %d for %s (%d) event.\n\n",
-             e.Message(), e.Name(), e.ID() );
-    errors.append( buf );
+    errors << "Error: Invalid message " << (short)e.Message() << " for "
+           << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
   }
 
 
@@ -803,10 +810,8 @@ unsigned short Mission::ValidateEvent( Event &e, string &errors ) const {
   case ETRIGGER_HAVE_BUILDING:
     if ( (e.GetTData(0) == -1) || !GetBuildingByID(e.GetTData(0)) ) {
       ++errs;
-      sprintf( buf, "Error: Trigger of %s (%d) event targets "
-                    "non-existing shop with ID %d.\n\n",
-               e.Name(), e.ID(), e.GetTData(0) );
-      errors.append( buf );
+      errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
+             << ") event targets non-existing shop with ID " << e.GetTData(0) << ".\n\n";
     }
     break;
 
@@ -814,10 +819,8 @@ unsigned short Mission::ValidateEvent( Event &e, string &errors ) const {
   case ETRIGGER_UNIT_POSITION:
     if ( (e.GetTData(0) == -1) || !GetUnitByID(e.GetTData(0)) ) {
       ++errs;
-      sprintf( buf, "Error: Trigger of %s (%d) event targets "
-                    "non-existing unit with ID %d.\n\n",
-               e.Name(), e.ID(), e.GetTData(0) );
-      errors.append( buf );
+      errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
+             << ") event targets non-existing unit with ID " << e.GetTData(0) << ".\n\n";
     }
     break;
 
@@ -830,16 +833,13 @@ unsigned short Mission::ValidateEvent( Event &e, string &errors ) const {
 
       if ( !u ) {
         ++errs;
-        sprintf( buf, "Error: Trigger of %s (%d) event targets "
-                      "non-existing unit with ID %d.\n\n",
-                 e.Name(), e.ID(), e.GetTData(0) );
-        errors.append( buf );
+        errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
+               << ") event targets non-existing unit with ID " << e.GetTData(0) << ".\n\n";
       } else if ( u->Owner() != e.GetTData(1) ) {
         ++errs;
-        sprintf( buf, "Warning: Trigger of %s (%d) event had "
-                      "incorrect owner of %s (%d). Fixed.\n\n",
-                 e.Name(), e.ID(), u->Name(), u->ID() );
-        errors.append( buf );
+        errors << "Warning: Trigger of " << e.Name() << " (" << (short)e.ID()
+               << ") event had incorrect owner of " << u->Name()
+               << " (" << u->ID() << "). Fixed.\n\n";
         e.SetTData( 1, u->Owner() );
       }
     }
@@ -847,53 +847,87 @@ unsigned short Mission::ValidateEvent( Event &e, string &errors ) const {
 
   default:
     ++errs;
-    sprintf( buf, "Warning: Unknown event trigger %s for %s (%d) event.\n\n",
-             e.TriggerName(), e.Name(), e.ID() );
-    errors.append( buf );
+    errors << "Warning: Unknown event trigger " << e.TriggerName() << " for "
+           << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
   }
 
 
   switch ( e.Type() ) {
   case EVENT_CREATE_UNIT:
+    break;
+
   case EVENT_MINING:
-  case EVENT_RESEARCH: {
-    int i = (e.Type() == EVENT_CREATE_UNIT) ? 1 : 0;
-    if ( (e.GetData(i) == -1) || !GetBuildingByID(e.GetData(i)) ) {
+  case EVENT_RESEARCH:
+    if ( (e.GetData(0) == -1) || !GetBuildingByID(e.GetData(0)) ) {
       ++errs;
-      sprintf( buf, "Error: %s (%d) event targets "
-                    "non-existing shop with ID %d.\n\n",
-               e.Name(), e.ID(), e.GetData(i) );
-      errors.append( buf );
+      errors << "Error: " << e.Name() << " (" << (short)e.ID()
+             << ") event targets non-existing shop with ID " << e.GetData(0) << ".\n\n";
     }
-    break; }
+    break;
 
   case EVENT_MANIPULATE_EVENT:
     if ( (e.GetData(0) == -1) || !GetEventByID(e.GetData(0)) ) {
       ++errs;
-      sprintf( buf, "Error: %s (%d) event targets "
-                    "non-existing event with ID %d.\n\n",
-               e.Name(), e.ID(), e.GetData(0) );
-      errors.append( buf );
+      errors << "Error: " << e.Name() << " (" << (short)e.ID()
+             << ") event targets non-existing event with ID " << e.GetData(0) << ".\n\n";
     }
     break;
 
   case EVENT_NEXT_MAP:
     if ( (e.GetData(0) != -1) && !GetMessage(e.GetData(0)) ) {
     ++errs;
-    sprintf( buf, "Error: Invalid mission file name %d for %s (%d) event.\n\n",
-             e.GetData(0), e.Name(), e.ID() );
-    errors.append( buf );
+    errors << "Error: Invalid mission file name " << e.GetData(0) << " for "
+           << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
   }
 
   case EVENT_MESSAGE:
+    break;
   case EVENT_SCORE:
+    if ( (e.GetData(1) != -1) && !GetMessage(e.GetData(1)) ) {
+      ++errs;
+      errors << "Error: Invalid message " << e.GetData(1) << " for "
+             << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
+    }
+
+    if ( (e.GetData(1) != -1) && (e.GetData(2) != -1) && !GetMessage(e.GetData(2)) ) {
+      ++errs;
+      errors << "Error: Event " << (short)e.ID() << " references illegal message "
+             << e.GetData(2) << ".\n\n";
+    } else if ( (e.GetData(1) == -1) && (e.GetData(2) != -1) ) {
+      ++errs;
+      errors << "Warning: Event " << (short)e.ID() << " set a title but no message.\n\n";
+    }
     break;
 
   default:
     ++errs;
-    sprintf( buf, "Warning: Unknown event type %s for event ID %d.\n\n",
-             e.Name(), e.ID() );
-    errors.append( buf );
+    errors << "Warning: Unknown event type " << e.Name() << " for event ID "
+           << (short)e.ID() << ".\n\n";
+  }
+
+  return errs;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : Mission::ValidateShop
+// DESCRIPTION: Check the shop for errors.
+// PARAMETERS : b      - shop to check
+//              errors - error stream
+// RETURNS    : number of errors and warnings detected
+////////////////////////////////////////////////////////////////////////
+
+unsigned short Mission::ValidateShop( Building &b, stringstream &errors ) const {
+  unsigned short errs = 0;
+
+  if ( b.NameID() == -1 ) {
+    ++errs;
+    errors << "Error: No name set for shop " << b.ID() << ".\n\n";
+  } else if ( !b.Name() ) {
+    ++errs;
+    errors << "Error: Invalid name " << (short)b.NameID() << " for shop " << b.ID() << ".\n\n";
+  } else if ( strlen(b.Name()) > 30 ) {
+    ++errs;
+    errors << "Error: Name for shop " << b.ID() << " is longer than 30 characters.\n\n";
   }
 
   return errs;
@@ -907,13 +941,14 @@ unsigned short Mission::ValidateEvent( Event &e, string &errors ) const {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-Player::Player( unsigned char id ) {
+Player::Player( unsigned char id ) :
+        p_col_light(id == PLAYER_ONE ? P1_COLOR_LIGHT : P2_COLOR_LIGHT ),
+        p_col_dark(id == PLAYER_ONE ? P1_COLOR_SHADOW : P2_COLOR_SHADOW ) {
   p_id = id;
-  p_level = 0;
   p_success = 0;
   p_briefing = -1;
-  sprintf( p_name, "Player %d", id + 1 );
-  for ( int i = 0; i < 8; ++i ) p_password[i] = '\0';
+  p_name_id = -1;
+  p_type = COMPUTER;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -923,13 +958,22 @@ Player::Player( unsigned char id ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Player::Load( SDL_RWops *file ) {
-  SDL_RWread( file, &p_id, 1, 1 );
-  SDL_RWread( file, &p_level, 1, 1 );
-  SDL_RWread( file, &p_success, 1, 1 );
-  SDL_RWread( file, &p_briefing, 1, 1 );
-  SDL_RWread( file, p_name, 1, 16 );
-  SDL_RWread( file, p_password, 1, 8 );
+int Player::Load( File &file ) {
+  p_id = file.Read8();
+  p_type = file.Read8();
+  p_success = file.Read8();
+  p_briefing = file.Read8();
+  p_name_id = file.Read8();
+
+  unsigned char len = file.Read8();
+  p_password = StringUtil::crypt(file.ReadS(len));
+
+  p_col_light.r = file.Read8();
+  p_col_light.g = file.Read8();
+  p_col_light.b = file.Read8();
+  p_col_dark.r = file.Read8();
+  p_col_dark.g = file.Read8();
+  p_col_dark.b = file.Read8();
   return 0;
 }
 
@@ -940,13 +984,21 @@ int Player::Load( SDL_RWops *file ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int Player::Save( SDL_RWops *file ) const {
-  SDL_RWwrite( file, &p_id, 1, 1 );
-  SDL_RWwrite( file, &p_level, 1, 1 );
-  SDL_RWwrite( file, &p_success, 1, 1 );
-  SDL_RWwrite( file, &p_briefing, 1, 1 );
-  SDL_RWwrite( file, p_name, 1, 16 );
-  SDL_RWwrite( file, p_password, 1, 8 );
+int Player::Save( File &file ) const {
+  file.Write8( p_id );
+  file.Write8( p_type );
+  file.Write8( p_success );
+  file.Write8( p_briefing );
+  file.Write8( p_name_id );
+  file.Write8( p_password.size() );
+  file.WriteS( StringUtil::crypt( p_password ) );
+
+  file.Write8( p_col_light.r );
+  file.Write8( p_col_light.g );
+  file.Write8( p_col_light.b );
+  file.Write8( p_col_dark.r );
+  file.Write8( p_col_dark.g );
+  file.Write8( p_col_dark.b );
   return 0;
 }
 
@@ -959,8 +1011,12 @@ int Player::Save( SDL_RWops *file ) const {
 
 int Player::Export( ofstream &file ) const {
   file << "[player]\n";
-  file << "name = " << p_name << '\n';
-  file << "briefing = " << (short)p_briefing << "\n\n";
+  file << "name = " << (short)p_name_id << '\n';
+  file << "briefing = " << (short)p_briefing << '\n';
+  file << "fcolor = " << (short)p_col_light.r << ',' << (short)p_col_light.g << ','
+                      << (short)p_col_light.b << '\n';
+  file << "bcolor = " << (short)p_col_dark.r << ',' << (short)p_col_dark.g << ','
+                      << (short)p_col_dark.b << "\n\n";
   return 0;
 }
 
@@ -995,7 +1051,7 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
     break;
   case EVENT_SCORE:
     e_data[0] = 100;       // award 100 points
-    e_data[1] = e_data[2] = 0;
+    e_data[1] = e_data[2] = -1; // don't show a message to the loser
     break;
   case EVENT_NEXT_MAP:
     e_data[0] = -1;       // prepare map with this name
@@ -1003,7 +1059,7 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
     break;
   case EVENT_CREATE_UNIT:
     e_data[0] = 0;        // build first unit type in set
-    e_data[1] = -1;       // build in shop with identifier -1
+    e_data[1] = 0;        // build at position (0, 0)
     e_data[2] = 0;
     break;
   case EVENT_RESEARCH:
@@ -1039,29 +1095,33 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
     e_tdata[1] = 0;    // x coordinate of target hex
     e_tdata[2] = 0;    // y coordinate of target hex
     break;
+  case ETRIGGER_HANDICAP:
+    e_tdata[0] = HANDICAP_NONE|HANDICAP_P1|HANDICAP_P2; // trigger always
+    e_tdata[1] = e_tdata[2] = 0;
+    break;
   }
 }
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Event::Load
 // DESCRIPTION: Load an event from a file.
-// PARAMETERS : file    - file descriptor
+// PARAMETERS : file - file descriptor
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Event::Load( SDL_RWops *file ) {
+int Event::Load( File &file ) {
   int i;
 
-  SDL_RWread( file, &e_id, 1, 1 );
-  SDL_RWread( file, &e_type, 1, 1 );
-  SDL_RWread( file, &e_trigger, 1, 1 );
-  SDL_RWread( file, &e_depend, 1, 1 );
-  for ( i = 0; i < 3; ++i ) e_tdata[i] = SDL_ReadLE16( file );
-  for ( i = 0; i < 3; ++i ) e_data[i] = SDL_ReadLE16( file );
-  e_title = SDL_ReadLE16( file );
-  e_message = SDL_ReadLE16( file );
-  e_flags = SDL_ReadLE16( file );
-  SDL_RWread( file, &e_player, 1, 1 );
+  e_id = file.Read8();
+  e_type = file.Read8();
+  e_trigger = file.Read8();
+  e_depend = file.Read8();
+  for ( i = 0; i < 3; ++i ) e_tdata[i] = file.Read16();
+  for ( i = 0; i < 3; ++i ) e_data[i] = file.Read16();
+  e_title = file.Read16();
+  e_message = file.Read16();
+  e_flags = file.Read16();
+  e_player = file.Read8();
   return 0;
 }
 
@@ -1072,19 +1132,19 @@ int Event::Load( SDL_RWops *file ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Event::Save( SDL_RWops *file ) const {
+int Event::Save( File &file ) const {
   int i;
 
-  SDL_RWwrite( file, &e_id, 1, 1 );
-  SDL_RWwrite( file, &e_type, 1, 1 );
-  SDL_RWwrite( file, &e_trigger, 1, 1 );
-  SDL_RWwrite( file, &e_depend, 1, 1 );
-  for ( i = 0; i < 3; ++i ) SDL_WriteLE16( file, e_tdata[i] );
-  for ( i = 0; i < 3; ++i ) SDL_WriteLE16( file, e_data[i] );
-  SDL_WriteLE16( file, e_title );
-  SDL_WriteLE16( file, e_message );
-  SDL_WriteLE16( file, e_flags );
-  SDL_RWwrite( file, &e_player, 1, 1 );
+  file.Write8( e_id );
+  file.Write8( e_type );
+  file.Write8( e_trigger );
+  file.Write8( e_depend );
+  for ( i = 0; i < 3; ++i ) file.Write16( e_tdata[i] );
+  for ( i = 0; i < 3; ++i ) file.Write16( e_data[i] );
+  file.Write16( e_title );
+  file.Write16( e_message );
+  file.Write16( e_flags );
+  file.Write8( e_player );
   return 0;
 }
 
@@ -1138,6 +1198,10 @@ int Event::Export( ofstream &file, const UnitSet *uset ) const {
     file << "txpos = " << e_tdata[1] << '\n';
     file << "typos = " << e_tdata[2] << '\n';
     break;
+  case ETRIGGER_HANDICAP:
+    file << "handicap\n";
+    file << "thandicap = " << e_tdata[0] << '\n';
+    break;
   }
 
   switch ( e_type ) {
@@ -1152,10 +1216,13 @@ int Event::Export( ofstream &file, const UnitSet *uset ) const {
     break;
   case EVENT_SCORE:
     file << "success = " << e_data[0] << '\n';
+    file << "othermsg = " << e_data[1] << '\n';
+    file << "othertitle = " << e_data[2] << '\n';
     break;
   case EVENT_CREATE_UNIT:
     file << "unit = " << uset->GetUnitInfo(e_data[0])->Name() << '\n';
-    file << "building = " << e_data[1] << '\n';
+    file << "xpos = " << e_data[1] << '\n';
+    file << "ypos = " << e_data[2] << '\n';
     break;
   case EVENT_NEXT_MAP:
     file << "map = " << e_data[0] << '\n';
@@ -1193,32 +1260,3 @@ const char *Event::TriggerName( void ) const {
   return etrigger_labels[Trigger()];
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : Message::Save
-// DESCRIPTION: Save the message to a file.
-// PARAMETERS : file - data file descriptor
-// RETURNS    : 0 on success, -1 on error
-////////////////////////////////////////////////////////////////////////
-
-int Message::Save( SDL_RWops *file ) const {
-  char *crypted = new char[text.length() + 1];
-  strcpy( crypted, text.c_str() );
-  crypt( crypted );
-
-  SDL_RWwrite( file, crypted, 1, text.length() + 1 );
-  delete [] crypted;
-  return 0;
-}
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : Message::Export
-// DESCRIPTION: Save the message to a plain text file.
-// PARAMETERS : file - data file descriptor
-// RETURNS    : 0 on success, -1 on error
-////////////////////////////////////////////////////////////////////////
-
-int Message::Export( ofstream &file ) const {
-  file << text << '\n';
-  return 0;
-}
-
