@@ -41,7 +41,11 @@
 ButtonWidget::ButtonWidget( short id, short x, short y, unsigned short w,
                        unsigned short h, unsigned short flags,
                        const char *title, Window *window ) :
-     Widget( id, x, y, w, h, flags, title, window ) {}
+     Widget( id, x, y, w, h, flags, title, window ) {
+  if ( !(this->flags & (WIDGET_ALIGN_CENTER|WIDGET_ALIGN_LEFT|
+                        WIDGET_ALIGN_RIGHT|WIDGET_ALIGN_TOP)) )
+    this->flags |= WIDGET_ALIGN_CENTER;
+}
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : ButtonWidget::Draw
@@ -310,3 +314,154 @@ GUI_Status MenuButtonWidget::MouseMove( const SDL_MouseMotionEvent &motion ) {
   return GUI_OK;
 }
 
+
+#define GFX_CYCLE_W 7
+#define GFX_CYCLE_H 12
+#define GFX_CYCLE_X 138
+#define GFX_CYCLE_Y 46
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CycleWidget::CycleWidget
+// DESCRIPTION: Create a new cycle widget.
+// PARAMETERS : id     - widget identifier
+//              x      - left edge of widget relative to window border
+//              y      - top edge of widget
+//              w      - widget width
+//              h      - widget height
+//              flags  - see widget.h for details (default title
+//                       placement is WIDGET_ALIGN_LEFT)
+//              title  - widget title (may be NULL)
+//              defval - default value, starting with 0
+//              labels - array of available choices, terminated by a
+//                       NULL-string
+//              window - widget parent window
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+CycleWidget::CycleWidget( short id, short x, short y, unsigned short w,
+             unsigned short h, unsigned short flags, const char *title,
+             unsigned short defval, const char *labels[], Window *window ) :
+     ButtonWidget( id, x, y, w, h, flags, title, window ),
+     maxval(0) {
+
+  if ( !(flags & (WIDGET_ALIGN_CENTER|WIDGET_ALIGN_LEFT|
+                  WIDGET_ALIGN_RIGHT|WIDGET_ALIGN_TOP)) )
+    this->flags |= WIDGET_ALIGN_LEFT;
+
+  image[0] = Image( window->GetView()->GetSystemIcons(),
+                    GFX_CYCLE_X, GFX_CYCLE_Y,
+                    GFX_CYCLE_W, GFX_CYCLE_H );
+
+  int i;
+  for ( i = 1; labels[i]; ++i ) ++maxval;
+  choices = new string[maxval+1];
+  for ( i = 0; i <= maxval; ++i ) choices[i].assign( labels[i] );
+
+  val = MIN( defval, maxval );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CycleWidget::Draw
+// DESCRIPTION: Draw the cycle widget.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void CycleWidget::Draw( void ) {
+  ButtonWidget::Draw();
+
+  short off = (Clicked() ? 1 : 0);
+  Color fcol;
+
+  surface->FillRect( x + w - image[0].Width() - 8 + off, y + 4 + off, 1, h - 8, Color(CF_COLOR_SHADOW) );
+  surface->FillRect( x + w - image[0].Width() - 7 + off, y + 4 + off, 1, h - 8, Color(CF_COLOR_HIGHLIGHT) );
+
+  image[0].Draw( surface, x + w - image[0].Width() - 4 + off, y + (h - image[0].Height())/2 + off );
+
+  if ( Disabled() ) {
+    fcol = font->GetColor();
+    font->SetColor( Color(CF_COLOR_GHOSTED) );
+  }
+
+  // draw current label
+  font->Write( choices[val].c_str(), surface,
+               x + (w - image[0].Width() - 10 - font->TextWidth(choices[val].c_str()))/2 + off,
+               y + (h - font->Height()) / 2 + off, *this );
+
+  if ( Disabled() ) font->SetColor( fcol );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CycleWidget::Release
+// DESCRIPTION: Update the button label and redraw the widget.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void CycleWidget::Release( void ) {
+  val = (val + 1) % (maxval + 1);
+
+  ButtonWidget::Release();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CycleWidget::SetValue
+// DESCRIPTION: Set the cycle value.
+// PARAMETERS : value - new value
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void CycleWidget::SetValue( unsigned short value ) {
+  val = value % (maxval + 1);
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : DropWidget::DropWidget
+// DESCRIPTION: Create a dropd-own widget, which is a ButtonWidget with
+//              a small arrow image.
+// PARAMETERS : id     - widget identifier
+//              x      - left edge of widget relative to window border
+//              y      - top edge of widget
+//              w      - widget width
+//              h      - widget height
+//              flags  - see widget.h for details
+//              title  - widget title (may be NULL)
+//              window - widget parent window
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+DropWidget::DropWidget( short id, short x, short y, unsigned short w,
+             unsigned short h, unsigned short flags, const char *title,
+             Window *window ) :
+     ButtonWidget( id, x, y, w, h, flags, title, window ) {
+  image[0] = Image( window->GetView()->GetSystemIcons(), 157, 53, 11, 7 );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : DropWidget::Draw
+// DESCRIPTION: Draw the drop-down widget.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void DropWidget::Draw( void ) {
+  surface->DrawBack( *this );
+
+  surface->DrawBox( *this, BOX_RECESSED );
+  surface->DrawBox( Rect(x+1,y+1,w-2,h-2), Clicked() ? BOX_RECESSED : BOX_RAISED );
+  if ( Clicked() )
+    surface->FillRectAlpha( x+2, y+2, w-4, h-4, Color(CF_COLOR_SHADOW) );
+
+  short off = (Clicked() ? 1 : 0);
+  short xoff = x + (w - font->TextWidth(title.c_str()) - image[0].Width() - 8) / 2;
+  short yoff = y + (h - font->Height()) / 2;
+  PrintTitle( xoff + off, yoff + off, surface->GetFGPen() );
+
+  surface->FillRect( x + w - image[0].Width() - 8 + off, y + 4 + off, 1, h - 8, Color(CF_COLOR_SHADOW) );
+  surface->FillRect( x + w - image[0].Width() - 7 + off, y + 4 + off, 1, h - 8, Color(CF_COLOR_HIGHLIGHT) );
+
+  image[0].Draw( surface, x + w - image[0].Width() - 4 + off, y + (h - image[0].Height())/2 + off );
+}
+
