@@ -23,6 +23,9 @@
 #ifndef _INCLUDE_BUTTON_H
 #define _INCLUDE_BUTTON_H
 
+#include <string>
+using namespace std;
+
 #include "widget.h"
 
 class ButtonWidget : public Widget {
@@ -79,5 +82,36 @@ public:
        { flags ^= (WIDGET_STYLE_HIGHLIGHT|WIDGET_DEFAULT); }
 };
 
+
+class CycleWidget : public ButtonWidget {
+public:
+  CycleWidget( short id, short x, short y, unsigned short w, unsigned short h,
+               unsigned short flags, const char *title, unsigned short defval,
+               const char *labels[], Window *window );
+  ~CycleWidget( void ) { delete [] choices; }
+
+  void SetValue( unsigned short value );
+  unsigned short GetValue( void ) const { return val; }
+  const char *GetLabel( void ) const { return choices[val].c_str(); }
+
+  void Draw( void );
+  void Release( void );
+
+private:
+  unsigned short val;
+  unsigned short maxval;
+  string *choices;
+};
+
+
+class DropWidget : public ButtonWidget {
+public:
+  DropWidget( short id, short x, short y, unsigned short w,
+              unsigned short h, unsigned short flags,
+              const char *title, Window *window );
+
+  void Draw( void );
+};
+
 #endif	/* _INCLUDE_BUTTON_H */
 
