@@ -25,6 +25,7 @@
 #include "extwindow.h"
 #include "misc.h"
 #include "sound.h"
+#include "msgs.h"
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : DialogWindow::DialogWindow
@@ -48,9 +49,9 @@ DialogWindow::DialogWindow( const char *title, const string &msg,
       Window(flags, view), btncnt(1), text(msg), head(title) {
   Font *tfont = (flags & WIN_FONT_BIG) ? LargeFont() : SmallFont();
 
-  unsigned long txtw = sfont->TextWidth( msg.c_str() );
+  unsigned short txtw = sfont->TextWidth( msg.c_str() );
   unsigned short defw = MAX( view->Width() * 3/4, 300 ),
-                 prefw = MIN( defw, txtw + 20 ),
+                 prefw = MIN( defw, txtw + 25 ),
                  btnw = 0;
 
   // count buttons
@@ -154,6 +155,21 @@ void DialogWindow::SetButtonHook( WidgetHook *hook ) const {
 }
 
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : NoteWindow::NoteWindow
+// DESCRIPTION: Create a window with a title, a short message, and an
+//              "OK" button.
+// PARAMETERS : title - window title (may be NULL)
+//              note  - the message to be displayed
+//              flags - see window.h for details
+//              view  - pointer to the window's view
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+NoteWindow::NoteWindow( const char *title, const string &note,
+                        unsigned short flags, View *view ) :
+            DialogWindow( title, note, MSG(MSG_B_OK), 0, flags, view ) {}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MessageWindow::MessageWindow
 // DESCRIPTION: Create a message window. It will contain a TextScroll
@@ -173,7 +189,7 @@ MessageWindow::MessageWindow( const char *title, const char *msg, View *view ) :
   SetSize( MAX(view->Width()/2, 320), MAX(view->Height()*3/4, 240) );
 
   button = new ButtonWidget( GUI_CLOSE, 1, h - sfont->Height() - 11, w - 2, sfont->Height() + 10,
-                             WIDGET_DEFAULT, "_OK", this );
+                             WIDGET_DEFAULT, MSG(MSG_B_OK), this );
 
   short boxy = 5;
   if ( title ) boxy += lfont->Height() * 2;
@@ -236,9 +252,13 @@ PasswordWindow::PasswordWindow( const char *title, const char *msg,
                              NULL, 7, WIDGET_ALIGN_TOP|WIDGET_STR_PASSWORD,
                              msg, this );
 
-  button = new ButtonWidget( GUI_CLOSE, 1, h - sfont->Height() - 9,
-                             w - 2, sfont->Height() + 8, WIDGET_DEFAULT,
-                             "_OK", this );
+  Widget *wd = new ButtonWidget( GUI_CLOSE, 1, h - sfont->Height() - 9,
+                                 w/2 - 2, sfont->Height() + 8, WIDGET_DEFAULT,
+                                 MSG(MSG_B_OK), this );
+  wd = new ButtonWidget( GUI_RESTART,
+                         wd->LeftEdge() + wd->Width() + 1, wd->TopEdge(),
+                         wd->Width(), wd->Height(), 0, MSG(MSG_B_CANCEL), this );
+
   Draw();
   Show();
   string->SetFocus();
@@ -314,9 +334,10 @@ ProgressWindow::ProgressWindow( short x, short y, unsigned short w,
   short bwidth = 0;
 
   if ( flags & WIN_PROG_ABORT ) {
-    bwidth = sfont->Width() * 5 + 20;
+    const char *lbl = MSG(MSG_B_SKIP);
+    bwidth = sfont->TextWidth( lbl ) + 15;
     new ButtonWidget( GUI_CLOSE, w - bwidth - 5, 5, bwidth, h - 10,
-                      WIDGET_DEFAULT, "_Abort", this );
+                      WIDGET_DEFAULT, lbl, this );
   }
 
   progress = new ProgressWidget( GUI_OK, 5, 5, w - 10 - bwidth, h - 10,
@@ -354,7 +375,7 @@ bool ProgressWindow::Aborted( void ) {
 // DESCRIPTION: Create a new menu window.
 // PARAMETERS : title - menu title (may be NULL)
 //              hook  - widget hook to be notified when one of the menu
-//                      items is activated (may be NULL); sub-menues
+//                      items is activated (may be NULL); sub-menus
 //                      are handled internally, so you won't get
 //                      messages for those
 //              view  - pointer to the window's view
@@ -364,6 +385,7 @@ bool ProgressWindow::Aborted( void ) {
 MenuWindow::MenuWindow( const char *title, WidgetHook *hook, View *view ) :
             Window( WIN_CLOSE_ESC|WIN_CLOSE_UNFOCUS, view ),
             layout(false), position(-1,-1), item_hook(hook), parent_menu(0) {
+  minwidth = 0;
   vspacing = 4;
   barheight = vspacing * 2 + 2;		// height of the separator bar
 
@@ -477,13 +499,12 @@ void MenuWindow::Layout( void ) {
     win.x = position.x;
     win.y = position.y;
   }
-  win.w = maxwidth + 20;
+  win.w = MAX( maxwidth + 20, minwidth );
   win.h = height;
 
   win.Align( *view );
   SetSize( win );
 
-
   // create the buttons
   mi = static_cast<MenuItem *>( items.Head() );
   unsigned short ypos = 5;
