@@ -32,14 +32,22 @@
  */
 
 #ifndef VERSION
-# define VERSION "0.3.8"
+# define VERSION "0.4.3"
 #endif
 
 #define CF_SHORTNAME	"crimson"
 #define CF_DATFILE	"cf.dat"
 #define CF_TITLE_SCREEN	"title.bmp"
 
-#define FILE_VERSION  9
+#define CF_FONT		"Bepa-Roman.ttf"
+#define CF_FONT_SMALL	12
+#define CF_FONT_LARGE	16
+
+#define CF_MUSIC_THEME		"theme"
+#define CF_MUSIC_DEFAULT	"default"
+#define CF_MUSIC_FADE_TIME	2000
+
+#define FILE_VERSION  11
 #define FID_MISSION   MakeID('M','S','S','N')  /* mission file identifier */
 
 #define DISPLAY_BPP	16	/* display depth */
