@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,7 +40,6 @@
 //                       levels and two of them were represented by one
 //                       slider position, so the knob would take up
 //                       2/12th of the widget size)
-//              key    - keystroke to control widget
 //              flags  - widget flags (see widget.h for details)
 //              title  - widget title (currently unused)
 //              window - window to add this widget to
@@ -49,8 +48,8 @@
 
 SliderWidget::SliderWidget( short id, short x, short y, unsigned short w,
     unsigned short h, short min, short max, short start, short ksize,
-    short key, unsigned short flags, const char *title, Window *window ) :
-    Widget( id, x, y, w, h, key, flags, title, window ) {
+    unsigned short flags, const char *title, Window *window ) :
+    Widget( id, x, y, w, h, flags, title, window ) {
   mousehit = false;
   current = start;
   keystep = 1;
@@ -300,7 +299,7 @@ GUI_Status SliderWidget::MouseMove( const SDL_MouseMotionEvent &motion ) {
 ProgressWidget::ProgressWidget( short id, short x, short y, unsigned short w,
     unsigned short h, short min, short max, unsigned short flags,
     const char *title, Window *window ) :
-    Widget( id, x, y, w, h, 0, flags, title, window ) {
+    Widget( id, x, y, w, h, flags, title, window ) {
   this->min = min;
   this->max = max;
   level = 0;
