@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,8 +39,8 @@ class SliderWidget : public Widget {
 public:
   SliderWidget( short id, short x, short y, unsigned short w,
     unsigned short h, short min, short max, short start,
-    short ksize, short key, unsigned short flags,
-    const char *title, Window *window );
+    short ksize, unsigned short flags, const char *title,
+    Window *window );
 
   void Adjust( short newmin, short newmax, short newsize );
   virtual void Draw( void );
