@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2003 Jens Granseuer
+// Copyright (C) 2000-2004 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,8 +20,6 @@
 // surface.cpp
 ////////////////////////////////////////////////////////////////////////////
 
-#include "SDL_endian.h"
-
 #include "surface.h"
 #include "misc.h"
 #include "globals.h"
@@ -93,7 +91,7 @@ Color Surface::GetColorKey( void ) const {
 int Surface::DisplayFormat( void ) {
   SDL_Surface *new_s;
 
-  if ( s_surface == 0 ) return -1;
+  if ( s_surface == 0 || !SDL_WasInit( SDL_INIT_VIDEO ) )  return -1;
   if ( (new_s = SDL_DisplayFormat( s_surface )) == 0 ) return -1;
   SDL_FreeSurface( s_surface );
   s_surface = new_s;
@@ -470,12 +468,12 @@ void Surface::DrawPixel( short x, short y, const Color &col ) const {
 // NAME       : Surface::LoadPalette
 // DESCRIPTION: Load the color palette from a data file. The file is
 //              supposed to be opened already.
-// PARAMETERS : file - SDL_RWops data file descriptor
+// PARAMETERS : file - data file descriptor
 //              num  - number of colors to read
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Surface::LoadPalette( SDL_RWops *file, unsigned short num ) {
+int Surface::LoadPalette( File &file, unsigned short num ) {
   unsigned char *pal, *col;
 
   // file contains num colors
@@ -483,7 +481,7 @@ int Surface::LoadPalette( SDL_RWops *file, unsigned short num ) {
   if ( !pal ) return -1;
   col = pal;
 
-  SDL_RWread( file, pal, 1, num * 3 );
+  file.Read( pal, num * 3 );
 
   for ( int i = 0; i < num; ++i ) {
     s_surface->format->palette->colors[i].r = *col++;
@@ -499,30 +497,30 @@ int Surface::LoadPalette( SDL_RWops *file, unsigned short num ) {
 // NAME       : Surface::LoadImageData
 // DESCRIPTION: Load image data from a data file. The file is supposed
 //              to be opened already.
-// PARAMETERS : file      - SDL_RWops data file descriptor
+// PARAMETERS : file      - data file descriptor
 //              hwsurface - tell SDL to try to put the image into video
 //                          memory (defaults to false)
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Surface::LoadImageData( SDL_RWops *file, bool hwsurface /* = false */ ) {
+int Surface::LoadImageData( File &file, bool hwsurface /* = false */ ) {
   unsigned char bpp, flags;
   unsigned short width, height, colors;
   unsigned long packed;
   Color ckey;
   int rc = 0;
 
-  width = SDL_ReadLE16( file );
-  height = SDL_ReadLE16( file );
-  SDL_RWread( file, &bpp, 1, 1 );   // nothing but 8 supported for now
-  SDL_RWread( file, &flags, 1, 1 );
-  packed = SDL_ReadLE32( file );
-  colors = SDL_ReadLE16( file );
+  width = file.Read16();
+  height = file.Read16();
+  bpp = file.Read8();   // nothing but 8 supported for now
+  flags = file.Read8();
+  packed = file.Read32();
+  colors = file.Read16();
 
   if ( flags & RAW_DATA_TRANSPARENT ) {
-    SDL_RWread( file, &ckey.r, 1, 1 );
-    SDL_RWread( file, &ckey.g, 1, 1 );
-    SDL_RWread( file, &ckey.b, 1, 1 );
+    ckey.r = file.Read8();
+    ckey.g = file.Read8();
+    ckey.b = file.Read8();
   }
 
   if ( Create( width, height, bpp, (hwsurface ? SDL_HWSURFACE : 0) ) ) return -1;
@@ -531,7 +529,7 @@ int Surface::LoadImageData( SDL_RWops *file, bool hwsurface /* = false */ ) {
   unsigned char *buf = new unsigned char[packed], *unbuf;
   if ( !buf ) rc = -1;
   else {
-    SDL_RWread( file, buf, 1, packed );
+    file.Read( buf, packed );
 
     if ( packed != (unsigned long)(width * height) ) {   // data is compressed
       unbuf = new unsigned char[width * height];
