@@ -20,6 +20,7 @@
 // textbox.cpp
 ////////////////////////////////////////////////////////////////////////
 
+#include <cctype>
 #include <string.h>
 
 #include "textbox.h"
@@ -44,8 +45,7 @@ TextWidget::TextWidget( short id, short x, short y,
            unsigned short w, unsigned short h, const char *str,
            unsigned short flags, const char *title, Window *window ) :
     Widget( id, x, y, w, h, flags, title, window ) {
-  text = NULL;
-  lineptr = NULL;
+  textsurface = NULL;
   spacing = 2;
   SetText( str );
 }
@@ -58,8 +58,7 @@ TextWidget::TextWidget( short id, short x, short y,
 ////////////////////////////////////////////////////////////////////////
 
 TextWidget::~TextWidget( void ) {
-  delete [] text;
-  delete [] lineptr;
+  delete textsurface;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -70,103 +69,48 @@ TextWidget::~TextWidget( void ) {
 ////////////////////////////////////////////////////////////////////////
 
 void TextWidget::SetText( const char *str ) {
-  delete [] text;
-  delete [] lineptr;
-  text = NULL;
-  lineptr = NULL;
-  lines = rows = visrows = toprow = 0;
+  delete textsurface;
+  textsurface = NULL;
+  rows = visrows = toprow = 0;
 
   if ( !str ) return;
 
-  lines = 1;
-  const int maxwidth = w - 10;
-  int len = 1, linelen = 0;
-  int lastspace = 0, lastspacewidth = 0;
-  int linewidth = 0, charwidth;
-
-  // find out length of text after we're done with it
-  const char *ptr;
-  for ( ptr = str; *ptr != '\0'; ++len, ++ptr ) {
-    ++linelen;
-    charwidth = font->CharWidth( *ptr );
-    if ( *ptr == ' ' ) {
-      lastspace = linelen;
-      lastspacewidth = 0;
-    } else lastspacewidth += charwidth;
-
-    if ( (linewidth + charwidth <= maxwidth) && (*ptr != '\n') ) {
-      linewidth += charwidth;
-    } else {
-      ++lines;
-      if ( *ptr == '\n' ) {
-        linelen = 0;
-        linewidth = 0;
-      } else if ( lastspace > 0 ) {	// break line at last space
-        linelen -= lastspace;
-        linewidth = lastspacewidth;
-      } else {				// break line in the middle of a word
-        ++len;				// we need to insert a NUL-byte here
-        linelen = 1;
-        linewidth = charwidth;
-      }
-      lastspace = 0;
-      lastspacewidth = 0;
+  rows = font->TextHeight( str, w - 10, spacing );
+  if ( rows > 0 ) {
+    textsurface = new Surface();
+    if ( textsurface->Create( w - 10, rows, surface->GetView()->ScreenBPP(), 0 ) ) {
+      delete textsurface;
+      textsurface = NULL;
+      return;
     }
-  }
 
-  // allocate buffer
-  text = new char[ len ];
-  lineptr = new char *[ lines ];
-  lineptr[0] = text;
+    textsurface->SetColorKey( Color(CF_COLOR_BLACK) );
+    textsurface->Flood( Color(CF_COLOR_BLACK) );
 
-  len = linelen = 0;
-  lastspace = lastspacewidth = 0;
-  linewidth = 0;
-  int line = 1;
+    string full( str );
+    int pos = 0, endpos = full.size(), rowcnt = 0;
+    unsigned short xoff = 0;
 
-  // now format the string correctly to fit into the box
-  for ( ptr = str; *ptr != '\0'; ++ptr ) {
-    charwidth = font->CharWidth( *ptr );
-    linewidth += charwidth;
+    while ( pos < endpos ) {
+      size_t linelen = font->FitText( &str[pos], w - 10, true );
+      string sub( full.substr( pos, linelen ) );
 
-    if ( *ptr == ' ' ) {
-      lastspace = linelen;
-      lastspacewidth = 0;
-    } else lastspacewidth += charwidth;
+      // remove newlines
+      size_t rep = 0;
+      while ( (rep = sub.find( '\n', rep )) != string::npos )
+        sub.erase( rep, 1 );
 
-    if ( (linewidth > maxwidth) || (*ptr == '\n') ) {
-      if ( *ptr == '\n' ) {
-        text[len++] = '\0';
-        lineptr[line] = &text[len];
-        linelen = 0;
-        linewidth = 0;
-      } else if ( lastspace > 0 ) {		// break line at last space
-        text[len - linelen + lastspace] = '\0';
-        lineptr[line] = &text[len - linelen + lastspace + 1];
-        if ( lastspace != linelen ) text[len] = *ptr;
-        ++len;
-        linelen -= lastspace + 1;
-        linewidth = lastspacewidth;
-      } else {				// break line in the middle of a word
-        text[len++] = '\0';		// we need to insert a NUL-byte here
-        lineptr[line] = &text[len];
-        linelen = 1;
-        linewidth = charwidth;
-        text[len++] = *ptr;
-      }
-      ++line;
-      lastspace = 0;
-      lastspacewidth = 0;
-    } else {
-      text[len++] = *ptr;
-      ++linelen;
+      if ( flags & WIDGET_ALIGN_CENTER )
+        xoff = (w - 10 - font->TextWidth( sub.c_str() )) / 2;
+      font->Write( sub.c_str(), textsurface, xoff, rowcnt );
+
+      rowcnt += font->Height() + spacing;
+      pos += linelen;
     }
-  }
-  text[len] = '\0';
 
-  rows = lines * (font->Height() + spacing);
-  visrows = h - 2 * spacing;
-  if ( visrows > rows ) visrows = rows;
+    visrows = h - 6;
+    if ( visrows > rows ) visrows = rows;
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -180,18 +124,9 @@ void TextWidget::Draw( void ) {
   surface->DrawBack( *this );
   surface->DrawBox( *this, BOX_RECESSED );
 
-  short xoff = x + 5;
-  short yoff = y + 2 - toprow % (font->Height() + spacing);
-  short topline = toprow / (font->Height() + spacing);
-  Rect clip( x + 2, y + 2, w - 4, h - 4 );
-
-  for ( int i = topline; i < lines; ++i ) {
-    if ( flags & WIDGET_ALIGN_CENTER )
-      xoff = x + 5 + (w - 10 - font->TextWidth( lineptr[i] )) / 2;
-    font->Write( lineptr[i], surface, xoff, yoff, clip );
-    yoff += font->Height() + spacing;
-    if ( yoff >= y + h - 2 ) break;
-  }
+  if ( textsurface )
+    textsurface->Blit( surface, Rect(0,toprow,textsurface->Width(),visrows),
+                       x + 5, y + 3 );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -231,14 +166,13 @@ unsigned short TextWidget::SetRow( unsigned short top ) {
 TextScrollWidget::TextScrollWidget( short id, short x, short y,
            unsigned short w, unsigned short h, const char *str,
            unsigned short flags, const char *title, Window *window ) :
-    TextWidget( id, x, y, w - DEFAULT_SLIDER_SIZE, h, str, flags, title, window ) {
-  current = 0;
-  slider = new SliderWidget( id + 1, x + w - DEFAULT_SLIDER_SIZE, y,
-               DEFAULT_SLIDER_SIZE, h, 0, Rows() - RowsVisible(), 0, RowsVisible(),
-               WIDGET_VSCROLL|WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY,
-               NULL, window );
-  slider->SetKeyStep( font->Height() + 2 );
-  slider->SetHook( this );
+    CompositeWidget( id, x, y, w, h, 0, 0, window ) {
+
+  TextWidget *text = new TextWidget( 0, x, y, w, h, 0,
+              flags|WIDGET_COMPOSITE, title, window );
+  AddWidget( text );
+
+  SetText( str );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -251,10 +185,8 @@ TextScrollWidget::TextScrollWidget( short id, short x, short y,
 ////////////////////////////////////////////////////////////////////////
 
 GUI_Status TextScrollWidget::WidgetActivated( Widget *widget, Window *win ) {
-  if ( widget == slider ) {
-    current = slider->Level();
-    SetRow( current );
-  }
+  TextWidget *t = static_cast<TextWidget *>( GetWidget( 0 ) );
+  t->SetRow( static_cast<SliderWidget *>(widget)->Level() );
   return GUI_OK;
 }
 
@@ -266,9 +198,35 @@ GUI_Status TextScrollWidget::WidgetActivated( Widget *widget, Window *win ) {
 ////////////////////////////////////////////////////////////////////////
 
 void TextScrollWidget::SetText( const char *str ) {
-  TextWidget::SetText( str  );
-  slider->Adjust( 0, Rows() - RowsVisible(), MAX(RowsVisible(),1) );
-  slider->ScrollTo( 0 );
+  TextWidget *t = static_cast<TextWidget *>( GetWidget(0) );
+  SliderWidget *s = static_cast<SliderWidget *>( GetWidget(1) );
+
+  bool needslider = str && (font->TextHeight( str, w - 10, 2 ) > h - 6);
+
+  if ( needslider ) {
+    if ( !s ) {
+      s = new SliderWidget( 1, x + w - DEFAULT_SLIDER_SIZE, y,
+          DEFAULT_SLIDER_SIZE, h, 0, 0, 0, 0,
+          WIDGET_VSCROLL|WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY|WIDGET_COMPOSITE,
+          NULL, surface );
+      s->SetKeyStep( font->Height() + 2 );
+      s->SetHook( this );
+      AddWidget( s );
+
+      t->SetSize( x, y, w - s->Width(), h );
+    }
+  } else if ( s ) {
+    RemoveWidget( s );
+    t->SetSize( x, y, w + s->Width(), h );
+    delete s;
+  }
+
+  t->SetText( str );
+
+  if ( needslider ) {
+    s->Adjust( 0, t->Rows() - t->RowsVisible(), MAX(t->RowsVisible(),1) );
+    s->ScrollTo( 0 );
+  }
 }
 
 
@@ -280,8 +238,8 @@ void TextScrollWidget::SetText( const char *str ) {
 ////////////////////////////////////////////////////////////////////////
 
 void TextListWidget::DrawNodes( void ) {
-  Rect box( x + 4, y + 1 + spacing, w - 8, h - 2 - 2 * spacing );
-  Rect area( x + 1, y + 1, w - 2, h - 2 );
+  Rect box( x + 4, y + 1 + spacing, listboxw - 8, h - 2 - 2 * spacing );
+  Rect area( x + 1, y + 1, listboxw - 2, h - 2 );
   short num = toprow / ItemHeight();                        // number of top node
   TLWNode *n = static_cast<TLWNode *>(list->GetNode( num ));  // top node
   short xoff = box.x, yoff = box.y + (num * ItemHeight()) - toprow;
@@ -293,7 +251,7 @@ void TextListWidget::DrawNodes( void ) {
       xoff = box.x + (box.w - font->TextWidth(n->Name())) / 2;
 
     if ( num == current ) {
-      Rect hilite( x + 2, yoff, w - 4, ItemHeight() );
+      Rect hilite( x + 2, yoff, listboxw - 4, ItemHeight() );
       hilite.Clip( area );
       surface->FillRectAlpha( hilite, surface->GetFGPen() );
     }
@@ -350,24 +308,12 @@ StringWidget::StringWidget( short id, short x, short y, unsigned short w,
               unsigned short maxlen, unsigned short flags,
               const char *title, Window *window ) :
       Widget( id, x, y, w, h, flags, title, window ),
-      cursor(0), maxlen(maxlen), cursorpix(0), offset(0),
+      cursor(0), maxlen(maxlen), offset(0),
       strbox(x+4,y+(h-font->Height())/2,w-8,font->Height()),
       validator(0) {
-  buffer = new char[maxlen+1];
   SetString( str, false );
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : StringWidget::~StringWidget
-// DESCRIPTION: Destroy the StringWidget.
-// PARAMETERS : -
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-StringWidget::~StringWidget( void ) {
-  delete [] buffer;
-}
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : StringWidget::SetString
 // DESCRIPTION: Set a new string to display in the widget.
@@ -378,12 +324,8 @@ StringWidget::~StringWidget( void ) {
 ////////////////////////////////////////////////////////////////////////
 
 void StringWidget::SetString( const char *newstr, bool upd /* = true */ ) {
-  if ( newstr ) {
-    len = strlen( newstr );
-    strncpy( buffer, newstr, maxlen );
-  } else len = 0;
-
-  buffer[MIN(len, maxlen)] = '\0';
+  if ( newstr ) buffer.assign( newstr );
+  else buffer.erase();
 
   if ( upd ) {
     Draw();
@@ -399,7 +341,7 @@ void StringWidget::SetString( const char *newstr, bool upd /* = true */ ) {
 ////////////////////////////////////////////////////////////////////////
 
 const char *StringWidget::String( void ) const {
-  if ( len > 0 ) return buffer;
+  if ( !buffer.empty() ) return buffer.c_str();
   return NULL;
 }
 
@@ -411,34 +353,36 @@ const char *StringWidget::String( void ) const {
 ////////////////////////////////////////////////////////////////////////
 
 void StringWidget::Draw( void ) {
-  unsigned short firstc = offset / font->Width();
-
+  Rect clip;
   surface->DrawBack( *this );
   surface->DrawBox( *this, BOX_RECESSED );
-  if ( Clicked() ) surface->FillRect( strbox.x + cursorpix, strbox.y,
-                   font->Width(), font->Height(), surface->GetFGPen() );
+  surface->GetClipRect( clip );
+  surface->SetClipRect( strbox );
 
+  if ( Clicked() )
+    surface->FillRect( strbox.x + CursorPos(), strbox.y,
+                       CursorWidth(), font->Height(), surface->GetFGPen() );
+
+  short xoff = strbox.x - offset;
   if ( flags & WIDGET_STR_PASSWORD ) {
-    short xoff = firstc * font->Width() - offset + strbox.x;
-    Rect clip;
-    surface->GetClipRect( clip );
-    surface->SetClipRect( strbox );
+    unsigned short starw = font->CharWidth('*');
+    unsigned short len = buffer.size();
 
-    for ( int i = firstc; i < len; ++i ) {
+    for ( int i = 0; i < len; ++i ) {
       if ( Clicked() && (cursor == i) )
         font->Write( '*', surface, xoff, strbox.y, surface->GetBGPen() );
       else font->Write( '*', surface, xoff, strbox.y );
-      xoff += font->CharWidth( '*' );
+      xoff += starw;
     }
 
-    surface->SetClipRect( clip );
   } else {
-    font->Write( &buffer[firstc], surface, strbox.x + firstc * font->Width() - offset, strbox.y, strbox );
-    if ( Clicked() && (cursor < len) ) {
-      font->Write( buffer[cursor], surface, strbox.x + cursorpix, strbox.y, surface->GetBGPen() );
+    font->Write( buffer.c_str(), surface, strbox.x - offset, strbox.y, strbox );
+    if ( Clicked() && (cursor < buffer.size()) ) {
+      font->Write( buffer[cursor], surface, strbox.x + CursorPos(), strbox.y, surface->GetBGPen() );
     }
   }
 
+  surface->SetClipRect( clip );
   PrintTitle( surface->GetFGPen() );
 }
 
@@ -455,16 +399,17 @@ GUI_Status StringWidget::MouseDown( const SDL_MouseButtonEvent &button ) {
         Contains( button.x - surface->LeftEdge(),
                   button.y - surface->TopEdge() ) ) {
     short xoff = button.x - surface->LeftEdge() - x;
-    cursor = cursorpix = 0;
-    while ( (cursor < len) &&
-          (cursorpix + font->CharWidth( buffer[cursor] ) < xoff) )
-      cursorpix += font->CharWidth( buffer[cursor++] );
+    unsigned short len = buffer.size();
 
-    if ( cursorpix < offset ) offset = cursorpix;
-    else if ( cursorpix + font->Width() >= offset + strbox.w ) {
-      offset = cursorpix + font->Width() - strbox.w;
+    cursor = 0;
+    while ( (cursor < len) && (CursorPos() + CursorWidth() < xoff) )
+      ++cursor;
+
+    short curpix = CursorPos();
+    if ( curpix < 0 ) offset -= curpix;
+    else if ( curpix + CursorWidth() >= strbox.w ) {
+      offset += curpix + CursorWidth() - strbox.w;
     }
-    cursorpix -= offset;
 
     return InputLoop();
   }
@@ -480,11 +425,10 @@ GUI_Status StringWidget::MouseDown( const SDL_MouseButtonEvent &button ) {
 
 GUI_Status StringWidget::KeyDown( const SDL_keysym &key ) {
   if ( (key.sym == this->key) && !(flags & WIDGET_STR_CONST) ) {
-    unsigned short buflen = font->TextWidth( buffer );
+    unsigned short buflen = font->TextWidth( buffer.c_str() );
 
-    cursor = len;
-    offset = MAX( 0, buflen + font->Width() - strbox.w );
-    cursorpix = buflen - offset;
+    cursor = buffer.size();
+    offset = MAX( 0, buflen + CursorWidth() - strbox.w );
     return InputLoop();
   }
   return GUI_OK;
@@ -524,7 +468,7 @@ GUI_Status StringWidget::InputLoop( void ) {
     }
   } while ( !quit );
 
-  cursor = cursorpix = offset = 0;
+  cursor = offset = 0;
 
   SDL_EnableUNICODE( unicode );
   Release();
@@ -547,30 +491,23 @@ void StringWidget::CharInput( short sym, unsigned short unicode ) {
     switch ( unicode ) {
     case '\b':                // BACKSPACE - delete char at cursor-1
       if ( cursor > 0 ) {
-        cursorpix -= font->CharWidth( buffer[--cursor] );
-        buffer[cursor] = '\0';
-        if (cursor < --len) strcat( buffer, &buffer[cursor+1] );      // close the gap
+        buffer.erase( --cursor, 1 );
         changed = true;
       }
       break;
     case 127:                 // DELETE - delete char at cursor
-      if ( cursor < len ) {
-        buffer[cursor] = '\0';
-        strcat( buffer, &buffer[cursor+1]);
-        --len;
+      if ( cursor < buffer.size() ) {
+        buffer.erase( cursor, 1 );
         changed = true;
       }
       break;
     default:
-      if ( font->Allowed( unicode ) && (len < maxlen) ) {   // insert char at cursor pos
+      if ( buffer.size() < maxlen ) {   // insert char at cursor pos
         bool accept = (validator ?
-                       validator->ValidateKey( buffer, unicode, cursor ) :
-                       true );
+                       validator->ValidateKey( buffer.c_str(), unicode, cursor ) :
+                       (isalnum(unicode) && (unicode<=127)) );
         if ( accept ) {
-          for ( int i = len; i >= cursor; --i ) buffer[i+1] = buffer[i];
-          buffer[cursor++] = (char)unicode;
-          cursorpix += font->CharWidth( (char)unicode );
-          ++len;
+          buffer.insert( cursor++, 1, unicode );
           changed = true;
         }
       }
@@ -578,13 +515,13 @@ void StringWidget::CharInput( short sym, unsigned short unicode ) {
   } else switch ( sym ) {
     case SDLK_LEFT:                     // move cursor left
       if ( cursor > 0 ) {
-        cursorpix -= font->CharWidth( buffer[--cursor] );
+        --cursor;
         changed = true;
       }
       break;
     case SDLK_RIGHT:
-      if ( cursor < len ) {
-        cursorpix += font->CharWidth( buffer[cursor++] );
+      if ( cursor < buffer.size() ) {
+        ++cursor;
         changed = true;
       }
       break;
@@ -593,19 +530,67 @@ void StringWidget::CharInput( short sym, unsigned short unicode ) {
   }
 
   if ( changed ) {
-    if ( cursorpix < 0 ) {
-      offset += cursorpix;
-      cursorpix = 0;
-    } else if ( cursorpix + font->Width() >= strbox.w ) {
-      offset += cursorpix + font->Width() - strbox.w;
-      cursorpix = strbox.w - font->Width();
-   }
+    short curpix = CursorPos();
+    if ( curpix < 0 ) offset += curpix;
+    else if ( curpix + CursorWidth() >= strbox.w )
+      offset += curpix + CursorWidth() - strbox.w;
 
     Draw();
     Show();
   }
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : StringWidget::CursorWidth
+// DESCRIPTION: Get width of cursor in pixels.
+// PARAMETERS : -
+// RETURNS    : cursor width
+////////////////////////////////////////////////////////////////////////
+
+unsigned short StringWidget::CursorWidth( void ) const {
+  unsigned short curw;
+
+  if ( flags & WIDGET_STR_PASSWORD ) curw = font->CharWidth('*');
+  else if ( cursor < buffer.size() ) curw = font->CharWidth( buffer[cursor] );
+  else curw = font->Width();
+
+  return curw;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : StringWidget::CursorPos
+// DESCRIPTION: Get position of cursor on display in pixels.
+// PARAMETERS : -
+// RETURNS    : cursor position (- current offset)
+////////////////////////////////////////////////////////////////////////
+
+short StringWidget::CursorPos( void ) const {
+  unsigned short cp;
+  if ( flags & WIDGET_STR_PASSWORD )
+    cp = cursor * font->CharWidth('*');
+  else {
+    cp = font->TextWidth(buffer.substr(0, cursor + 1).c_str());
+    if ( buffer.size() > cursor ) cp -= CursorWidth();
+  }
+  return cp - offset;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : StringWidget::DispWidth
+// DESCRIPTION: Get width of a character in pixels.
+// PARAMETERS : ch - glyph to measure
+// RETURNS    : character width
+////////////////////////////////////////////////////////////////////////
+
+unsigned short StringWidget::DispWidth( short ch ) const {
+  unsigned short cw;
+
+  if ( flags & WIDGET_STR_PASSWORD ) cw = font->CharWidth('*');
+  else cw = font->CharWidth( ch );
+
+  return cw;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : StringWidget::SetFocus
 // DESCRIPTION: Activate the widget, i.e. show the cursor and wait for
@@ -616,7 +601,6 @@ void StringWidget::CharInput( short sym, unsigned short unicode ) {
 
 GUI_Status StringWidget::SetFocus( void ) {
   cursor = 0;
-  cursorpix = 0;
   return InputLoop();
 }
 
@@ -643,6 +627,23 @@ TLWNode::TLWNode( const char *name ) :
          name(name), id(0), user_data(0), user_flags(0) {
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : TLWList::Sort
+// DESCRIPTION: Sort the list in ascending alphabetical order.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void TLWList::Sort( void ) {
+  TLWList s;
+
+  while ( !IsEmpty() )
+    s.AddHead( RemHead() );
+
+  while ( !s.IsEmpty() )
+    InsertNodeSorted( static_cast<TLWNode *>(s.RemTail()) );
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : TLWList::InsertNodeSorted
 // DESCRIPTION: Insert a node into the list according to its name.
