@@ -24,6 +24,8 @@
 #define _INCLUDE_WIDGET_H
 
 #include <string>
+#include <vector>
+using namespace std;
 
 #include "surface.h"
 #include "font.h"
@@ -37,11 +39,12 @@ enum GUI_Status { GUI_ERROR = -6, GUI_RESTART, GUI_QUIT, GUI_CLOSE, GUI_NONE, GU
 #define WIDGET_DEFAULT        0x0001  // widget responds to RETURN key
 #define WIDGET_HIDDEN         0x0002  // widget is not shown and does not receive event messages
 #define WIDGET_DISABLED       0x0004  // widget is shown, but does not receive event messages
-
+#define WIDGET_COMPOSITE      0x0008  // widget is part of a composite widget (not the
+                                      // CompositeWidget itself)
 #define WIDGET_ALIGN_CENTER   0x0010
-#define WIDGET_ALIGN_LEFT     0x0020  // really only supported for button widgets...
-#define WIDGET_ALIGN_RIGHT    0x0040  //    "     "      "      "    "       "
-#define WIDGET_ALIGN_TOP      0x0080  // string widget only
+#define WIDGET_ALIGN_LEFT     0x0020
+#define WIDGET_ALIGN_RIGHT    0x0040
+#define WIDGET_ALIGN_TOP      0x0080
 
 // for buttons mostly; some combinations are not allowed (e.g. SUBMENU and GFX)
 #define WIDGET_STYLE_STD       0x0000 // standard bevelled box
@@ -81,6 +84,7 @@ public:
 
   void Show( void ) const;
   virtual void Draw( void ) = 0;
+  void PrintTitle( short xoff, short yoff, const Color &hcol ) const;
   void PrintTitle( const Color &hcol ) const;
 
   virtual void Push( void );
@@ -89,6 +93,7 @@ public:
   void Enable( void ) { flags &= (~WIDGET_DISABLED); }
   bool Clicked( void ) const { return clicked; }
   bool Hidden( void ) const { return (flags & WIDGET_HIDDEN) != 0; }
+  bool Composite( void ) const { return (flags & WIDGET_COMPOSITE) != 0; }
   bool Disabled( void ) const { return (flags & WIDGET_DISABLED) != 0; }
 
   GUI_Status HandleEvent( const SDL_Event &event );
@@ -98,19 +103,19 @@ public:
   virtual GUI_Status KeyUp( const SDL_keysym &key ) { return GUI_OK; }
   virtual GUI_Status MouseMove( const SDL_MouseMotionEvent &motion ) { return GUI_OK; }
 
-  Widget *next;
-
-protected:
   void SetSize( short x, short y, unsigned short w, unsigned short h )
               { this->x = x; this->y = y; this->w = w; this->h = h; }
 
+  Widget *next;
+
+protected:
   Window *surface;
   unsigned short flags;
   short id;
   short key;			// keystroke to activate widget
 
   Font *font;
-  std::string title;
+  string title;
   WidgetHook *hook;
 
   bool clicked;
@@ -126,7 +131,32 @@ public:
   virtual GUI_Status WidgetActivated( Widget *widget, Window *win ) = 0;
 };
 
+// compoiste widgets incapsulate several widgets but look and act like
+// a single widget to the outside
+class CompositeWidget : public Widget {
+public:
+  CompositeWidget( short id, short x, short y,  unsigned short w, unsigned short h,
+    unsigned short flags, const char *title, Window *window );
+  ~CompositeWidget( void );
+
+  virtual void Draw( void );
+
+  virtual GUI_Status MouseDown( const SDL_MouseButtonEvent &button );
+  virtual GUI_Status MouseUp( const SDL_MouseButtonEvent &button );
+  virtual GUI_Status KeyDown( const SDL_keysym &key );
+  virtual GUI_Status KeyUp( const SDL_keysym &key );
+  virtual GUI_Status MouseMove( const SDL_MouseMotionEvent &motion );
+
+protected:
+  void AddWidget( Widget *w ) { components.push_back( w ); }
+  void RemoveWidget( Widget *w );
+  Widget *GetWidget( short id ) const;
+
+private:
+  vector<Widget *> components;
+};
 
 #include "window.h"
 
 #endif	/* _INCLUDE_WIDGET_H */
+
