@@ -1,7 +1,7 @@
 /*
 bi2cf - A map converter for Crimson Fields
-copyright (c) 2002 by Florian Dietrich
-              2003 by Jens Granseuer
+copyright (c) 2002      by Florian Dietrich
+              2003-2004 by Jens Granseuer
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ const unsigned char *pathdelimiter = "/";
 #endif
 
 const unsigned char *mapstrings[5] = {"unknown","BI","BIDD1","BIDD2","HL"};
-const char verstr[] = "0.3.8";
+const char verstr[] = "0.4.0";
 
 const unsigned char helpstr[] =
 "Usage: %s mapname [options]\n"
@@ -116,8 +116,8 @@ int uncompress(unsigned char **buffer, int src_size)
                 repeat_offset = srcbuf[src_count++] | ( (temp << 4) & 0x0F00 );
                 for( j=0 ; j<=repeat_num ; j++)
                 {
-                    dstbuf[dst_count++] = dstbuf[dst_count-repeat_offset];
-                    if(dst_count >= dst_size) break;
+                    dstbuf[dst_count] = dstbuf[dst_count-repeat_offset];
+                    if(++dst_count >= dst_size) break;
                 }
             }
             else dstbuf[dst_count++] = srcbuf[src_count++];
@@ -240,17 +240,17 @@ void putbuilding(FILE *myfile, tconvdata convdata, tmapdata mapdata, int xpos, i
     fprintf(myfile,"[building]\n");
 
     // write name of building
-    if( (type+1)/3 == 0 )      fprintf(myfile,"name = Headquarters\n");
-    else if( (type+1)/3 == 1 ) fprintf(myfile,"name = Factory\n");
-    else                       fprintf(myfile,"name = Depot\n");
+    if( (type+1)/3 == 0 )      fprintf(myfile,"name = 5\n"); /* HQ */
+    else if( (type+1)/3 == 1 ) fprintf(myfile,"name = 6\n"); /* Factory */
+    else                       fprintf(myfile,"name = 7\n"); /* Depot */
 
     // all buildings are workshops (can repair units)
-    fprintf(myfile,"type = Workshop\n");
+    fprintf(myfile,"type = workshop\n");
 
     // make factories factories and set units you can build in them
     if( (type+1)/3 == 1 )
     {
-        fprintf(myfile,"type = Factory\n");
+        fprintf(myfile,"type = factory\n");
         for( i=0 ; i<convdata.maxunits ; i++ )
         {
             if( (mapdata.canbuild[i] == 0) && (strlen(convdata.units[i]) > 0) ) fprintf(myfile,"factory = %s\n",convdata.units[i]);
@@ -312,7 +312,7 @@ void write_generalinfo(FILE *myfile, tmapdata mapdata, tmapinfo mapinfo, int ski
 {
     // write mission stuff
     fprintf(myfile,"[mission]\n");
-    fprintf(myfile,"title = %s\n",mapinfo.title);
+    fprintf(myfile,"name = 2\n");
     fprintf(myfile,"players = %d\n",mapinfo.players);
     fprintf(myfile,"info = 1\n");
     if( mapinfo.tileset != NULL ) fprintf(myfile,"tileset = %s\n",mapinfo.tileset);
@@ -333,20 +333,34 @@ void write_generalinfo(FILE *myfile, tmapdata mapdata, tmapinfo mapinfo, int ski
 
     // briefing for both players
     fprintf(myfile,"[player]\n");
+    fprintf(myfile,"name = 3\n");
     fprintf(myfile,"briefing = 0\n");
     fprintf(myfile,"\n");
     fprintf(myfile,"[player]\n");
+    fprintf(myfile,"name = 4\n");
     fprintf(myfile,"briefing = 0\n");
     fprintf(myfile,"\n");
 
     // messages (briefing and info);
-    fprintf(myfile,"[messages]\n");
+    fprintf(myfile,"[messages(en)]\n");
     fprintf(myfile,"Destroy all enemy units or capture enemy headquarters!\n");
     fprintf(myfile,"%%\n");
     fprintf(myfile,"%s\n",mapinfo.title);
     fprintf(myfile,"%s\n",mapinfo.info);
     fprintf(myfile,"This map was automatically created by bi2cf %s.\n",verstr);
-    fprintf(myfile,"%%%%\n");
+    fprintf(myfile,"%%\n");
+    fprintf(myfile,"%s\n",mapinfo.title);
+    fprintf(myfile,"%%\n");
+    fprintf(myfile,"Player 1\n");
+    fprintf(myfile,"%%\n");
+    fprintf(myfile,"Player 2\n");
+    fprintf(myfile,"%%\n");
+    fprintf(myfile,"Headquarters\n");
+    fprintf(myfile,"%%\n");
+    fprintf(myfile,"Factory\n");
+    fprintf(myfile,"%%\n");
+    fprintf(myfile,"Depot\n");
+    fprintf(myfile,"[/messages]\n");
     fprintf(myfile,"\n");
 }
 
