@@ -1,5 +1,5 @@
 /* Default tile set definitions for Crimson Fields
-   Copyright (C) 2000-2003 Jens Granseuer
+   Copyright (C) 2000-2004 Jens Granseuer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 /* default_tiles.c */
 
-#include "editor.h"
+#include "ccompat.h"
 #include "gamedefs.h"
 
 const unsigned short NUM_TERRAIN_TYPES = 375;
