@@ -1,5 +1,5 @@
 /* mkdatafile -- create a Crimson Fields data file
-   Copyright (C) 2000-2003 Jens Granseuer
+   Copyright (C) 2000-2004 Jens Granseuer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* takes five file names: icons BMP, small font BMP, large font BMP, and output filename
+/* takes two file names: icons BMP and output filename
 
    The data file is written in little-endian format.
 
@@ -31,23 +31,13 @@
 
 extern int save_image_data( const char *filename, int trans, SDL_RWops *out );
 
-#define FONT_CAPS_ONLY	0x01
-
-struct FontInfo {
-  unsigned char w;
-  unsigned char h;
-  unsigned char chars;
-  unsigned char flags;
-};
-
 int main( int argc, char *argv[] ) {
   int status;
   SDL_RWops *out;
-  struct FontInfo font;
  
-  if ( argc != 5 ) {
+  if ( argc != 3 ) {
     fprintf(stderr, "Invalid number of arguments\n"
-                    "Usage: %s <icons.bmp> <smallfont.bmp> <largefont.bmp> <outfile>\n",
+                    "Usage: %s <icons.bmp> <outfile>\n",
                     argv[0] );
     exit(-1);
   }
@@ -59,7 +49,7 @@ int main( int argc, char *argv[] ) {
   }
   atexit(SDL_Quit);
 
-  out = SDL_RWFromFile( argv[4], "wb" );
+  out = SDL_RWFromFile( argv[2], "wb" );
   if ( !out ) {
     fprintf(stderr, "Couldn't open output file %s\n", argv[5] );
     exit(-1);
@@ -67,20 +57,7 @@ int main( int argc, char *argv[] ) {
 
   /* icons */
   status = save_image_data( argv[1], 1, out );
-  if ( status ) goto error_exit;
 
-  /* small font */
-  font.w = 7; font.h = 12; font.chars = 90; font.flags = 0;
-  SDL_RWwrite( out, (char *)&font, sizeof(struct FontInfo), 1 );
-  status = save_image_data( argv[2], 1, out );
-  if ( status ) goto error_exit;
-
-  /* large font */
-  font.w = 12; font.h = 14; font.chars = 64; font.flags = FONT_CAPS_ONLY;
-  SDL_RWwrite( out, (char *)&font, sizeof(struct FontInfo), 1 );
-  status = save_image_data( argv[3], 1, out );
-
-error_exit:
   SDL_RWclose( out );
   return status;
 }
